/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.*;
import javax.activity.coordination.*;


/**
 * This is the implementation of the
 * {@link javax.activity.ActivityCoordinator ActivityCoordinator}
 * interface.  This class manages the registration of Action objects
 * and the distribution of Signals from SignalSets to these Actions
 * as and when required.
 * <br>
 * It also has the responsibility to manage the <i>completion</i>
 * processing for the given Activity.  Each Activity has a corresponding
 * coordinator object.
 */

final class ActivityCoordinatorImpl implements ActivityCoordinator
   {
   /**
    * The <code>LocalActivityContext</code> of this coordinator
    */
   private LocalActivityContext context  = null;

   /**
    * a table which holds the utility <code>ActionTable</code> objects
    * (one for each <code>SignalSet</code>)
    */
   private java.util.Hashtable  sigsets  = new java.util.Hashtable ();

   /**
    * a list which holds the utility <code>ActionPriority</code> objects
    * (one for each Global <code>Action</code> object registered
    */
   private java.util.Vector     glblActs = new java.util.Vector ();

   /**
    * The Status of the Activity
    * @see javax.activity.Status
    */
   private int                  status   = Status.StatusActive;

   /**
    * The name of the completion SignalSet
    */
   private String               complSSName  = null;

   /**
    * The status on the coordinator
    */
   private boolean                              hibernated = false;

   /**
    * Construct an ActivityCoordinator object from the corresponding
    * LocalActivityContext.
    * Creates the various <i>system</i> <code>SignalSets</code> that
    * may be used during the lifetime of the Activity.
    * @param ctx     the LocalActivityContext represented by this coordinator
    * @param tcm     the ThreadContextManager for ctx
    */

   public ActivityCoordinatorImpl (LocalActivityContext ctx)
      {
      Trace.entry ("ActivityCoordinatorImpl.ActivityCoordinatorImpl", ctx);
      context = ctx;

      // create system signalsets now ...
      // new Synchronization
      SubordinateSignalSet sigset = new SynchronizationSignalSet (context.getGlobalId ());
      sigsets.put (sigset.getSignalSetName (), new ActionTable (sigset));

      // new ChildBegin
      sigset = new ChildBeginSignalSet ();
      sigsets.put (sigset.getSignalSetName (), new ActionTable (sigset));

      // new ChildComplete
      // sigset = new ChildCompleteSignalSet ();
      // sigsets.put (sigset.getSignalSetName (), new ActionTable (sigset));

      Trace.exit ("ActivityCoordinatorImpl.ActivityCoordinatorImpl");
      }

   /**
    * Utility method to handle the distribution of a single <code>Signal</code>
    * to a set of <code>Actions</code> and coordinate setting the
    * corresponding <code>Outcome</code>s in the <code>SignalSet</code>
    *
    * @param signal     the Signal to be distributed
    * @param sigset     the SignalSet from which the Signal comes
    * @param actionList the list of Actions to which the Signal is sent
    *
    * @throws SystemException  propagated from Action.processsignal
    *   or if the SignalSet throws SignalSetInactiveException
    */
   private void processSignal (Signal signal, SignalSet sigset,
      java.util.Vector actionList) throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.processSignal", signal);
      int i = 0;
      while (i < actionList.size ())
         {
         // send signal NB exceptions possible
         ActionPriority ap =  (ActionPriority) actionList.elementAt (i);
         if (ap.enabled)
            {
            Outcome outcome = null;
            try {
               outcome = ap.action.processSignal (signal);
               }
            catch (ActionErrorException e)
               {
               outcome = new javax.activity.Outcome (
                  "org.omg.CosActivity.ActionError", (java.io.Serializable) null);
               }
            // catch general exception and turn into org.omg.ActionSystemException
            // with the exception contained?
            catch (Exception e)
               {
               outcome = new javax.activity.Outcome (
                  "org.omg.CosActivity.ActionSystemException", e);
               }

            // process result
            CoordinationInformation coordInfo = null;
            try {
               coordInfo = sigset.setResponse (outcome);
               }
            catch (SignalSetInactiveException e)
               {
               Trace.exception ("setResponse failed", e);
               throw new SystemException ("Received SignalSetInactiveException:");
               }

            // disable Action if SignalSet indicates
            if (coordInfo != null && !coordInfo.isActionInterested ())
               ap.enabled = false;

            // decide next action/signal
            if (coordInfo != null && coordInfo.useNextSignal ())
               break;

            }  // end if

         i++;

         }  // end while i

      Trace.exit ("ActivityCoordinatorImpl.processSignal");
      }  // end processSignal


   /**
    * Utility method to handle the distribution of a set of <code>Signal</code>s
    * to a set of <code>Actions</code>.
    * This method is used by both completeActivity and processSignalSet
    *
    * @param atable            contains the SignalSet and Actions to use
    * @param completionStatus  the <code>CompletionStatus</code> to use
    * @return                  SignalSet <code>Outcome</code>
    *
    * @throws SystemException  propagated from called methods
    *   or if the SignalSet throws SignalSetActiveException
    */
   private Outcome processSignalActions (ActionTable atable,
      int completionStatus) throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.processSignalActions", atable);
      Outcome result = null;

      // get list of Actions (ordered by priority)
      // may need to synchronize this activity?
      java.util.Vector  actionList = new java.util.Vector ();

      // add actions from atable and then global actions
      addOrderedActions (actionList, atable.actionList);
      addOrderedActions (actionList, glblActs);

      // set signalset completion status
      atable.sigset.setCompletionStatus (completionStatus, status);

      // get signal
      Signal signal = atable.sigset.getSignal ();
      while (signal != null)        // while more signals
         {
         processSignal (signal, atable.sigset, actionList);

         signal = atable.sigset.getSignal ();
         }  // end while

      try {
         result = atable.sigset.getOutcome ();
         }
      catch (SignalSetActiveException e)
         {
         throw new SystemException ("Received SignalSetActiveException");
         }

      Trace.exit ("ActivityCoordinatorImpl.processSignalActions", result);
      return result;
      }


   /**
    * Utility method to built a list of <code>ActionPriority</code>s in priority
    * order from an unsorted list
    * Objects with equal priority may be in any order
    *
    * @param inout   list of ordered <code>ActionsPriority</code> objects
    * @param in      list of unordered <code>ActionsPriority</code> objects
    *
    */
   private void addOrderedActions (java.util.Vector inout, java.util.Vector in)
      {
      Trace.entry ("ActivityCoordinatorImpl.addOrderedActions");
      for (int i = 0; i < in.size (); i++)
         {
         ActionPriority ap = (ActionPriority) in.elementAt (i);
         boolean found = false;
         for (int j = 0; j < inout.size (); j++)
            {
            ActionPriority ap2 = (ActionPriority) inout.elementAt (j);
            if (ap.priority >= ap2.priority)
               {
               inout.insertElementAt (ap, j);
               found = true;
               break;
               }

            }  // end for j
            if (!found)
               inout.add (ap);

         }  // end for i

      Trace.exit ("ActivityCoordinatorImpl.addOrderedActions");
      }  // end addOrderedActions


   /**
    * Utility method to distribute childBegin Signals
    * called from child contexts during 'begin' processing
    *
    */
   public  void  childBegun (LocalActivityContext ctx)
      {
      Trace.entry ("ActivityCoordinatorImpl.childBegun", ctx);

      java.util.Vector   beginActionList = new java.util.Vector ();
      ActionTable syncTable = (ActionTable) sigsets.get (
         "org.omg.CosActivity.ChildLifetime");
      if (syncTable == null)
         {
         // error ... must exist?
         Trace.exit ("ActivityCoordinatorImpl.childBegun",
            " Failed to find 'org.omg.CosActivity.ChildLifetime'");
         return;   // quietly exit
         }

      // add actions from atable and then global actions
      addOrderedActions (beginActionList, syncTable.actionList);
      addOrderedActions (beginActionList, glblActs);

      // set signalset completion status
      syncTable.sigset.setCompletionStatus (
         ctx.getCompletionStatus (), status);

      // set child context in SignalSet
      ((ChildBeginSignalSet) syncTable.sigset).setContext (ctx);

      // get signal
      Signal signal = syncTable.sigset.getSignal ();
      try {
         processSignal (signal, syncTable.sigset, beginActionList);
         }
      catch (SystemException e)
         {
         // just swallow ... nothing we can do
         Trace.exception ("ChildBegin processing", e);
         }

      // now extract updated completion status from sigset
      try {
         ctx.setCompletionStatus (syncTable.sigset.getCompletionStatus ());
         }
      catch (InvalidStateException e)
         {
         // shouldn't happen ... nothing we can do
         Trace.exception ("ChildBegin updating completionStatus", e);
         }
      catch (SignalSetActiveException e)
         {
         // shouldn't happen ... nothing we can do
         Trace.exception ("ChildBegin updating completionStatus", e);
         }

      Trace.exit ("ActivityCoordinatorImpl.childBegun");
      }  // end childBegun


   /**
    * Utility method to distribute childComplete Signals
    * called from completeActivity
    *
    */
   public  void  childComplete (LocalActivityContext ctx)
      {
      Trace.entry ("ActivityCoordinatorImpl.childComplete", ctx);

      java.util.Vector   beginActionList = new java.util.Vector ();
      ActionTable syncTable =
         (ActionTable) sigsets.get ("org.omg.CosActivity.ChildLifetime");
      if (syncTable == null)
         {
         // error ... must exist?
         Trace.exit ("ActivityCoordinatorImpl.childComplete",
            " Failed to find 'org.omg.CosActivity.ChildLifetime'");
         return;   // quietly exit
         }

      // add actions from atable and then global actions
      addOrderedActions (beginActionList, syncTable.actionList);
      addOrderedActions (beginActionList, glblActs);

      // no need to set signalset completion status

      // set child context in SignalSet
      ((ChildCompleteSignalSet) syncTable.sigset).setContext (ctx);

      // get signal
      Signal signal = syncTable.sigset.getSignal ();
      try {
         processSignal (signal, syncTable.sigset, beginActionList);
         }
      catch (SystemException e)
         {
         // just swallow ... nothing we can do
         Trace.exception ("ChildComplete processing", e);
         }

      Trace.exit ("ActivityCoordinatorImpl.childComplete");
      }  // end childComplete


   /**
    * Utility method to distribute destroy message to Actions.
    * Only Actions that have not been sent the 'postComplete' Signal
    * are 'destroyed'.  Called from completeActivity
    * @param   syncActions  Vector of Actions that have been sent 'postComplete'
    */
   public  void actionCleanup (java.util.Vector syncActions)
      {
      Trace.entry ("ActivityCoordinatorImpl.actionCleanup");

      // build a SET of all actions
      java.util.HashSet  allActions = new java.util.HashSet ();
      java.util.Enumeration enum   = sigsets.elements ();
      while (enum.hasMoreElements ())
         {
         ActionTable aTable = (ActionTable)  enum.nextElement ();

         // could take this opportunity to notify SignalSet of 'destroy' ????
         aTable.sigset.destroy ();

         // may want to check that aTable.sigset.getSignalSetName ()
         //  is not 'org.omg.CosActivity.Synchronization' ?

         // now extract each Action from the actionList
         java.util.Enumeration actions = aTable.actionList.elements ();
         while (actions.hasMoreElements ())
            {
            Action a = ((ActionPriority) actions.nextElement ()).action;
            allActions.add (a);
            }  // end while

         // no need to add global actions as these must be removed
         }  // end while

      // now remove any that are in syncActions ...
      enum = syncActions.elements ();
      while (enum.hasMoreElements ())
         {
         Action a = ((ActionPriority) enum.nextElement ()).action;
         allActions.remove (a);
         }  // end while

      // finally notify the Actions
      java.util.Iterator  iter = allActions.iterator ();
      while (iter.hasNext ())
         {
         Action act = (Action) iter.next ();
         act.destroy ();
         }  // end while

      // cleanup SignalSets now ... or during earlier loop ?

      Trace.exit ("ActivityCoordinatorImpl.actionCleanup");
      }  // end actionCleanup


// *****************************************************
//
// INTERFACE methods from ActivityCoordinator interface
//
// *****************************************************

// TODO calls new completeActivity private method
   public Outcome completeActivity (int completionStatus) throws
      ActivityPendingException, ContextPendingException,
      NotOriginatorException, InvalidStateException,
      ActivityNotProcessedException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.completeActivity",
         new Integer (completionStatus));

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new SystemException ("Activity already completing/completed");

      // ... are we being called by LocalActivityContext?  If not, then error?
      // make context checks ... this is duplicated if we're called by context

      context.completionChecks(completionStatus);
      context.setCompletionStatus(completionStatus);

//         // check that context is active on current thread!
//         if (! context.isAssociated ())
//            {
//            status = Status.StatusActive;   // reset status
//            throw new SystemException ("Context is not active on current thread");
//            }  // end if

      status = Status.StatusCompleting;

      Outcome result = internalCompleteActivity(completionStatus);
      Trace.exit ("ActivityCoordinatorImpl.completeActivity", result);
      return result;
      }

   public Outcome processSignalSet (String signalSetName, int completionStatus)
      throws SignalSetUnknownException, ActivityNotProcessedException,
      InvalidActivityException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.processSignalSet", signalSetName);

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // parameter check
      if (completionStatus != CompletionStatus.CompletionStatusSuccess &&
          completionStatus != CompletionStatus.CompletionStatusFail &&
          completionStatus != CompletionStatus.CompletionStatusFailOnly)
          throw new SystemException ("Invalid parameter " + completionStatus);

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new InvalidActivityException ("Activity already completing/completed");

      // don't allow synch/child signalsets? (OMG spec)
      //  (when superior calls us with these it will be via CosActivity
      //   interface, and so a different method)
      if (signalSetName != null && (
         signalSetName.equals ("org.omg.CosActivity.ChildLifetime") ||
         signalSetName.equals ("org.omg.CosActivity.Synchronization") ))
         throw new SystemException (
            "System SignalSets are not allowed: " + signalSetName);


      Outcome result = null;

      // if we don't already have the signalset, then there are no actions
      // and nothing to do?  .. but may want to check for SignalSetUnknown !
      ActionTable atable = (ActionTable) sigsets.get (signalSetName);
      if (atable == null)
         {
         SubordinateSignalSet sigset = context.getSignalSet (signalSetName);
         if (sigset != null)  // could throw SystemException for this????
            {
            atable        = new ActionTable (sigset);
            sigsets.put (signalSetName, atable);
            }  // end if
         else
            throw new SignalSetUnknownException (/*no supported*/);

         }  // end if


      if (atable != null)
         {
         // check not already processing this signalset !
         // should be synchronized for thread safety
         synchronized (atable)
           {
           if (atable.processing)
              throw new InvalidActivityException ("SignalSet already in use");
           atable.processing = true;
           }

         try {
            result = processSignalActions (atable, completionStatus);
            }
         finally
            {
            // reset in-use flag
            atable.processing = false;
            }

         }  // end if

      Trace.exit ("ActivityCoordinatorImpl.processSignalSet", result);
      return result;
      }

   public void addAction (Action action, String signalSetName, int priority)
      throws SignalSetUnknownException, SystemException, IllegalStateException
      {
      Trace.entry ("ActivityCoordinatorImpl.addAction", signalSetName);

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // parameter check
      if (priority < 0)
         throw new SystemException ("negative priority not allowed");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new IllegalStateException (
            "Activity already completing/completed");

      ActionTable   actTbl = (ActionTable) sigsets.get (signalSetName);

      // check whether we have signal set
      if (actTbl == null)
         {
         // if not get it/throw exception & add to hashtable
         SubordinateSignalSet sigset = context.getSignalSet (signalSetName);
         if (sigset == null)
            throw new SignalSetUnknownException (/*not supported*/);

         actTbl           = new ActionTable (sigset);
         sigsets.put (signalSetName, actTbl);
         }

      // add Action + priority to ActionList (no check for dup's)
      actTbl.actionList.add (new ActionPriority (action, priority));

      Trace.exit ("ActivityCoordinatorImpl.addAction");
      }

   public void addGlobalAction (Action action, int priority)
      throws SystemException, IllegalStateException
      {
      Trace.entry ("ActivityCoordinatorImpl.addGlobalAction",
         new Integer (priority));

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // parameter check
      if (priority < 0)
         throw new SystemException ("negative priority not allowed");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new IllegalStateException (
            "Activity already completing/completed");

      // just add to global vector of ActionPriority's
      glblActs.add (new ActionPriority (action, priority));
      Trace.exit ("ActivityCoordinatorImpl.addGlobalAction");
      }

   public void removeAction (Action action) throws
      ActionNotFoundException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.removeAction", action);

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new SystemException ("Activity already completing/completed");

      boolean found = false;

      // iterate over all ActionTables in sigsets
      java.util.Enumeration enum = sigsets.elements ();
      while (enum.hasMoreElements ())
         {
         java.util.Vector actions =
            ((ActionTable) enum.nextElement ()).actionList;

         // for each element in Vector actionList
         for (int i = 0; i < actions.size (); /*iterate in loop body*/)
            {
            ActionPriority  ap = (ActionPriority) actions.elementAt (i);
            if (ap.action == action)
               {
               actions.remove (i);
               // could now send 'destroy' message to action?
               // ... but could be in many sigsets ... so keep a list?
               found = true;
               }
            else
               i++;

            }  // end for

         }  // end while

      if (! found)
         throw new ActionNotFoundException ();

      Trace.exit ("ActivityCoordinatorImpl.removeAction");
      }

   public void removeAction (Action action, String signalSetName) throws
      ActionNotFoundException, SystemException, IllegalStateException
      {
      Trace.entry ("ActivityCoordinatorImpl.removeAction", action);

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new IllegalStateException (
            "Activity already completing/completed");

      java.util.Hashtable ht = sigsets;
      if (signalSetName != null && !signalSetName.equals (""))
         {
         ht = new java.util.Hashtable ();
         ActionTable  actTbl = (ActionTable) sigsets.get (signalSetName);
         if (actTbl != null)
            ht.put (signalSetName, actTbl);
         }  // end if

      boolean found = false;

      // iterate over all ActionTables in ht (only 1 or 0 if sigsetName != null)
      java.util.Enumeration enum = ht.elements ();
      while (enum.hasMoreElements ())
         {
         java.util.Vector actions =
            ((ActionTable) enum.nextElement ()).actionList;

         // for each element in Vector actionList
         for (int i = 0; i < actions.size (); /*iterate in loop body*/)
            {
            ActionPriority  ap = (ActionPriority) actions.elementAt (i);
            if (ap.action == action)
               {
               actions.remove (i);
               // could now send 'destroy' message to action?
               // ... but could be in many sigsets ... so keep a list?
               found = true;
               break;      // ONLY REMOVE ONE
               }
            else
               i++;

            }  // end for

         }  // end while

      if (! found)
         throw new ActionNotFoundException ();

      Trace.exit ("ActivityCoordinatorImpl.removeAction");
      }

   public void removeGlobalAction(Action action) throws
      ActionNotFoundException, SystemException, IllegalStateException
      {
      Trace.entry ("ActivityCoordinatorImpl.removeGlobalAction", action);

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new IllegalStateException (
            "Activity already completing/completed");

      boolean found = false;

      // for each element in Vector glblActs
      for (int i = 0; i < glblActs.size (); /*iterate in loop body*/)
         {
         ActionPriority  ap = (ActionPriority) glblActs.elementAt (i);
         if (ap.action == action)
            {
            glblActs.remove (i);
            // could now send 'destroy' message to action?
            // ... but could be in many sigsets ... so keep a list?
            found = true;
            break;      // ONLY REMOVE ONE
            }
          else
            i++;

         }  // end for

      if (! found)
         throw new ActionNotFoundException ();

      Trace.exit ("ActivityCoordinatorImpl.removeGlobalAction");
      }

   public int getNumberRegisteredActions(String signalSetName) throws
      SignalSetUnknownException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getNumberRegisteredActions",
         signalSetName);

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      int result = 0;
      ActionTable   actTbl = (ActionTable) sigsets.get (signalSetName);
      if (actTbl == null)
         {
         // try and get signalset from context/servicemgr?
         if (context.getSignalSet (signalSetName) == null)  // throws excp if unknown
            throw new SignalSetUnknownException (/*not supported*/);

         // or just return 0 (or glblActs.size() ) ?
         result = glblActs.size();
         }
      else
         {
         result =  actTbl.actionList.size () + glblActs.size ();
         }

      Trace.exit ("ActivityCoordinatorImpl.getNumberRegisteredActions",
         new Integer (result));
      return result;
      }

   public ActivityCoordinator getParent() throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getParent");

          if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      ActivityCoordinator  coord = null;
      // use context to locate parent coord
      LocalThreadContext ltc = context.getParentContext ();
      if (ltc != null && (ltc instanceof LocalActivityContext))
         {
         coord = ((LocalActivityContext) ltc).getCoordinator ();
         }

      Trace.exit ("ActivityCoordinatorImpl.getParent", coord);
      return coord;
      }

   public GlobalId getGlobalId() throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getGlobalId");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      GlobalId result = context.getGlobalId ();
      Trace.exit ("ActivityCoordinatorImpl.getGlobalId",result);
      return result;
      }

   public int getStatus() throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getStatus");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

          Trace.exit ("ActivityCoordinatorImpl.getStatus", new Integer (status));
      return status;
      }

   public int getParentStatus() throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getParentStatus");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      int result = Status.StatusUnknown;
      ActivityCoordinator parent = getParent ();
      if (parent == null)
         result = status;
      else
         result = parent.getStatus ();

      Trace.exit ("ActivityCoordinatorImpl.getParentStatus",
         new Integer (result));
      return result;
      }

   public String getName() throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getName");

          if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      String result = context.toString ();
      Trace.exit ("ActivityCoordinatorImpl.getName", result);
      return result;
      }

   public boolean isSameActivity(ActivityCoordinator coord)
      throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.isSameActivity");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      boolean result = (this == coord);
      Trace.exit ("ActivityCoordinatorImpl.isSameActivity",
         new Boolean (result));
      // check GlobalIds ????
      return result;
      }

   public RecreateData setPersistent () throws SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.setPersistent");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");
      context.setPersistent ();

      Trace.exit ("ActivityCoordinatorImpl.setPersistent");
      return null;   // we have no recovery info
      }

   public void setCompletionSignalSetName(String css)
      throws SignalSetUnknownException, IllegalStateException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.setCompletionSignalSetName");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // check state ...
      if (status != Status.StatusActive)
         throw new IllegalStateException (
              "Activity already completing/completed");

      // check whether this is valid?
      if (css == null || context.getSignalSet (css) == null)
         throw new SignalSetUnknownException(/*"null name"*/);

      complSSName = css;

      Trace.exit ("ActivityCoordinatorImpl.setCompletionSignalSetName");
      }

   public String getCompletionSignalSetName()
      {
      Trace.entry ("ActivityCoordinatorImpl.getCompletionSignalSetName");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      Trace.exit ("ActivityCoordinatorImpl.getCompletionSignalSetName");
      return complSSName;
      }

   /**
    * Return list of actions for given signalset.
    * List may contain duplicates
    */
   public Action[] getActions(String ssName) throws SignalSetUnknownException, SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.getActions");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      Action[]         result = null;
      java.util.Vector list   = glblActs;   // assume global actions
      if (ssName != null)                   // specific signalset
         {

         ActionTable   actTbl = (ActionTable) sigsets.get (ssName);
         if (actTbl == null)
            {
            // check that ssName is supported
            if (context.getSignalSet (ssName) == null)
               throw new SignalSetUnknownException(/*"invalid name"*/);

            list = null;
            }
         else
            list = actTbl.actionList;
         }

      result = new Action[(list == null) ? 0 : list.size()];
      if (list != null)
         for (int i = 0; i < list.size(); i++)
            result[i] = (Action)
               ((ActionPriority)list.elementAt (i)).action;

      Trace.exit ("ActivityCoordinatorImpl.getActions");
      return result;
      }

   /*
    * Method to perform completion of Activity.  Called by both completeActivity and
    * heuristicComplete after making completion checks and setting the status appropriately.
    */
   private Outcome internalCompleteActivity(int completionStatus) throws
      SystemException
      {
      Trace.entry ("ActivityCoordinatorImpl.internalCompleteActivity",
         new Integer (completionStatus));

      Outcome result = null;

      //   ... preCompletion distribution
      java.util.Vector   syncActionList = new java.util.Vector ();
      ActionTable syncTable =
         (ActionTable) sigsets.get ("org.omg.CosActivity.Synchronization");
      if (syncTable == null)
         {
         // error ... must exist?
         status = Status.StatusActive;   // reset status
         SystemException e =
                new SystemException ("could not find Synchronization sigset");
         Trace.exception ("ActivityCoordinatorImpl.completeActivity", e);
         throw e;
         }


      // add actions from atable and then global actions
      addOrderedActions (syncActionList, syncTable.actionList);
      addOrderedActions (syncActionList, glblActs);

      // set signalset completion status
      syncTable.sigset.setCompletionStatus (completionStatus, status);

      // only drive pre-completion if completionStatus is success
      if (completionStatus == CompletionStatus.CompletionStatusSuccess)
         {
         // get signal
         Signal signal = syncTable.sigset.getSignal ();
         processSignal (signal, syncTable.sigset, syncActionList);

         // update completionStatus to reflect pre completion
         try {
            completionStatus =  syncTable.sigset.getCompletionStatus ();
            context.setCompletionStatus (completionStatus);
            }
         catch (SignalSetActiveException e)
            {
            Trace.exception ("ActivityCoordinatorImpl.completeActivity", e);
            status = Status.StatusActive;   // reset status
            throw new SystemException ("Failed to get preCompletion status");
            }
         catch (InvalidStateException e)
            {
            Trace.exception ("ActivityCoordinatorImpl.completeActivity", e);
            status = Status.StatusActive;   // reset status
            throw new SystemException (
               "preCompletion returned invalid CompletionStatus");
            }

         }  // end if

      //   ... get completion signal set (via context) ... may be null!
      String sigsetName = complSSName;
      if (sigsetName == null)
         sigsetName = context.getCompletionSignalSetName ();

      if (sigsetName != null)
         {
         ActionTable compTable = (ActionTable) sigsets.get (sigsetName);
         // if we don't already have this SignalSet then get it now!
         if (compTable == null)
                {
                // even if there are no actions registered we still need
                // the completion status of the SignalSet!
                try {
                       SubordinateSignalSet sigset = context.getSignalSet (sigsetName);
                       if (sigset != null)
                              {
                              compTable        = new ActionTable (sigset);
                              sigsets.put (sigsetName, compTable);
                              }  // end if

                        }
                 catch (SignalSetUnknownException e)
                        {
                        // could throw a SystemException here, but could equally
                        // just swallow and treat as if no completion sigset
                        Trace.exception ("ActivityCoordinatorImpl.completeActivity", e);
                        }

                }  // end if

         if (compTable != null)  // then there is something to do!
                {
                // ... completion distribution / store result
                result = processSignalActions (compTable, completionStatus);

                // get latest completion status
                try {
                        completionStatus =  compTable.sigset.getCompletionStatus ();
                        // no need to update context as it will never now be used
                       }
                catch (SignalSetActiveException e)
                       {
                       Trace.exception ("ActivityCoordinatorImpl.completeActivity", e);
                       status = Status.StatusActive;   // reset status
                       throw new SystemException (
                              "Failed to get CompletionStatus from completion SignalSet");
                       }
                }  // end if

         }  // end if

      //   ... childCompleted distribution (parent coord)
      // ActivityCoordinator parent = getParent ();
      // if (parent != null)
      //    ( (ActivityCoordinatorImpl) parent).childComplete (context);

      //   ... context.completed ()
      context.completed ();

      //   ... postCompletion distribution
      // set signalset finalOutcome
      syncTable.sigset.setSignal (new Signal ("", "",
         new ActivityInformation (null, completionStatus, result)));

      // get signal ... this is second call, so should be postCompletion
      Signal signal = syncTable.sigset.getSignal ();
      processSignal (signal, syncTable.sigset, syncActionList);

      // no need to update completionStatus or get outcome

      status = Status.StatusCompleted;

      //   ... cleanup (destroy on unused Actions)
      //       need to send destroy to any actions NOT sent 'postCompletion'
      actionCleanup (syncActionList);

      Trace.exit ("ActivityCoordinatorImpl.internalCompleteActivity", result);
      return result;
      }


   public Outcome heuristicComplete(int completionStatus) throws
      ActivityPendingException, ContextPendingException, InvalidStateException,
      ActivityNotProcessedException, SystemException
      {
      Trace.entry("ActivityCoordinatorImpl.heuristicComplete", new Integer(completionStatus));

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
         throw new SystemException ("Activity already completing/completed");

      // ... are we being called by LocalActivityContext?  If not, then error?
      // make context checks ... this is duplicated if we're called by context
      try
         {
         context.completionChecks(completionStatus);
         context.setCompletionStatus(completionStatus);
         }
      catch (NotOriginatorException ne)
         {
         // just swallow and carry on
         }

//         // check that context is active on current thread!
//         if (! context.isAssociated ())
//            {
//            status = Status.StatusActive;   // reset status
//            throw new SystemException ("Context is not active on current thread");
//            }  // end if

      status = Status.StatusCompletingHeuristic;
      Outcome result = internalCompleteActivity(completionStatus);

      Trace.exit("ActivityCoordinatorImpl.heuristicComplete", result);
      return result;
      }


   public RecreateData hibernate() throws SystemException, ActivityPendingException, ContextPendingException
      {
      Trace.entry ("ActivityCoordinatorImpl.hibernate");

      if(hibernated) throw new IllegalStateException("Activity has been hibernated");

      // ensure not already completed/completing
      if (status != Status.StatusActive)
          throw new IllegalStateException ("Activity already completing/completed");

      // GlobalId result = context.getGlobalId ();
      context.hibernate();

      // Now drop references so GC can happen early on those objects
      context = null;
      sigsets = null;
      glblActs = null;

      hibernated = true;

      Trace.exit ("ActivityCoordinatorImpl.hibernate");
      return null;
      }


   }  // end class ActivityCoordinatorIml

/**
 * Utility class used to store registered <code>Action</code>s with the
 * appropriate priority
 * <br>
 * The fields are public so it can be used like a 'C' struct
 */

final class ActionPriority
   {
   public  Action    action = null;
   public  int       priority = 0;
   public  boolean   enabled  = true;

   public ActionPriority (Action act, int pri)
      {
      action = act; priority = pri;
      }

   }  // end ActionPriority


/**
 * Utility class used to store a list of <code>ActionPriority</code> objects
 * with the <code>SignalSet</code> that they are registered for.
 * There is no ordering to the list other than the order in which Actions
 * are added.
 * <br>
 * The fields are public so it can be used like a 'C' struct
 */

final class ActionTable
   {
   public   SubordinateSignalSet          sigset = null;
   public   boolean                   processing = false;
   /**
    * list of ActionPriority objects
    */
   public   java.util.Vector   actionList = null;

   public   ActionTable (SubordinateSignalSet  sig)
      {
      sigset = sig;
      actionList = new java.util.Vector ();
      }

   }  // end ActionTable
