/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.*;

/**
 * The ActivityManager implementation is essentially a <i>facade</i>
 * design pattern.  The real work of the various methods is done by
 * the <code>LocalThreadContext</code> object which actually represents
 * a specific Activity context.
 * <br>
 * The <i>current</i> context is located through the use of the
 * <code>ThreadContextManager</code> object which is available from
 * our super class once a service has registered.
 */

public class ActivityManagerImpl
   extends UserActivityImpl implements ActivityManager
   {
   // IMPLEMENTATION specific methods
   /**
    *  Constructor
    *  The test case code will need access this object/class via some
    *  form of lookup, so make c'tor public to allow instantiation
    */
    public ActivityManagerImpl ()
       {
       Trace.entry ("ActivityManagerImpl.ActivityManagerImpl");
       Trace.exit ("ActivityManagerImpl.ActivityManagerImpl");
       }

   /**
    *  instance
    *  We give the impression that this is a singleton class, so provide a
    *  method to give access
    */
    static public ActivityManager  getInstance ()
       {
       Trace.entry ("ActivityManagerImpl.instance");
       Trace.exit ("ActivityManagerImpl.instance");

       // we're not really a singleton ...
       return new ActivityManagerImpl ();
       }




// *****************************************************
//
// INTERFACE methods from ActivityManager interface
//
// *****************************************************

   public ActivityToken suspend () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("ActivityManagerImpl.suspend");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      ActivityToken result = null;

      if (this.threadContextManager != null) // ie we have a service registered
         {
         LocalThreadContext  context =
            this.threadContextManager.getCurrentContext (
            this.serviceManager.getServiceInformation ().getServiceName ());

         if (context != null)
            result = context.suspend ();
         }  // end if

      Trace.exit ("ActivityManagerImpl.suspend", result);
      return result;
      }


   public void resume (ActivityToken activity) throws InvalidActivityException,
      ServiceNotRegisteredException, InvalidParentContextException,
      SystemException
      {
      Trace.entry ("ActivityManagerImpl.resume", activity);
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      if (activity != null)
         {
         if (! (activity instanceof ActivityTokenImpl) )
            throw new InvalidActivityException ("invalid token");

         LocalThreadContext  context =
            ((ActivityTokenImpl) activity).getContext ();

         if (context != null)
            {
            // if (! (context instanceof LocalActivityContext))
            //    throw new InvalidActivityException ("invalid context");

            context.resume ((ActivityTokenImpl) activity);
            }  // end if

         }  // end if

      Trace.exit ("ActivityManagerImpl.resume");
      }


   public ActivityToken suspendGroup () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("ActivityManagerImpl.suspendGroup");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      ActivityToken result = null;

      if (this.threadContextManager != null) // ie we have a service registered
         {
         LocalThreadContext  context =
            this.threadContextManager.getTopContext ();

         if (context != null)
            result = new GroupToken (context.suspend ());

         }  // end if

      Trace.exit ("ActivityManagerImpl.suspendGroup", result);
      return result;
      }


   public void resumeGroup (ActivityToken activity) throws
      ServiceNotRegisteredException, InvalidActivityException,
      InvalidParentContextException, SystemException
      {
      Trace.entry ("ActivityManagerImpl.resumeGroup", activity);
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      if (activity != null)
         {
         if (! (activity instanceof GroupToken) )
            throw new InvalidActivityException ("invalid token");

         ActivityTokenImpl  token = ((GroupToken) activity).getToken ();
         if (token != null)
            {
            LocalThreadContext  context = token.getContext ();

            if (context != null)
               {
               // check parent context is null !
               if (context.getParentContext () != null) // token not suspGroup
                  throw new InvalidActivityException (
                     "token not from suspendGroup");

               // or ... check no current context?
               //    ... this is effectively done by context.resume which
               //    ... ensures that parent context == current (ie null)

               context.resume (token);
               }  // end if context

            }  // end if token

         }  // end if activity

      Trace.exit ("ActivityManagerImpl.resumeGroup");
      }


   public ActivityToken suspendAll () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("ActivityManagerImpl.suspendAll");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      ActivityToken result = null;

      if (this.threadContextManager != null)
         {
         ActivityTokenImpl token = ThreadContextManager.suspendAllContexts ();
         if (token != null)
            result = new AllToken (token);
         }  // end if

      Trace.exit ("ActivityManagerImpl.suspendAll", result);
      return result;
      }


   public void resumeAll (ActivityToken activity) throws
      ServiceNotRegisteredException, InvalidActivityException,
      InvalidParentContextException, SystemException
      {
      Trace.entry ("ActivityManagerImpl.resumeAll", activity);
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      if (activity != null)
         {
         if (! (activity instanceof AllToken) )
            throw new InvalidActivityException ("invalid token");

         ActivityTokenImpl token = ((AllToken) activity).getToken ();

         if (token != null)
            {
            LocalThreadContext  context = token.getContext ();

            if (context != null)
               {
               // check parent context is null !
               if (context.getParentContext () != null) // token not suspAll
                  throw new InvalidActivityException (
                     "token not from suspendAll");

               // or ... check no current context?
               //    ... this is effectively done by context.resume which
               //    ... ensures that parent context == current (ie null)

               // check for ANY existing context !
               // must have a threadContextManager!
               if (this.threadContextManager == null)
                  throw new SystemException (
                     "ResumeAll -- No Service Registered");

               if (ThreadContextManager.anyCurrentContext ())
                  throw new InvalidParentContextException ();

               context.resume (token);
               }  // end if context

            }  // end if token

         }  // end if activity

      Trace.exit ("ActivityManagerImpl.resumeAll");
      }

/**
 * Private class which implements the ActivityToken interface.
 * This wrappers the internally used ActivityTokenImpl object
 * created by suspend operations and is used to differentiate the
 * tokens created by suspendGroup.
 */

   private class GroupToken implements ActivityToken
      {
      private ActivityTokenImpl    _token = null;

      public GroupToken (ActivityTokenImpl token)
         {
         Trace.entry ("ActivityManagerImpl.GroupToken.GroupToken");
         _token = token;
         Trace.exit ("ActivityManagerImpl.GroupToken.GroupToken");
         }  // end c'tor

      public ActivityTokenImpl getToken ()
         {
         Trace.entry ("ActivityManagerImpl.GroupToken.getToken");
         Trace.exit ("ActivityManagerImpl.GroupToken.getToken");
         return _token;
         }  // end getToken

      }  // end inner class GroupToken

/**
 * Private class which implements the ActivityToken interface.
 * This wrappers the internally used ActivityTokenImpl object
 * created by suspend operations and is used to differentiate the
 * tokens created by suspendAll.
 */

   private class AllToken implements ActivityToken
      {
      private ActivityTokenImpl    _token = null;

      public AllToken (ActivityTokenImpl token)
         {
         Trace.entry ("ActivityManagerImpl.AllToken.AllToken");
         _token = token;
         Trace.exit ("ActivityManagerImpl.AllToken.AllToken");
         }  // end c'tor

      public ActivityTokenImpl getToken ()
         {
         Trace.entry ("ActivityManagerImpl.AllToken.getToken");
         Trace.exit ("ActivityManagerImpl.AllToken.getToken");
         return _token;
         }  // end getToken

      }  // end inner class AllToken

   }  // end class ActivityManagerImpl
