/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.ActivityToken;

/**
 * The <code>ActivityTokenImpl</code> class is used by <code>suspend()</code>
 * and <code>resume()</code> to enable an Activity to be suspended
 * on one thread and resumed later on the same thread or on a different thread,
 * whilst preserving the nested scoping of other Activities
 *
*/

final class ActivityTokenImpl
   implements javax.activity.ActivityToken
   {
   /**
    * The <code>LocalThreadContext</code> of the represented context.
    */
   private LocalThreadContext context = null;

   /**
    * A reference to the <code>ActivityTokenImpl</code> of a suspended child.
    */
   private ActivityTokenImpl  childToken = null;


   /**
    * Initilise from context and child <code>ActivityToken</code>
    *
    * @param context     the context to be represented by this token
    * @param childToken  the child <code>ActivityTokenImpl</code>
    *
    */
   public ActivityTokenImpl (LocalThreadContext context,
      ActivityTokenImpl childToken)
      {
      Trace.entry ("ActivityTokenImpl.ActivityTokenImpl", context);
      this.context    = context;
      this.childToken = childToken;
      Trace.exit ("ActivityTokenImpl.ActivityTokenImpl");
      }  // end c'tor


   /**
    * Getter for the suspended context.
    *
    * @return  the suspended context
    *
    */
   public LocalThreadContext getContext ()
      {
      Trace.entry ("ActivityTokenImpl.getContext");
      Trace.exit ("ActivityTokenImpl.getContext", this.context);
      return this.context;
      }  // end getContext


   /**
    * Getter for the child token.
    *
    * @return  <code>ActivityTokenImpl</code> representing
    *     suspended child context
    *
    */
   public ActivityTokenImpl getChildToken ()
      {
      Trace.entry ("ActivityTokenImpl.getChildToken");
      Trace.exit ("ActivityTokenImpl.getChildToken", this.childToken);
      return this.childToken;
      }  // end getChildToken


   /**
    * Setter for the child token.
    *
    * @param  <code>ActivityTokenImpl</code> representing child context
    *
    */
   public void setChildToken (ActivityTokenImpl childToken)
      {
      Trace.entry ("ActivityTokenImpl.getChildToken", childToken );

      this.childToken = childToken;

      Trace.exit ("ActivityTokenImpl.getChildToken");
      }  // end setChildToken


   /**
    * Links child ActivityTokenImpl to the end of this ActivityTokenImpl chain
    *
    */
   public void link (ActivityTokenImpl childToken)
      {
      Trace.entry ("ActivityTokenImpl.link", childToken);

      // Find the ActivityTokenImpl at the end of this token chain

      ActivityTokenImpl child = this.childToken;
      ActivityTokenImpl parent = this;

      while (child != null)
         {
         parent = child;
         child = parent.getChildToken ();
         }  // end while

      // Add the input token as the new child at the end of the chain
      parent.setChildToken (childToken);

      Trace.exit ("ActivityTokenImpl.link");
      }  // end link

   } // class ActivityTokenImpl
