/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.*;
import javax.activity.coordination.*;


/**
  * The ChildBeginSignalSet class is the implementation of the
  * org.omg.CosActivity.ChildLifetime signal set that is one of the signal sets
  * that must be supplied by a CORBA or J2EE compliant Activity
  * Service.
  *
  * The ChildBegin signal set is driven on the parent coordinator
  * when a child activity is begun. This occurs after the property
  * groups have been created, once the new activity is on the thread.
  *
  * The ChildBeginSignalSet produces the following signal:
  * childBegin - the recipient (an Action registered with the parent
  *              coordinator) is informed that the child activity has
  *              begun. The child activity's completion status and its
  *              identity are encoded within the Signal via the
  *              ActivityInformation class. This signal will be sent after
  *              the child activity context has been attached to the thread.
  */
final class ChildBeginSignalSet implements
   javax.activity.coordination.SubordinateSignalSet
   {
   private LocalActivityContext  _context = null;
   private String      _signalSetName     = "org.omg.CosActivity.ChildLifetime";
   private String         _signalName     = "childBegin";
   private int      _completionStatus     = 0;

   public ChildBeginSignalSet ()
      {
      Trace.entry ("ChildBeginSignalSet");
      Trace.exit ("ChildBeginSignalSet");
      } // end ChildBeginSignalSet c'tor

   public void   setContext (LocalActivityContext  ctx)
      {
      Trace.entry ("ChildBeginSignalSet.setContext", ctx);
      _context = ctx;
      Trace.exit ("ChildBeginSignalSet.setContext");
      }

   public String getSignalSetName ()
      {
      Trace.entry ("ChildBeginSignalSet.getSignalSetName");
      Trace.exit ("ChildBeginSignalSet.getSignalSetName", _signalSetName);

      return _signalSetName;
      }  // end ChildBeginSignalSet.getSignalSetName

 /**
   * Signals produced by a supplied signal set must create an
   * {@link com.ibm.ws.javax.activity.ActivityInformation ActivityInformation}
   * object, containing the activity's {@link com.ibm.ws.javax.activity.GlobalId GlobalId}
   * and {@link com.ibm.ws.javax.activity.CompletionStatus CompletionStatus}.
   */
   public Signal getSignal ()
      {
      Trace.entry ("ChildBeginSignalSet.getSignal");

      int      compStatus = CompletionStatus.CompletionStatusFail;
      GlobalId   globalId = null;
      if (_context != null)
         {
         // get global id
         globalId   = _context.getGlobalId ();
         compStatus = _context.getCompletionStatus ();
         }  // end if

      Signal sig = new Signal (_signalName, _signalSetName,
         new ActivityInformation (globalId, compStatus));

      Trace.exit ("ChildBeginSignalSet.getSignal", sig);
      return sig;
      }  // end ChildBeginSignalSet.getSignal


   public void setSignal (Signal signal)
      {
      Trace.entry ("ChildBeginSignalSet.setSignal");
      Trace.exit ("ChildBeginSignalSet.setSignal");
      // no implementation necessary
      }  // end ChildBeginSignalSet.setSignal

   public CoordinationInformation setResponse (Outcome response) throws
      SignalSetInactiveException
      {
      Trace.entry ("ChildBeginSignalSet.setResponse", response);

      CoordinationInformation result = null;

      // deal with ActionError response
      if (response != null &&
         response.getName().equals ("org.omg.CosActivity.ActionError"))
         {
         _completionStatus = CompletionStatus.CompletionStatusFailOnly;
         result = new CoordinationInformation (true, true);
         }

      Trace.exit ("ChildBeginSignalSet.setResponse", result);
      return result;
      }  // end ChildBeginSignalSet.setResponse

   public Outcome getOutcome () throws SignalSetActiveException
      {
      Trace.entry ("ChildBeginSignalSet.getOutcome");
      // no implementation necessary
      Trace.exit ("ChildBeginSignalSet.getOutcome");
      return null;
      }  // end ChildBeginSignalSet.getOutcome

   public Outcome getCurrentOutcome ()
      {
      Trace.entry ("ChildBeginSignalSet.getCurrentOutcome");
      // no implementation necessary
      Trace.exit ("ChildBeginSignalSet.getCurrentOutcome");
      return null;
      }  // end ChildBeginSignalSet.getCurrentOutcome

   public void setCompletionStatus (int completionStatus, int status)
      {
      Trace.entry ("ChildBeginSignalSet.setCompletionStatus",
         new Integer (completionStatus));
      // no implementation necessary
      _completionStatus = completionStatus;
      Trace.exit ("ChildBeginSignalSet.setCompletionStatus");
      }  // end ChildBeginSignalSet.setCompletionStatus

   public int getCompletionStatus () throws SignalSetActiveException
      {
      Trace.entry ("ChildBeginSignalSet.getCompletionStatus");
      Trace.exit ("ChildBeginSignalSet.getCompletionStatus",
         new Integer (_completionStatus));
      return _completionStatus;
      }  // end ChildBeginSignalSet.getCompletionStatus

   public void destroy ()
      {
      Trace.entry ("ChildBeginSignalSet.destroy");
      Trace.exit ("ChildBeginSignalSet.destroy");
      }  // end ChildBeginSignalSet.destroy

   public void setActivityCoordinator (ActivityCoordinator coord)
      throws SignalSetActiveException
      {
      Trace.entry ("ChildBeginSignalSet.setActivityCoordinator");
      Trace.exit ("ChildBeginSignalSet.setActivityCoordinator");
      }  // end ChildBeginSignalSet.setActivityCoordinator

   }  // end class ChildBeginSignalSet
