/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.propertygroup.*;
import javax.activity.coordination.*;
import javax.activity.*;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Vector;


/**
 * Represents an activity.  Most of the logic in UserActivityImpl delegates
 * to this class.
 *
 */
final class LocalActivityContext extends LocalThreadContext
   {
   /**
    * The <code>GlobalId</code> for this activity
    */
   private GlobalId globalId = null;

   /**
    * The completion status.
    */
   private int completionStatus =
      CompletionStatus.CompletionStatusFail;

   /**
    * The <code>PropertyGroup</code>s associated with this activity.
    */
   private Hashtable propertyGroups = new Hashtable();

   /**
    * The <code>PropertyGroupManager</code>s to be used to create
    * <code>PropertyGroup</code>s for this activity and any of its children.
    */
   private Hashtable propertyGroupManagers = null;

   /**
    * The registered service that created this context
    */
   protected ServiceManager   serviceManager = null;

   /**
    * The ActivityCoordinator for this context
    */
   protected ActivityCoordinatorImpl  activityCoordinator = null;

   /**
    * Indicator that this Activity has been completed and is no longer valid
    */
   private boolean completed = false;


   /**
    * Table of persistent/hibernated activities (keyed on GlobalId)
    */
   private static HashMap persistentActivities = new HashMap();


   /**
    * Initialise object and create coordinator
    *
    * @param propertyGroupManagers  the <code>PropertyGroupManager</code>s
    *     to be used by this Activity
    * @param mgr                    the ServiceManager that is creating
    *     this Activity
    * @param tcm                    the (shared) ThreadContextManager
    *     representing Activities in the contextGroup of mgr
    *
    */
   public LocalActivityContext (Hashtable propertyGroupManagers,
      ServiceManager  mgr, ThreadContextManager  tcm)
      {
        this(propertyGroupManagers, mgr, tcm, new GlobalIdImpl());
      }  // end c'tor

   /*
    * Alternative constructor, used when globalId supplied as for recreate.
    */
   public LocalActivityContext (Hashtable propertyGroupManagers,
      ServiceManager  mgr, ThreadContextManager  tcm, GlobalId gid)
      {
      super (mgr.getServiceInformation ().getServiceName (), tcm);
      Trace.entry ("LocalActivityContext.LocalActivityContext", mgr);

      // Copy the PropertyGroupManagers so any further changes to the registered
      // PropertyGroupManagers are not reflected here. ???? is this necessary?
      this.propertyGroupManagers = (Hashtable) propertyGroupManagers.clone ();

      this.serviceManager = mgr;
      // Set the GlobalId (before creating the coordinator)
      if(gid == null) gid = new GlobalIdImpl ();
      this.globalId = gid;
      this.activityCoordinator = new ActivityCoordinatorImpl (this);

      Trace.exit ("LocalActivityContext.LocalActivityContext");
      }  // end c'tor

   /*
    * Begin this Activity, but do not create any thread associations unless the
    * resume flag is set to true.
    * @param timeout  the timeout value for this Activity
    * @param parent  the GlobalId of the parent Activity for this Activity
    * @param resume  flag to indicate whether this Activity should be associated with the
    *                current thread or not.
    * @exception ActivityCompletedException thrown if the globalId of this Activity
    *            is not known.
    * @exception SystemException thrown for unexpected conditions.
    */
   protected void beginOffThread (RecreateData recData, GlobalId parent, boolean resume)
      throws ActivityCompletedException, SystemException
      {
      Trace.entry ("LocalActivityContext.beginOffThread");

      // check that it is valid to begin this gid off thread
      if (ThreadContextManager.getLocalActivityContext (globalId) != null)
          throw new SystemException ("Activity is already active");

      // has this activity been made persistent?
      if (!persistentActivities.containsKey (globalId))
          throw new ActivityCompletedException ("Activity is not persistent/hibernated");

      if (persistentActivities.get (globalId) != null)
          throw new SystemException ("Activity is already recreated");

      // Set the timeout
      // this.timeout = timeout;

      LocalActivityContext parentThread  = null;
      LocalActivityContext parentContext = null;
      if (parent != null)
         {
         parentThread = ThreadContextManager.getLocalActivityContext(parent);
         if (parentThread == null)
            {
            // then parent is not associated with a thread ... maybe recreated?
            parentThread = (LocalActivityContext)
               persistentActivities.get (parent);
            if (parentThread == null)
               throw new ActivityCompletedException ("Unknown Parent Activity");
            }

         // this is the parent for PropertyGroups ... must by same HLS
         parentContext = (LocalActivityContext)
            parentThread.getCurrentContext (this.serviceName);

         // ensure parent is CORRECT group
         if (parentThread.threadContextManager != this.threadContextManager)
            throw new SystemException ("Invalid Parent context - wrong ContextGroup");
         }  // end if

      // Create PropertyGroups
      Enumeration enum = propertyGroupManagers.keys ();
      while (enum.hasMoreElements ())
         {
         String               name = (String) enum.nextElement ();
         PropertyGroupManager mgr  =
            (PropertyGroupManager) propertyGroupManagers.get (name);

         // Create PropertyGroup
         PropertyGroup parentPropertyGroup = null;

         if (parentContext != null)
            {
            parentPropertyGroup = parentContext.getPropertyGroup (name);
            }  // end if

         PropertyGroup propertyGroup =
            mgr.create (parentPropertyGroup, this.globalId);

         if (propertyGroup == null)
            throw new SystemException ("null property group returned");

         propertyGroups.put (name, propertyGroup);
         }  // end while

      if (resume) // parent must be null!
         {
         addAssociation ();
         }
      else
         {
         this.setParentContext (parentThread);
         if (parentThread != null)
            {
            parentThread.addToChildren(this);
            }
         }

      // NO childBegin for recreated contexts ???
      // if (parentThread != null &&
      //    (parentThread instanceof LocalActivityContext))
      //    {
      //    ((LocalActivityContext) parentThread).
      //        activityCoordinator.childBegun (this);
      //    }  // end if

      // mark as recreated
      persistentActivities.put (globalId, this);

      Trace.exit ("LocalActivityContext.beginOffThread");
      }  // end beginOffThread

   /**
    * Begin this activity.
    *
    * @param timeout  the timeout for this activity
    *
    * @exception InvalidStateException
    *              if the current activity has been marked as failed.
    * @exception SystemException
    *              if an unexpected condition arises
    */
   protected void begin (int timeout)
      throws InvalidStateException, SystemException
      {
      Trace.entry ("LocalActivityContext.begin");

      // Set the timeout
      this.timeout = timeout;

      // Get the activity for the same service that we are to be nested in
      // (if any)
      LocalActivityContext parentThread = (LocalActivityContext)
         threadContextManager.getCurrentContext ();
      LocalActivityContext parentActivity = (LocalActivityContext)
         threadContextManager.getCurrentContext (this.serviceName);

      if (parentThread != null)
         {
         if (parentThread.completionStatus ==
            CompletionStatus.CompletionStatusFailOnly)
            {
            // The current activity has been marked as failed - throw
            // InvalidState
            throw new InvalidStateException ();
            }

         }  // end if

      // Create PropertyGroups
      Enumeration enum = propertyGroupManagers.keys ();
      while (enum.hasMoreElements ())
         {
         String               name = (String) enum.nextElement ();
         PropertyGroupManager mgr  =
            (PropertyGroupManager) propertyGroupManagers.get (name);

         // Create PropertyGroup
         PropertyGroup parentPropertyGroup = null;

         if (parentActivity != null)
            {
            parentPropertyGroup = parentActivity.getPropertyGroup (name);
            }  // end if

         PropertyGroup propertyGroup =
            mgr.create (parentPropertyGroup, this.globalId);

         if (propertyGroup == null)
            throw new SystemException ("null property group returned");

         propertyGroups.put (name, propertyGroup);
         }  // end while

      // Associate this activity with the current thread
      this.addAssociation ();

      // now notify parent so childBegin can be sent ...
      if (parentContext != null &&
         (parentContext instanceof LocalActivityContext))
         {
         ((LocalActivityContext) parentContext).
            activityCoordinator.childBegun (this);
         }  // end if

      Trace.exit ("LocalActivityContext.begin");
      }  // end begin


   /**
    * Complete this Activity.
    * This method is called from the <code>UserActivity</code> interface
    * and after making some state checks, delegates the action to the
    * contained <code>ActivityCoordinator</code> object.
    *
    * @return   the <code>Outcome</code> from the <i>completion SignalSet
    *    </i> or null if there isn't one.
    *
    * @exception ActivityPendingException
    *    if the thread from which this method is called is not the only
    *    thread on which the Activity is active. The
    *    application's response should be to try again later when any
    *    asynchronous work on other threads has been suspended.
    * @exception NotOriginatorException
    *    if a call to complete the Activity is made from a process
    *    into which the Activity was imported.  Only the Activity
    *    originator (defined as the process in which the Activity
    *    is rooted) may call complete().
    * @exception ContextPendingException
    *    the Activity contains nested active or suspended Activity contexts
    *    and the completion status of the Activity is
    *    <code>CompletionStatusSuccess</code>.  The application must
    *    either complete the outstanding nested contexts or force the
    *    Activity to end by setting the completion status to
    *    <code>CompletionStatusFail</code> and then completing the Activity
    * @exception SystemException
    *    An unexpected error condition has arisen.
    *
    * @see ActivityCoordinatorImpl.completeActivity
    */
    protected Outcome complete () throws ContextPendingException,
       NotOriginatorException, ActivityPendingException,
       ActivityNotProcessedException, SystemException
       {
       Trace.entry ("LocalActivityContext.complete");
       Outcome result = null;

       // if this is an IMPORTED context, then throw NotOriginatorException

       // completionChecks (completionStatus);    // also called by activity coord

       // check not already in completion/completed????
       try {
          result = activityCoordinator.completeActivity (completionStatus);
          }
       catch (InvalidStateException e)
          {
          // can't happen as state is already set
          }

       Trace.exit ("LocalActivityContext.complete", result);
       return result;
       }  // end complete

   /**
    * Perform various state checks to ensure that <i>completion</i>
    * can proceed.  This method is separated out from <code>complete</code>
    * so that it can also be used by the <code>ActivityCoordinator</code>
    *
    * @exception ContextPendingException
    *    see <code>complete</code> method
    * @exception NotOriginatorException
    *    see <code>complete</code> method
    * @exception ActivityPendingException
    *    see <code>complete</code> method
    *
    */
   public void completionChecks (int compStatus) throws ContextPendingException,
      NotOriginatorException, ActivityPendingException
      {
      Trace.entry ("LocalActivityContext.completionChecks",
         new Integer (compStatus));

      // for imported context, add a check and throw NotOriginatorException

      if (this.countAssociations() > 1)
         {
         // The calling thread is not the only thread on which the activity
         // is active - throw ActivityPending
         ActivityPendingException exc =  new ActivityPendingException ();
         Trace.exit ("LocalActivityContext.completionChecks");
         throw exc;
         }  // end if

       if ((this.children.size() > 0) &&
          (compStatus == CompletionStatus.CompletionStatusSuccess))
          {
          // We are trying to complete successfully but there are outstanding
          // child contexts - throw ContextPending
          ContextPendingException exc = new ContextPendingException ();
          Trace.exit ("LocalActivityContext.completionChecks");
          throw exc;
          }  // end if

      Trace.exit ("LocalActivityContext.completionChecks");
      }  // end LocalActivityContext.completionChecks


   /**
    * Suspend and remove the context from the hierarchy and notify
    * <code>PropertyGroups</code> of this event.
    * Called by the <code>ActivityCoordinator</code> to indicate that
    * completion processing has reached a point where the Activity
    * context is no longer required.
    *
    * @exception SystemException
    *      if an unexpected error condition arises
    */
   public void completed () throws SystemException
      {
      Trace.entry ("LocalActivityContext.completed");

      // remove from persistent table if there ...
      persistentActivities.remove (globalId);

      // Suspend context (drives PG's suspended ...)
      ActivityTokenImpl token = this.suspendContext ();

      // Mark children failed (we only have children if completionStatusFail)
      this.markChildrenFailed ();

      // Remove from context chain
      this.removeFromChain ();

      // Call completed on Property Groups
      Enumeration enum = this.propertyGroups.elements ();
      while (enum.hasMoreElements ())
         {
         PropertyGroup propertyGroup = (PropertyGroup) enum.nextElement ();
         propertyGroup.completed ();
         }  // end while

      // Mark this as completed so it can no longer be resumed
      this.completed = true;

      // Resume children
      ActivityTokenImpl childToken = token.getChildToken ();
      if (childToken != null)
         {
         LocalThreadContext child = childToken.getContext();
         try
            {
            child.resumeContext (childToken);
            }
         catch (InvalidActivityException exc)
            {
            // Absorb this exception ???? why
            Trace.exception ("LocalActivityContext.complete", exc);
            }
         }  // end if

      Trace.exit ("LocalActivityContext.completed");
      }  // end completed


   /**
    * Sets the completion status of this Activity.
    *
    * @param status  the new completion status
    *
    * @exception InvalidStateException
    *    if the Activity has been marked as failed and we are
    *    trying to set <code>CompletionStatusSuccess</code>
    *    or status is an invalid value.
    *
    */
   public void setCompletionStatus (int status)
      throws InvalidStateException
      {
      Trace.entry ("LocalActivityContext.setCompletionStatus",
         new Integer (status));

      if (
         // check state
         (completionStatus == CompletionStatus.CompletionStatusFailOnly &&
          status != CompletionStatus.CompletionStatusFailOnly) ||
         // check validity of parameter
         (status != CompletionStatus.CompletionStatusSuccess &&
          status != CompletionStatus.CompletionStatusFail &&
          status != CompletionStatus.CompletionStatusFailOnly)
         )
         {
         throw new InvalidStateException ();
         }  // end if

      this.completionStatus = status;

      Trace.exit ("LocalActivityContext.setCompletionStatus");
      }  // end setCompletionStatus


   /**
    * Gets the completion status of this Activity context
    *
    * @return  the completion status
    *
    */
   public int getCompletionStatus ()
      {
      Trace.entry ("LocalActivityContext.getCompletionStatus");

      Trace.exit ("LocalActivityContext.getCompletionStatus",
         new Integer (this.completionStatus));
      return this.completionStatus;
      }  // end getCompletionStatus



   /**
    * Resumes the Activity and any the nested scopes represented
    * by the <code>ActivityToken</code> as a child of the context that was on
    * the thread prior to the resume.  A context may only be resumed as a child
    * of the context within which it was originally begun.  The current thread
    * becomes associated with the Activity represented by the token.
    *
    * @param activityToken  an <code>ActivityToken</code> representing the
    *    Activity and any nested scopes, to be resumed
    *
    * @exception InvalidActivityException
    *   if this context is marked as completed, the token supplied by
    *   the caller was invalid
    *
    * @exception InvalidParentContextException
    *   if the context into which an activity is resumed is not
    *   the same as the context from which it was suspended
    *
    * @exception SystemException
    *   unexpected error condition
    */
   public void resume (ActivityTokenImpl activityToken) throws
      InvalidParentContextException, InvalidActivityException, SystemException
      {
      Trace.entry ("LocalActivityContext.resume", activityToken);

      // If this context has already completed then the user has passed an
      // invalid ActivityToken.  This is checked both here and in resumeContext
      // because we want to catch it as early as possible, but have to check it
      // at every level of nested context.
      // if (this.completed == true)
      if (this.activityCoordinator.getStatus () != Status.StatusActive)
         throw new InvalidActivityException (/*"already completed"*/);

      if (threadContextManager.getCurrentContext () != this.parentContext)
         {
         // The current context wasn't the original parent for this activity
         // - throw InvalidParentContext
         throw  new InvalidParentContextException ();
         }  // end if

      this.resumeContext (activityToken);

      Trace.exit ("LocalActivityContext.resume");
      }  //  end resume


  /**
   * Resumes the context contained in the <code>ActivityToken</code> and then
   * resumes any child context.
   *
   * @param activityToken  an <code>ActivityToken</code> representing the
   *     context to resume
   *
   * @exception InvalidActivityException
   *     if the activity state is not valid for this operation
   *
   * @exception SystemException
   *     if the system encounters an unexpected condition
   */
   public void resumeContext (ActivityTokenImpl activityToken)
      throws InvalidActivityException, SystemException
      {
      Trace.entry ("LocalActivityContext.resumeContext", activityToken);

      // If this context has already completed then the user has passed an
      // invalid ActivityToken.
      // if (this.completed == true)
      if (this.activityCoordinator.getStatus () != Status.StatusActive)
         throw new InvalidActivityException (/*"already completed"*/);

      super.resumeContext (activityToken);

      }  // end resumeContext


   /**
    * Marks this context as failed.
    *
    */
   protected void markFailed ()
      {
      Trace.entry ("LocalActivityContext.markFailed");

      super.markFailed ();

      // Set completion status to failed (irreversibly)
      this.completionStatus = CompletionStatus.CompletionStatusFailOnly;

      // Call completed on PropertyGroups (WHY ????)
      Enumeration enum = this.propertyGroups.elements ();
      while (enum.hasMoreElements ())
         {
         PropertyGroup propertyGroup = (PropertyGroup) enum.nextElement ();
         propertyGroup.completed ();
         }  // end while

      // Remove all the PropertyGroups so that any future call to
      // getPropertyGroup  throws PropertyGroupUnknown
      this.propertyGroups.clear ();

      Trace.exit ("LocalActivityContext.markFailed");
      }  // end markFailed


   /**
    * Notification that context is about to be suspended.
    * Inform property groups ...
    *
    */
   protected void suspended ()
      {
      Trace.entry ("LocalActivityContext.suspended");

      // Call suspended on PropertyGroups
      Enumeration enum = this.propertyGroups.elements ();
      while (enum.hasMoreElements ())
         {
         PropertyGroup propertyGroup = (PropertyGroup) enum.nextElement ();
         propertyGroup.suspended ();
         }  // end while

      Trace.exit ("LocalActivityContext.suspended");
      }  // end suspended


   /**
    * Notification that context has been resumed.
    * Notify property groups.
    *
    */
    protected void resumed ()
       {
       Trace.entry ("LocalActivityContext.resumed");

       // Call resumed on PropertyGroups
       Enumeration enum = this.propertyGroups.elements ();
       while (enum.hasMoreElements ())
          {
          PropertyGroup propertyGroup = (PropertyGroup) enum.nextElement ();
          propertyGroup.resumed ();
          }  // end while

       Trace.exit ("LocalActivityContext.resumed");
       }  // end resumed


   /**
    * Returns the <code>PropertyGroupManager</code>s for this Activity.
    *
    * @return the <code>PropertyGroupManager</code>s
    *
    */
   public Hashtable getPropertyGroupManagers ()
      {
      Trace.entry ("LocalActivityContext.getPropertyGroupManagers");

      Trace.exit ("LocalActivityContext.getPropertyGroupManagers",
         this.propertyGroupManagers);
      return this.propertyGroupManagers;
      }  // end getPropertyGroupManagers


   /**
    * Returns the <code>PropertyGroup</code> of the given name.
    *
    * @param name  the name of the property group to return
    *
    * @return the named <code>PropertyGroup</code> or null if it does not exist
    *
    */
   public PropertyGroup getPropertyGroup (String name)
      {
      Trace.entry ("LocalActivityContext.getPropertyGroup", name);

      PropertyGroup result = (PropertyGroup) propertyGroups.get (name);

      Trace.exit ("LocalActivityContext.getPropertyGroup", result);
      return result;
      }  // end getPropertyGroup



   /**
    * Getter for the GlobalId for this Activity.
    *
    * @return  the GlobalId for this Activity
    *
    */
   public GlobalId getGlobalId ()
      {
      Trace.entry ("LocalActivityContext.getGlobalId");

      Trace.exit ("LocalActivityContext.getGlobalId", globalId);
      return globalId;
      }  // end getGlobalId

   /**
    * Returns a printable form of the global id for this Activity
    *
    * @return  the name of this Activity
    */
   public String getName ()
      {
      Trace.entry ("LocalActivityContext.getName");

      String result = this.globalId.print ();

      Trace.exit ("LocalActivityContext.getName", result);
      return result;
      }  // end getName


   /**
    * Obtains the named <code>SignalSet</code> from the <code>ServiceManager
    * </code> and returns it.
    * Used by the <code>ActivityCoordinator</code> to obtain the
    * <code>SignalSet</code>s that it requires for processing
    *
    * @param signalSetName the name of the <code>SignalSet</code> to retrieve
    * @return        the required <code>SignalSet</code>
    *
    * @throws SignalSetUnknownException
    *    propagated from the <code>Servicemanager</code>
    * @throws SystemException
    *    if the ServiceManager does not implement this function
    *
    */
   public SubordinateSignalSet getSignalSet (String signalSetName)
      throws SignalSetUnknownException, SystemException
      {
      Trace.entry ("LocalActivityContext.getSignalSet");
      SubordinateSignalSet result = null;
      try {
         // call the servicemanager to get signal set ...
         result = (SubordinateSignalSet)serviceManager.getSignalSet (signalSetName);
         }
      catch (NoImplementException e)
         {
         throw new SystemException (
            "serviceManager getSignalSet not implemented");
         }

      Trace.exit ("LocalActivityContext.getSignalSet", result);
      return result;
      }

   /**
    * Obtains the name of the <i>completion SignalSet</i> from the
    * <code>ServiceManager</code> and returns it.
    * Used by the <code>ActivityCoordinator</code> to obtain the
    * completion <code>SignalSet</code> that it requires for processing
    *
    * @return        the required name
    *
    * @throws SystemException
    *    propagated from the ServiceManager call
    *
    */
   public String getCompletionSignalSetName ()
      throws SystemException
      {
      Trace.entry ("LocalActivityContext.getCompletionSignalSet");
      String result = serviceManager.getCompletionSignalSetName ();

      Trace.exit ("LocalActivityContext.getCompletionSignalSet", result);
      return result;
      }


   /**
    * Getter for the <code>ActivityCoordinator</code> object associated with
    * this Activity
    *
    * @return        the coordinator
    *
    */
   public ActivityCoordinator getCoordinator ()
      {
      Trace.entry ("LocalActivityContext.getCoordinator");
      Trace.exit ("LocalActivityContext.getCoordinator", activityCoordinator);
      return activityCoordinator;
      }

   /*
    * Essentially deletes this Activity.  If the Activity is persistent, then
    * the globalId is stored in the persistent table.
    * The Activity must be in the same state of thread assocations as for complete.
    */
   public void hibernate () throws SystemException,
      ActivityPendingException, ContextPendingException
      {
      Trace.entry ("LocalActivityContext.hibernate");

      // check that context is active on current thread and only one thread
      if(countAssociations()>1 || ! isAssociated ())
         throw new ActivityPendingException();

      if(children.size() > 0)
         throw new ContextPendingException();

      // mark persistent table entry ...
      if (persistentActivities.get (globalId) != null)
         persistentActivities.put (globalId, null);

      // Throw away references to activityCorrdinator and other activity service stuff
      activityCoordinator = null;
      serviceManager = null;
      propertyGroups = null;
      propertyGroupManagers = null;
      globalId = null;
      threadContextManager.removeContextFromThread();

      // Get thrown away by everything else (threadContextManager)
      // No associations to get rid of

      if (this.parentContext != null)
         // Remove chaining to parent
         this.parentContext.removeFromChildren (this);

      Trace.exit ("LocalActivityContext.hibernate");
      }

   /*
    * Marks this Activity as having persistent state.
    * We save a reference to the globalId.
    */
   public void setPersistent ()
      {
      Trace.entry ("LocalActivityContext.setPersistent");

      // add to persistent table
      persistentActivities.put (globalId, this);

      Trace.exit ("LocalActivityContext.setPersistent");
      }

   } // class LocalActivityContext

