/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.*;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;


/**
 * Abstract base class, extended by ActivityThreadContext.
 * It is used to manage the nesting
 * of contexts and thread association.
 *
 *
 */
abstract  class LocalThreadContext
   {

   /**
    * The <code>ThreadContextManager</code> instance.
    */
   protected ThreadContextManager threadContextManager = null;


   /**
    * Indicates which service created this <code>LocalThreadContext</code>
    */
   protected String  serviceName = "";

   /**
    * The timeout in seconds.
    */
   protected int timeout = -1;

   /**
    * Thread ids on which this context is active.
    */
   protected Vector associations = new Vector ();

   /**
    * <code>LocalThreadContext</code> of the immediate parent context (which
    * has to be the same on all threads).  It is set the first time that
    * <code>addAssociation</code> is called.
    */
   protected LocalThreadContext parentContext = null;

   /**
    * A mapping between threads and their corresponding active child
    * <code>LocalThreadContext</code>, as there can be different child contexts
    * active on different threads.
    */
   protected Hashtable activeChildren = new Hashtable ();

   /**
    * List of all children, active and suspended, on all threads.
    */
   protected Vector children = new Vector ();



   /**
    * Base class constructor.
    *
    * @param name  the service that created this context
    * @param tcm   the ThreadContextManager for our contextGroup
    *
    */
   public LocalThreadContext (String name, ThreadContextManager tcm)
      {
      Trace.entry ( "LocalThreadContext.LocalThreadContext", name);
      this.serviceName = name;
      threadContextManager = tcm;
      Trace.exit ( "LocalThreadContext.LocalThreadContext");
      }


   /**
    * Adds the current thread id to the list of threads on which this context is
    * active in the list of associations and calls
    * <code>ThreadContextManager.addContextToThread</code> to add this
    * thread/<code>LocalThreadContext</code> pair to the
    * <code>threadContextMap</code>.  Sets up parent and child chaining
    * information.
    *
    * @exception javax.activity.SystemException
    *              if this context is already associated with the given thread
    *
    */
   public void addAssociation() throws javax.activity.SystemException
      {
      Trace.entry ("LocalThreadContext.addAssociation");

      if (this.associations.contains (Thread.currentThread()))
         {
         // Thread is already associated - throw INTERNAL
         SystemException exc =
            new SystemException ("thread already associated");
         Trace.exit ("LocalActivityContext.addAssociation");
         throw exc;
         }

      // Get the current context (if any) on this thread as
      // this will be the new parent
      LocalThreadContext parent = threadContextManager.getCurrentContext ();

      // Chain together parent and child
      this.setParentContext (parent);
      if (parent != null)
         {
         parent.addActiveChild (this);
         }

      // Add thread to list of associations
      this.associations.addElement (Thread.currentThread ());

      // Add context to thread context map
      this.threadContextManager.addContextToThread (this);

      Trace.exit("LocalThreadContext.addAssociation");
      }


   /**
    * Removes the current thread from the list of threads on which this context
    * is active in the list of associations and calls
    * <code>ThreadContextManager.removeContextFromThread</code> to remove this
    * thread/<code>LocalThreadContext</code> pair from the threadContextMap.
    * Removes itself as active child of parent.
    *
    */
   public void removeAssociation ()
      {
      Trace.entry ("LocalThreadContext.removeAssociation");

      // Remove the thread from our list of associations.
      if (this.associations.removeElement (Thread.currentThread ()))
         {
         // Remove this context from the thread context map.
         this.threadContextManager.removeContextFromThread ();

         if (this.parentContext != null)
            {
            // Remove ourselves from our parent's list of active children.
            this.parentContext.removeActiveChild ();
            }
         }

      Trace.exit ("LocalThreadContext.removeAssociation");
      }


   /**
    * Returns the number of threads on which this context is active.
    *
    * @return  the number of threads on which this context is active
    *
    */
   public long countAssociations ()
      {
      Trace.entry ("LocalThreadContext.countAssociations");
      long result = this.associations.size();
      Trace.exit ("LocalThreadContext.countAssociations", new Long(result));
      return result;
      }


   /**
    * Indicates whether the context is active on the current thread.
    *
    * @return  boolean true if the current thread is associated with this ctx
    */
   public boolean isAssociated ()
      {
      Trace.entry ("LocalThreadContext.isAssociated");
      boolean result = this.associations.contains (Thread.currentThread());
      Trace.exit ("LocalThreadContext.isAssociated", new Boolean (result));
      return result;
      }


  /**
   * Returns a reference to the parent LTC for this context.
   *
   * @return  a reference to the parent LTC for this context
   *
   */
   public LocalThreadContext getParentContext ()
      {
      Trace.entry ("LocalThreadContext.getParentContext");
      Trace.exit ("LocalThreadContext.getParentContext", this.parentContext);
      return this.parentContext;
      }


  /**
   * Returns reference to context of requested type at or above the
   * nesting level of this context.
   *
   * @param name  the service name of context to return
   *
   * @return  the next parent context of the given type
   *
   */
   public LocalThreadContext getCurrentContext (String name)
      {
      Trace.entry ("LocalThreadContext.getCurrentContext", name);

      LocalThreadContext context = this;

      if (this.serviceName.equals (name))
         {
         context = this;
         }
      else
         {
         while ((context != null) && !(context.serviceName.equals (name)))
            {
            context = context.getParentContext ();
            }
         }
      Trace.exit ("LocalThreadContext.getCurrentContext", context);
      return context;
      }



  /**
   * Returns context at the top of the current nesting.
   *
   * @return  context at top of current nesting.
   *
   */
   public LocalThreadContext getTopContext ()
      {
      Trace.entry ("LocalThreadContext.getTopContext");

      LocalThreadContext context = this;

      while (context.getParentContext () != null)
         {
         context = context.getParentContext ();
         }

      Trace.exit ("LocalThreadContext.getTopContext", context);
      return context;
      }


  /**
   * Suspends the activity associated with the current thread and any nested
   * child scopes.
   *
   * @return    an <code>ActivityToken</code> representing the activity
   *    that was associated with the current thread prior to this call (or
   *    null if there was no such association)
   *
   */
   public ActivityTokenImpl suspend ()
      {
      Trace.entry ("LocalThreadContext.suspend");

      ActivityTokenImpl result = null;

      result = this.suspendContext ();

      Trace.exit ("LocalThreadContext.suspend", result);
      return result;
      }


  /**
   * Resumes the activity and any the nested scopes represented
   * by the <code>ActivityToken</code> as a child of the context that was on the
   * thread prior to the resume.  A context may only be resumed as a child of
   * the context within which it was originally begun.  The current thread
   * becomes associated with the activity represented by the token.
   *
   * @param activityToken  an <code>ActivityToken</code> representing the
   *                       activity and any nested scopes, to be resumed
   *
   * @exception InvalidActivityException
   *              if the activity state is not valid for this operation
   *
   * @exception InvalidParentContextException
   *              if the context into which an activity is resumed is not
   *              the same as the context from which it was suspended
   *
   * @exception SystemException
   *              if the system encounters an unexpected condition
   */
   public void resume (ActivityTokenImpl activityToken) throws
      InvalidParentContextException, InvalidActivityException, SystemException
      {
      Trace.entry ("LocalThreadContext.resume", activityToken);

      if (threadContextManager.getCurrentContext() != this.parentContext)
         {
         // The current context wasn't the original parent for this activity
         // - throw InvalidParentContext
         throw  new InvalidParentContextException ();
         }

      this.resumeContext (activityToken);

      Trace.exit ("LocalThreadContext.resume");
      }



  /**
   * Suspends any active child context, then removes current context from
   * thread.  Creates and returns a <code>ActivityToken</code>.
   *
   * @return  an <code>ActivityToken</code> representing the suspended context
   *
   */
   public ActivityTokenImpl suspendContext ()
      {
      Trace.entry ("LocalThreadContext.suspendContext");

      ActivityTokenImpl result = null;

      LocalThreadContext activeChild = this.getActiveChild ();

      if (activeChild != null)
         {
         result = new ActivityTokenImpl (this,
            activeChild.suspendContext ());
         }
      else
         {
         result = new ActivityTokenImpl (this, null);
         }

      // notify that context is now suspended (or about to be).
      this.suspended ();

      // Remove association
      this.removeAssociation ();

      Trace.exit ("LocalThreadContext.suspendContext", result);
      return result;
      }

  /**
   * Resumes the context contained in the <code>ActivityToken</code> and then
   * resumes any child context.
   *
   * @param activityToken  an <code>ActivityToken</code> representing
   *     the context to resume
   *
   * @exception InvalidActivityException
   *     if the activity state is not valid for this operation
   *
   * @exception SystemException
   *     if the system encounters an unexpected condition
   */
   public void resumeContext (ActivityTokenImpl activityToken)
      throws InvalidActivityException, SystemException
      {
      Trace.entry ("LocalThreadContext.resumeContext", activityToken);

      try {
         // Add association
         this.addAssociation ();

         // notify of resumption ...
         this.resumed ();

         // now resume any child tokens
         ActivityTokenImpl childToken = activityToken.getChildToken ();
         if (childToken != null)
            {
            // Resume child context
            LocalThreadContext childContext = childToken.getContext ();
            childContext.resumeContext (childToken);
            }
         }
      catch (InvalidActivityException e)
         {
         // if any of the contexts in the activityToken are not valid,
         // then 'resume' must not resume ANY contexts ... so undo
         this.suspended ();
         this.removeAssociation ();
         throw e;                  // pass on exception
         }

      Trace.exit ("LocalThreadContext.resumeContext");
      }


  /**
   * Provide notification that this context has been suspended.
   * Abstract method to be implemented by derived classes.
   */
   protected abstract void suspended ();

  /**
   * Provide notification that this context has been resumed.
   * Abstract method to be implemented by derived classes.
   */
   protected abstract void resumed ();



  /**
   * Returns an <code>ActivityToken</code> for this context and its active
   * children on the current thread.  It does not change the states of these
   * contexts.
   *
   * @return  an <code>ActivityToken</code> representing this context
   *
   */
   public ActivityTokenImpl getContextInfo ()
      {
      Trace.entry ("LocalThreadContext.getContextInfo");

      ActivityTokenImpl result = null;

      LocalThreadContext activeChild = this.getActiveChild ();

      if (activeChild == null)
         {
         result = new ActivityTokenImpl (this, null);
         }
      else
         {
         result = new ActivityTokenImpl (this, activeChild.getContextInfo ());
         }

      Trace.exit ("LocalThreadContext.getContextInfo", result);
      return result;
      }



  /**
   * Returns name of service that created this context.
   *
   * @return  the service name of this context
   *
   */
   public String getServiceName ()
      {
      Trace.entry ("LocalThreadContext.getServiceName");
      Trace.exit ("LocalThreadContext.getServiceName", serviceName);
      return this.serviceName;
      }


  /**
   * Marks this context as failed.
   * Used by derived classes
   */
   protected void markFailed ()
      {
      Trace.entry ("LocalThreadContext.markFailed");
      Trace.exit ("LocalThreadContext.markFailed");
      }


  /**
   * Removes this context from the chain.  The parent of this context replaces
   * this context as the parent of our children.
   *
   */
   protected void removeFromChain () throws javax.activity.SystemException
      {
      Trace.entry ("LocalThreadContext.removeFromChain");

      if (this.parentContext != null)
         {
         // Remove chaining to parent
         this.parentContext.removeFromChildren (this);

         // Add our active child
         this.parentContext.addActiveChild (this.getActiveChild ());
         }

      // Set new parent for children
      Enumeration enum = this.children.elements ();
      while (enum.hasMoreElements ())
         {
         LocalThreadContext child = (LocalThreadContext) enum.nextElement ();
         child.setParentContext (this.parentContext);
         }

      Trace.exit ("LocalThreadContext.removeFromChain");
      }


  /**
   * Marks all the children failed (active and suspended, on all threads).
   *
   */
   protected void markChildrenFailed ()
      {
      Trace.entry ("LocalThreadContext.markChildrenFailed");

      Enumeration enum = this.children.elements ();
      while (enum.hasMoreElements ())
         {
         LocalThreadContext child = (LocalThreadContext) enum.nextElement ();

         //Mark the child's children as failed
         child.markChildrenFailed ();

         // Mark the child failed
         child.markFailed ();
         }

      Trace.exit ("LocalThreadContext.markChildrenFailed");
      }


  /**
   * Sets the given context as the parent for this context and sets this as one of
   * that context's children.
   *
   * @param parent  the new parent context
   *
   */
   public void setParentContext (LocalThreadContext parent)
      {
      Trace.entry ("LocalThreadContext.setParentContext", parent);

      // Set parent and add this as one of its children
      this.parentContext = parent;
      if (parent != null)
         {
         parent.addToChildren (this);
         }

      Trace.exit ("LocalThreadContext.setParentContext");
      }


  /**
   * Returns the active child of this context on the current thread.
   *
   * @return  the active child of this context on the current thread
   *
   */
   public LocalThreadContext getActiveChild ()
      {
      Trace.entry ("LocalThreadContext.getActiveChild");

      LocalThreadContext result =
         (LocalThreadContext) this.activeChildren.get (Thread.currentThread ());

      Trace.exit ("LocalThreadContext.getActiveChild", result);
      return result;
      }


  /**
   * Maps the given child to the current thread.
   *
   * @param child  the active child to add
   *
   * @exception SystemException
   *          thrown if child context is already associated with this thread
   */
   public void addActiveChild (LocalThreadContext child)
      throws javax.activity.SystemException
      {
      Trace.entry ("LocalThreadContext.addActiveChild", child);

      if (this.activeChildren.containsKey (Thread.currentThread ()))
         {
         // There is already a context active on this thread - throw INTERNAL
         SystemException exc = new SystemException ("INTERNAL error");
         Trace.exception ("LocalThreadContext.addActiveChild", exc);
         Trace.exit ("LocalThreadContext.addActiveChild");
         throw exc;
         }

      if (child != null)
         {
         this.activeChildren.put (Thread.currentThread (), child);
         }

      Trace.exit ("LocalThreadContext.addActiveChild");
      }


  /**
   * Removes the entry for the current thread from the activeChildren map and
   * returns a reference to the context from that entry.
   *
   * @return   the active child removed from the current thread
   *
   */
   public LocalThreadContext removeActiveChild ()
      {
      Trace.entry ("LocalThreadContext.removeActiveChild");

      LocalThreadContext result =
         (LocalThreadContext) activeChildren.remove (Thread.currentThread ());

      Trace.exit ("LocalThreadContext.removeActiveChild", result);
      return result;
      }


  /**
   * Adds the given context to the list of children for this context.
   *
   * @param child  the child context to add
   *
   */
   public void addToChildren (LocalThreadContext child)
      {
      Trace.entry ("LocalThreadContext.addToChildren", child);

      // Make sure we don't add it more than once!
      if (!this.children.contains (child))
         {
         this.children.addElement (child);
         }

      Trace.exit ("LocalThreadContext.addToChildren");
      }


  /**
   * Removes the given context from the list of children for this context.
   *
   * @param child  the child context to remove
   *
   */
   public void removeFromChildren (LocalThreadContext child)
      {
      Trace.entry ("LocalThreadContext.removeFromChildren", child);

      this.children.removeElement (child);

      Trace.exit ("LocalThreadContext.removeFromChildren");
      }


  /**
   * Getter for timeout.
   *
   * @return  the timeout
   *
   */
   public int getTimeout ()
      {
      Trace.entry ("LocalThreadContext.getTimeout");

      Trace.exit ("LocalThreadContext.getTimeout", new Integer(this.timeout));
      return this.timeout;
      }

   }  // end class LocalThreadContext


