/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.*;
import javax.activity.coordination.*;


/**
 * The SynchronizationSignalSet class is the implementation of
 * the org.omg.CosActivity.Synchronization signal set that is one of the
 * signal sets that must be supplied by a CORBA or J2EE compliant
 * Activity Service.
 *
 * It is invoked before and after completion of an activity.
 * The completion status of an activity may be changed by the
 * {@link javax.activity.Action Actions} registered with this signal
 * set, such that the {@link javax.activity.Outcome Outcome} when
 * postCompletion is called may be different from that when preCompletion
 * was invoked. If an Action error occurs during preCompletion (e.g. the
 * {@link javax.activity.ActionErrorException ActionErrorException} is
 * thrown) then the activity completion status will be set to
 * CompletionStatusFailOnly.
 *
 * The SynchronizationSignalSet produces two signals:
 * preCompletion - the recipient is informed that the activity is
 * about to complete.
 * postCompletion - the recipient is informed that the Activity has
 * completed.
 */
final class SynchronizationSignalSet implements
   javax.activity.coordination.SubordinateSignalSet
   {
   private String    _signalSetName     = "org.omg.CosActivity.Synchronization";
   private int       _completionStatus  = -1;
   private boolean   _outcomeSuccess    = true;
   private GlobalId  _globalId          = null;
   private Outcome   _finalOutcome      = null;

   /**
    * _thisStage represents an internal state to reflect which
    * stage the coordinator is driving, and also if the
    * completionStatus has been set yet.
    * It can have the constant values listed below.
    */
   private int _thisStage = _stageNull;

   /**
    * holding state before setCompletionStatus called
    */
   private static final int _stageNull = 0;

   /**
    * set after setCompletionStatus CompletionStatusSuccess
    */
   private static final int _stageBegin = 1;

   /**
    * set after setCompletionStatus CompletionStatusFail/Only or
    * after preCompletion has been returned by getSignal
    */
   private final int _stagePreCompletion = 2;

   /**
    * All signals have been sent
    */
   private final int _stagePostCompletion = 3;


   /**
    * Constructor which initializes all the fields.
    * @param   globalId   the <code>Globalid</code> of the completing Activity
    *
    */
   public SynchronizationSignalSet (javax.activity.GlobalId globalId)
      {
      Trace.entry ("SynchronizationSignalSet.SynchronizationSignalSet",
         globalId);

      _globalId      = globalId;

      Trace.exit ("SynchronizationSignalSet.SynchronizationSignalSet");
      }

   /**
    * Getter for the name of this SignalSet.
    *
    * @return always returns "org.omg.CosActivity.Synchronization"
    */
   public String getSignalSetName ()
      {
      Trace.entry ("SynchronizationSignalSet.getSignalSetName");
      Trace.exit ("SynchronizationSignalSet.getSignalSetName", _signalSetName);

      return _signalSetName;
      }

   /**
    * Signals produced by a supplied signal set must create an
    * {@link javax.activity.ActivityInformation ActivityInformation} object,
    * containing the activity's {@link javax.activity.GlobalId GlobalId}
    * and {@link javax.activity.CompletionStatus CompletionStatus}.
    *
    * @return The next <code>Signal<code> from the
    *     <code>SynchronizationSignalSet</code>
    */
   public Signal getSignal ()
      {
      Trace.entry ("SynchronizationSignalSet.getSignal");

      // string representation of either pre or postCompletion
      String thisStageString = null;

      Signal sig = null;

      switch (_thisStage)
         {
      case _stageBegin:
         // setCompletionStatus method was called previously,
         // so we are in preCompletion
         _thisStage = _stagePreCompletion;
         thisStageString = "preCompletion";

         sig = new Signal (thisStageString, _signalSetName,
            new ActivityInformation(_globalId, _completionStatus));
         break;

      case _stagePreCompletion:
         // switch from preCompletion to postCompletion
         _thisStage = _stagePostCompletion;
         thisStageString = "postCompletion";

         sig = new Signal (thisStageString, _signalSetName,
            new ActivityInformation (
               _globalId, _completionStatus, _finalOutcome));
         break;

      default:
         //  setCompletionStatus has not yet been set (_stageNull)
         //  or, postCompletion already finished (_stagePostCompletion)
         sig = null;
         }  // end case

      Trace.exit ("SynchronizationSignalSet.getSignal", sig);
      return sig;
      }

   /**
    * A means of passing in update completionStatus and finalOutcome
    * This will change the completionStatus accordingly.
    */
   public void setSignal (Signal signal)
      {
      Trace.entry ("SynchronizationSignalSet.setSignal", signal);

      if (signal != null)
         {
         try {
            java.io.Serializable ioser = signal.getExtendedValue ();
            if (ioser != null && (ioser instanceof ActivityInformation))
               {
               ActivityInformation activityInfo = (ActivityInformation) ioser;
               _completionStatus = activityInfo.getCompletionStatus ();
               _finalOutcome     = activityInfo.getFinalOutcome ();
               }  // end if
            }
         catch (Exception e)
            {
            Trace.exception ("SynchronizationSignalSet.setSignal", e);
            }

         }  // end if

      Trace.exit ("SynchronizationSignalSet.setSignal");
      }

   /**
    * If an attempt to set the response is made before getSignal() has been
    * called then a {@link javax.activity.SignalSetInactiveException
    *    SignalSetInactiveException} is thrown.
    *
    * @return The <code>CoordinationInformation</code> object containing the <code>
    *    isActionInterested</code> and <code>useNextSignal</code> parameters
    */
   public CoordinationInformation setResponse (Outcome response) throws
      SignalSetInactiveException
      {
      Trace.entry ("SynchronizationSignalSet.setResponse", response);

      CoordinationInformation result = null;
      if (_thisStage == _stageNull)
         {
         throw new SignalSetInactiveException ();
         }

      boolean _isa = true;
      boolean _useNextSignal = false;

      if (response != null &&
         (response.getName().equals (
            "org.omg.CosActivity.preCompletionFailed") ||
         response.getName().equals ("org.omg.CosActivity.ActionError")) &&
         (_thisStage == _stageBegin || _thisStage == _stagePreCompletion))
         {
         _completionStatus =
            javax.activity.CompletionStatus.CompletionStatusFailOnly;
         _useNextSignal = true;
         _outcomeSuccess = false;
         }

      result = new CoordinationInformation (_isa, _useNextSignal);
      Trace.exit ("SynchronizationSignalSet.setResponse", result);

      return result;
      }

   /**
    * A subordinate coordinator can use getOutcome() to obtain the final
    * Outcome from the SynchronizationSignalSet, but only after it has finished producing the preCompletion Signal.
    * If called before getSignal() or setSignal() then a
    * {@link javax.activity.SignalSetActiveException SignalSetActiveException}
    * is thrown.
    *
    * @return    The final <code>Outcome</code> from the
    *     <code>SynchronizationSignalSet</code>
    */
   public Outcome getOutcome () throws SignalSetActiveException
      {
      Trace.entry ("SynchronizationSignalSet.getOutcome");

      if (_thisStage == _stageNull)
         {
         throw new SignalSetActiveException ();
         }

      Outcome _outcome;
      if (_outcomeSuccess == true)
         {
         _outcome = new Outcome ("org.omg.CosActivity.preCompletionSuccess",
            (java.io.Serializable) null);
         }
      else
         {
         _outcome = new Outcome ("org.omg.CosActivity.preCompletionFailed",
            (java.io.Serializable) null);
         }

      Trace.exit ("SynchronizationSignalSet.getOutcome", _outcome);
      return _outcome;
      }

   /**
    * A subordinate coordinator can use getCurrentOutcome() to obtain the
    * current Outcome from the SynchronizationSignalSet, at any time.
    *
    * @return The current <code>Outcome</code> from the
    *     <code>SynchronizationSignalSet</code>
    */
   public Outcome getCurrentOutcome ()
      {
      Trace.entry ("SynchronizationSignalSet.getCurrentOutcome");

      Outcome _outcome;
      if (_outcomeSuccess == true)
         {
         _outcome = new Outcome ("org.omg.CosActivity.preCompletionSuccess",
            (java.io.Serializable) null);
         }
      else
         {
         _outcome = new Outcome ("org.omg.CosActivity.preCompletionFailed",
            (java.io.Serializable) null);
         }

      Trace.exit ("SynchronizationSignalSet.getCurrentOutcome", _outcome);
      return _outcome;
      }

   /**
    * Called by the Activity service to set both the CompletionStatus
    * and Status before any signals are requested. The signals produced
    * may be influenced by these two statuses.
    * Once completion of an Activity has begun, control of the
    * CompletionStatus belongs to the Completion SignalSet and cannot
    * be overridden by any other entity.
    *
    * @param completionStatus The {@link javax.activity.CompletionStatus
    *    CompletionStatus}  of the Activity at the time of the invocation.
    *
    * @param status The {@link javax.activity.Status Status}
    *    of the Activity at the time of the invocation. This can be ignored
    *    in the SynchronizationSignalSet (always StatusCompleting)
    *
    */
   public void setCompletionStatus (int completionStatus, int status)
      {
      Trace.entry ("SynchronizationSignalSet.setCompletionStatus",
         new Integer (completionStatus));

      if ((_thisStage >= _stagePreCompletion) && (_completionStatus !=
         javax.activity.CompletionStatus.CompletionStatusSuccess))
         Trace.out (
            "setCompletionStatus() called after getSignal() is not allowed");
      else
         {
         _completionStatus = completionStatus;

         if (_completionStatus ==
            javax.activity.CompletionStatus.CompletionStatusSuccess)
            {
            _thisStage = _stageBegin;
            _outcomeSuccess = true;
            }
         else
            {
            _thisStage = _stagePreCompletion;
            _outcomeSuccess = false;
            }
         }

      Trace.exit ("SynchronizationSignalSet.setCompletionStatus");
      }

   /**
    * Called by the Activity service to get the CompletionStatus, after the
    * SynchronizationSignalSet has finished producing signals. If called too
    * early then a SignalSetActiveException is thrown.
    *
    * @return The CompletionStatus after the
    *     <code>SynchronizationSignalSet</code> has finished producing signals
    */
   public int getCompletionStatus () throws SignalSetActiveException
      {
      Trace.entry ("SynchronizationSignalSet.getCompletionStatus");

      if (_thisStage < _stagePreCompletion)
         {
         throw new SignalSetActiveException ();
         }

      Trace.exit ("SynchronizationSignalSet.getCompletionStatus",
         new Integer (_completionStatus));
      return _completionStatus;
      }

   /**
    * Called on the target object, by the Activity service, when the
    * target object is no longer required and indicates to the object
    * that it should perform any necessary clean-up.
    * The ActivityCoordinator with which the target object was associated
    * is not available at the time this method is called.
    */
   public void destroy ()
      {
      Trace.entry ("SynchronizationSignalSet.destroy");

      // nullify existing object references
      _signalSetName = null;
      _globalId = null;

      Trace.exit ("SynchronizationSignalSet.destroy");
      }

   public void setActivityCoordinator (ActivityCoordinator coord)
      throws SignalSetActiveException
      {
      Trace.entry ("SynchronizationSignalSet.setActivityCoordinator");
      Trace.exit ("SynchronizationSignalSet.setActivityCoordinator");
      }  // end SynchronizationSignalSet.setActivityCoordinator

   }   // end class SynchronizationSignalSet
