/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

// import javax.activity.ActivityToken;

import java.util.Hashtable;
import java.util.Enumeration;

import javax.activity.ActivityToken;
import javax.activity.GlobalId;


/**
 * This class manages the association of activity contexts
 * with threads in the current process.  There is an instance for
 * each context group
 *
 */
final class ThreadContextManager
   {

   /**
    * A mapping between a threadId and the active context on that thread
    * in the contextGroup.
    */
   private Hashtable threadContextMap = new Hashtable();

   /**
    * The context group represented by this instance
    */
   private String    contextGroup;

   /**
    * A mapping between contextGroup name and its specific ThreadContextManager
    */
   static  private Hashtable contextGroupMap = new Hashtable();


   /**
    * Constructor.
    *
    */
   protected ThreadContextManager (String cGroup)
      {
      Trace.entry ("ThreadContextManager.ThreadContextManager", cGroup);
      contextGroup = cGroup;
      contextGroupMap.put (cGroup, this);
      Trace.exit ("ThreadContextManager.ThreadContextManager");
      }




   /**
    * Returns the current context (of ANY type) on the current thread.
    * This is the lowest-level nested context on the thread.
    *
    * @return  the current <code>LocalThreadContext</code> of any type on the
    *          current thread.
    *
    */
   public LocalThreadContext getCurrentContext ()
      {
      Trace.entry ("ThreadContextManager.getCurrentContext");

      LocalThreadContext result = (LocalThreadContext)
         this.threadContextMap.get (Thread.currentThread ());

      Trace.exit ("ThreadContextManager.getCurrentContext", result);
      return result;
      }



   /**
    * Returns the current context of the specified service for the current
    * thread.
    *
    * @param serviceName  the type of context required
    * @return  the current context of the specified type for the current thread
    *
    */
   public LocalThreadContext getCurrentContext (String serviceName)
      {
      Trace.entry ("ThreadContextManager.getCurrentContext", serviceName);

      LocalThreadContext result = null;

      // Get the current context of any type
      LocalThreadContext currentContext = this.getCurrentContext ();
      if (currentContext != null)
         {
         // Ask it for the current context of the given type
         result = currentContext.getCurrentContext (serviceName);
         }

      Trace.exit ("ThreadContextManager.getCurrentContext", result);
      return result;
      }



   /**
    * Returns context at the top of the current nesting.
    *
    * @return  context at top of current nesting
    *
    */
   public LocalThreadContext getTopContext ()
      {
      Trace.entry ("ThreadContextManager.getTopContext");

      LocalThreadContext result = null;

      // Get the current context
      LocalThreadContext currentContext = this.getCurrentContext ();

      if (currentContext != null)
         {
         // Get it's top context
         result = currentContext.getTopContext ();
         }

      Trace.exit ("ThreadContextManager.getTopContext", result);
      return result;
      }


   /**
    * Add a context to the current thread in the threadContextMap
    *
    * @param context  the <code>LocalThreadContext</code> to add
    *
    */
   public void addContextToThread (LocalThreadContext context)
      {
      Trace.entry ("ThreadContextManager.addContextToThread", context);

      this.threadContextMap.put (Thread.currentThread (), context);

      Trace.exit ("ThreadContextManager.addContextToThread");
      }


   /**
    * Removes a context from the current thread in the threadContextMap
    * Adds parent of current context as new thread-context pair in
    * thread context map.
    *
    */
   public void removeContextFromThread ()
      {
      Trace.entry ("ThreadContextManager.removeContextFromThread");

      LocalThreadContext removedContext = (LocalThreadContext)
         this.threadContextMap.remove (Thread.currentThread ());

      if (removedContext != null)
         {
         LocalThreadContext newCurrentContext =
            removedContext.getParentContext ();
         if (newCurrentContext != null)
            {
            this.threadContextMap.put (Thread.currentThread (),
               newCurrentContext);
            }
         }

      Trace.exit ("ThreadContextManager.removeContextFromThread");
      }


   /**
    * Getter for the size of the thread context map
    *
    * @return  size of thread context map
    *
    */
   public int getThreadContextMapSize ()
      {
      Trace.entry ("ThreadContextManager.getThreadContextMapSize");

      int result = this.threadContextMap.size ();

      Trace.exit ("ThreadContextManager.getThreadContextMapSize",
         new Integer (result));
      return result;
      }


//------------------------------------------------------------------------------
// CLASS METHODS
//------------------------------------------------------------------------------


   /**
    * Constructs and returns an ActivityToken that represents all the contexts
    * that are active on the current thread.
    *
    * @return  token representing all active contexts
    *
    */
   static public ActivityTokenImpl getRootActivityToken ()
      {
      Trace.entry ("ThreadContextManager.getRootActivityToken");
      ActivityTokenImpl returnToken = null;
      ActivityTokenImpl       token = null;

      // loop through all the thread context maps and get an ActivityToken for
      // each, linking them together via the childToken reference to construct
      // a single token for return.
      Enumeration enum = contextGroupMap.elements ();

      while (enum.hasMoreElements ())
         {
         ThreadContextManager tcm = (ThreadContextManager) enum.nextElement ();
         Hashtable threadGroupMap = tcm.threadContextMap;

         // Get the current context (if any) for this thread
         LocalThreadContext context = (LocalThreadContext)
            threadGroupMap.get (Thread.currentThread ());
         if (context != null)
            {
            // get the top context for this contextGroup
            LocalThreadContext topContext = context.getTopContext ();

            token = topContext.getContextInfo ();
            if (returnToken != null)
               {
               // link the new token to the bottom of the existing one
               returnToken.link (token);
               }
            else
               {
               // Save this first token as the one to return
               returnToken = token;
               }
            }
         }

      Trace.exit ("ThreadContextManager.getRootActivityToken", returnToken);
      return returnToken;
      }


   /**
    * Suspends all contexts that are active on the current thread
    *
    * @return  token representing all the suspended contexts
    *
    */
   static public ActivityTokenImpl suspendAllContexts ()
      {
      Trace.entry ("ThreadContextManager.suspendAllContexts");

      ActivityTokenImpl returnToken = null;
      ActivityTokenImpl       token = null;

      // loop through all the thread context maps and get an ActivityToken for
      // each, linking them together via the childToken reference to construct
      // a single token for return.
      Enumeration enum = contextGroupMap.elements ();

      while (enum.hasMoreElements ())
         {
         ThreadContextManager tcm = (ThreadContextManager) enum.nextElement ();
         Hashtable threadGroupMap = tcm.threadContextMap;

         // Get the current context (if any) for this thread
         LocalThreadContext context = (LocalThreadContext)
            threadGroupMap.get (Thread.currentThread ());
         if (context != null)
            {
            LocalThreadContext topContext = context.getTopContext ();

            token = topContext.suspendContext ();
            if (returnToken != null)
               {
               // link the new token to the bottom of the existing one
               returnToken.link (token);
               }
            else
               {
               // Save this first token as the one to return
               returnToken =  token;
               }
            }
         }

      Trace.exit ("ThreadContextManager.suspendAllContexts", returnToken);
      return returnToken;
      }


   /**
    * Returns true if there is a context active on the current thread.
    *
    * @return true if there is an active context on the current thread.
    *
    */
   static public boolean anyCurrentContext ()
      {
      Trace.entry ("ThreadContextManager.anyCurrentContext");

      boolean result = false;
      LocalThreadContext context = null;

      // loop through the threadContextMaps looking for a context on the
      // current thread.  If one is found, return true.

      Enumeration enum = contextGroupMap.elements ();

      while (enum.hasMoreElements ())
         {
         ThreadContextManager tcm = (ThreadContextManager) enum.nextElement ();
         Hashtable threadGroupMap = tcm.threadContextMap;

         // Get the current context (if any) for this thread
         context = (LocalThreadContext)
            threadGroupMap.get (Thread.currentThread ());
         if (context != null)
            {
            result = true;
            break;
            }
         }

      Trace.exit ("ThreadContextManager.anyCurrentContext",
         new Boolean(result));
      return result;
      }


   /** Returns the number of contexts of any type active on the current thread.
    *
    * @param boolean will be set TRUE if one or more of the contexts is an
    *                           Activity context
    *
    * @return the number of contexts of any type active on the current thread.
    *
    */
   static public int getNumActiveContexts (
      org.omg.CORBA.BooleanHolder ag_included)
      {
      int num = 0;
      // ag_included = false;

      Trace.entry ("ThreadContextManager.getNumActiveContexts");

      // Loop through the map of contextGroup thread maps.
      Enumeration enum = contextGroupMap.elements ();

      while (enum.hasMoreElements ())
         {
         ThreadContextManager tcm = (ThreadContextManager) enum.nextElement ();
         Hashtable threadGroupMap = tcm.threadContextMap;

         // Start with the current context from this map then chain up
         // to the top, counting contexts and looking out for an activity
         // on the way.
         LocalThreadContext context = (LocalThreadContext)
            threadGroupMap.get (Thread.currentThread ());

         while (context != null)
            {
            num++;
            context = context.getParentContext ();
            }
         }

      Trace.exit ("ThreadContextManager.getNumActiveContexts",
         new Integer(num));
      return num;
      }


   /**
    * Returns the ThreadContextManager instance for the given context group.
    * Creates the TCM if necessary.
    *
    * @param contextGroup
    *    the name of the context group required (may be empty string)
    *
    * @return the appropriate TCM
    *
    */
  static public ThreadContextManager getThreadContextManager (
     String contextGroup)
     {
     Trace.entry ("ThreadContextManager.getThreadContextManager",
       contextGroup);
     ThreadContextManager  tcm =
        (ThreadContextManager) contextGroupMap.get (contextGroup);
     if (tcm == null)
        {
        tcm = new ThreadContextManager (contextGroup);
        }  // end if

     Trace.exit ("ThreadContextManager.getThreadContextManager", tcm);
     return tcm;
     }

   /*
    * Find active context (on any thread) based on its globalId.
    */
   static public LocalActivityContext getLocalActivityContext(GlobalId gid) 
      {
      Trace.entry("ThreadContextManager.getLocalActivityContext");
      LocalActivityContext result = null;

      Enumeration enum = contextGroupMap.elements();

      while (enum.hasMoreElements() && (result == null)) 
         {
         ThreadContextManager tcm =
            (ThreadContextManager) enum.nextElement();
         Hashtable threadGroupMap = tcm.threadContextMap;

         Enumeration ltcs = threadGroupMap.elements();
         while (ltcs.hasMoreElements() && (result == null)) 
            {
            LocalActivityContext ltc =
               (LocalActivityContext) ltcs.nextElement();
            if (ltc.getGlobalId().equals(gid)) 
               {
               result = ltc;
               }
            }  // end while

         } // end while

      Trace.exit("ThreadContextManager.getLocalActivityContext", result);
      return result;
      }

   }; // Class ThreadContextManager
