/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/* A Trace utility class for the reference implementation. The only   */
/* public methods are static clients need create no instance.         */
/* This single-instance class writes trace to the                     */
/* standard output stream, conditional upon the following property    */
/* being set:                                                         */
/* com.ibm.activity.Trace=true                                        */
/*                                                                    */
/* This can be set on the command line when invoking the main java    */
/* executable, eg                                                     */
/*    java -Dcom.ibm.activity.Trace=true someJavaProg                 */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;
import java.util.Properties;

final class Trace
   {
   /**
    * Simply prints the text String to stdout.
    * @param   text   the String to print.
    */
   public static void out (String text)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println (text);
         }
      }

   /**
    * Trace the entry to a method call.
    * @param   text   the String to print. The name of the method
    */
   public static void entry (String text)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println ("<" + text);
         }
      }

   /**
    * Trace the entry to a method call.
    * @param   text   the String to print. The name of the method
    * @parm    o      ancilliary object to add to the trace output
    */
   public static void entry (String text, java.lang.Object o)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println ("<" + text + ":" + o);
         }
      }

   /**
    * Trace the exit from a method call.
    * @param   text   the String to print. The name of the method
    */
   public static void exit (String text)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println (">" + text);
         }
      }

   /**
    * Trace the exit from a method call.
    * @param   text   the String to print. The name of the method
    * @parm    o      ancilliary object to add to the trace output
    */
   public static void exit (String text, java.lang.Object o)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println (">" + text + ":" + o);
         }
      }

   /**
    * Trace an exception
    * @param   text   the String to print. The name of the method
    * @param   e      the Exception that is also added to the trace.
    */
   public static void exception (String text, Exception e)
      {
      if (_instance == false)
         {
         instance ();
         _instance = true;
         }

      if (_traceFlag)
         {
         System.out.println ("***" + text);
         e.printStackTrace ();
         }
      }

   /**
    * Initialises the Trace class.
    *  Sets the _instance class variable so that initialization only
    *  occurs once.
    */
   private static void instance ()
      {
      Properties props = System.getProperties ();
      String traceProp = props.getProperty ("com.ibm.activity.Trace");
      if ((traceProp != null) && traceProp.equalsIgnoreCase ("true"))
         {
         _traceFlag = true;
         System.out.println ("-- Trace enabled --");
         }
      }

   private static boolean _traceFlag = false;
   private static boolean _instance = false;
   }
