/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.reference;

import javax.activity.propertygroup.*;
import javax.activity.coordination.*;
import javax.activity.*;

import java.util.Hashtable;

/**
 * The UserActivity interface defines the methods offered to unit-of-work
 * (UOW) services constructed on top of the Activity service framework.
 * These methods control the demarcation scope of an Activity associated
 * with the calling thread of execution.
 * <br> The specific behaviour of individual UOW services is encapsulated by
 * a {@link javax.activity.coordination.ServiceManager ServiceManager}. An
 * instance of the UserActivity interface has a <i>ServiceManager</i> registered
 * with it prior to any Activities being begun by that instance.
 * <br> Each Activity started by a UserActivity instance is an
 * Activity instance of the UOW service represented by the registered
 * <i>ServiceManager</i>. Methods of the UserActivity interface that operate
 * on the <i>active</i> Activity context are operating on the UOW service
 * Activity instance most recently associated with the calling thread.
 *
 */
public class UserActivityImpl implements UserActivity
   {
    // CLASS Variables
   /**
    *  Table of all registered services
    */
   static private Hashtable    serviceManagers = new Hashtable ();


   // INSTANCE variables
   /**
    *  List of PropertyGroupManagers keyed on propertyGroupName
    */
   private Hashtable    propertyGroupManagers = new Hashtable ();

   /**
    *  our service manager (null == no manager registered)
    */
   protected ServiceManager    serviceManager = null;

   /**
    *  our thread context manager (shared with others in same contextGroup)
    */
   protected ThreadContextManager threadContextManager = null;

   /**
    *  our default timeout
    */
   protected int defaultTimeout = 0;

   // IMPLEMENTATION specific methods
   /**
    *  Constructor
    *  gives the impression that this is a singleton class, so make the
    *  c'tor protected
    */
    protected UserActivityImpl ()
       {
       Trace.entry ("UserActivityImpl.UserActivityImpl");

       // get a default ThreadContextManager ... until a service registers
       threadContextManager = ThreadContextManager.
          getThreadContextManager ("");   // default context group

       Trace.exit ("UserActivityImpl.UserActivityImpl");
       }

   /**
    *  Give the impression that this is a singleton class, so provide a
    *  method to access this object.
    *  @return   a new instance of UserActivity
    */
    static public UserActivity  instance ()
       {
       Trace.entry ("UserActivityImpl.instance");

       // we're not really a singleton ...
       UserActivity result = new UserActivityImpl ();

       Trace.exit ("UserActivityImpl.instance", result);
       return result;
       }




   // INTERFACE methods

   public void begin (int timeout) throws InvalidStateException,
      ServiceNotRegisteredException, TimeoutRangeException, SystemException
      {
      begin(timeout, null);
      }

   public void begin (int timeout, GlobalId id) throws InvalidStateException,
      ServiceNotRegisteredException, TimeoutRangeException, SystemException
      {
      Trace.entry ("UserActivityImpl.begin", new Integer (timeout));
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      if (timeout < -1)
         throw new TimeoutRangeException ();

      LocalActivityContext context = new LocalActivityContext (
         this.propertyGroupManagers,
         this.serviceManager,
         this.threadContextManager, id);

      context.begin (timeout);

      Trace.exit ("UserActivityImpl.begin");
      }



   public Outcome complete() throws
      ServiceNotRegisteredException, NoActivityException,
      ActivityPendingException, ActivityNotProcessedException,
      ContextPendingException, NotOriginatorException, SystemException
      {
      Trace.entry ("UserActivityImpl.complete");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      Outcome result = null;

      // ???? what if no service registered ????
      // ... then can't be any contexts (if remote, then complete will fail)

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      result = context.complete ();

      Trace.exit ("UserActivityImpl.complete", result);
      return result;
      }


   public Outcome completeWithStatus (int completionStatus) throws
      NoActivityException, ActivityPendingException, ContextPendingException,
      ServiceNotRegisteredException, NotOriginatorException,
      ActivityNotProcessedException, InvalidStateException, SystemException
      {
      Trace.entry ("UserActivityImpl.completeWithStatus",
         new Integer (completionStatus));
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();


      Outcome result = null; // new Outcome ("", new org.omg.any ()) ????

      // ???? what if no service registered ????
      // ... then can't be any contexts (if remote, then complete will fail)

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      context.setCompletionStatus (completionStatus);
      result = context.complete ();

      Trace.exit ("UserActivityImpl.completeWithStatus", result);
      return result;
      }


   public void setCompletionStatus (int completionStatus) throws
      ServiceNotRegisteredException, NoActivityException,
      InvalidStateException, SystemException
      {
      Trace.entry ("UserActivityImpl.setCompletionStatus",
         new Integer (completionStatus));
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();


      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      context.setCompletionStatus (completionStatus);

      Trace.exit ("UserActivityImpl.setCompletionStatus");
      }


   public int getCompletionStatus () throws NoActivityException,
      ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.getCompletionStatus");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      int result = 0;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      result = context.getCompletionStatus ();

      Trace.exit ("UserActivityImpl.getCompletionStatus", new Integer (result));
      return result;
      }


   public int getStatus() throws ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.getStatus");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      int result = 0;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         result = Status.StatusNoActivity;
      else
         {
         ActivityCoordinator coord = context.getCoordinator ();
         if (coord == null)
            throw new SystemException ("Missing coordinator");

         result = coord.getStatus ();
         }  // end else

      Trace.exit ("UserActivityImpl.getStatus", new Integer (result));
      return result;
      }


   public String getName () throws ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.getName");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      String result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context != null)
         {
         // build a name from perhaps the GlobalId ????
         result = context.toString ();
         }

      Trace.exit ("UserActivityImpl.getName", result);
      return result;
      }


   public void setTimeout (int timeout) throws
      ServiceNotRegisteredException, TimeoutRangeException, SystemException
      {
      Trace.entry ("UserActivityImpl.setTimeout", new Integer (timeout));
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      if (timeout < -1)
         throw new TimeoutRangeException ();

      defaultTimeout = timeout;
      Trace.exit ("UserActivityImpl.setTimeout");
      }


   public int getTimeout() throws ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.getTimeout");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      Trace.exit ("UserActivityImpl.getTimeout", new Integer (defaultTimeout));
      return defaultTimeout;
      }


   public GlobalId getGlobalId() throws ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.getGlobalId");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      GlobalId result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context != null)
         result = context.getGlobalId ();

      Trace.exit ("UserActivityImpl.getGlobalId", result);
      return result;
      }


   public Outcome broadcast (String signalSetName) throws NoActivityException,
      ServiceNotRegisteredException, SignalSetUnknownException,
      ActivityNotProcessedException, InvalidActivityException, SystemException
      {
      Trace.entry ("UserActivityImpl.broadcast", signalSetName);
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      Outcome result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      // don't allow synch/child signalsets
      if (signalSetName != null && (
         signalSetName.equals ("org.omg.CosActivity.ChildLifetime") ||
         signalSetName.equals ("org.omg.CosActivity.Synchronization") ))
         throw new InvalidActivityException ();

      ActivityCoordinator coord = context.getCoordinator ();
      if (coord != null)
         {
         if (coord.getStatus () != Status.StatusActive)
            throw new InvalidActivityException ();

         result = ((ActivityCoordinatorImpl) coord).processSignalSet (
            signalSetName, context.getCompletionStatus ());
         }  // end if

      Trace.exit ("UserActivityImpl.broadcast", result);
      return result;
      }


   public ActivityCoordinator getCoordinator () throws
      ServiceNotRegisteredException, NoImplementException, SystemException
      {
      Trace.entry ("UserActivityImpl.getCoordinator");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      ActivityCoordinator result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context != null)
         result = context.getCoordinator ();

      Trace.exit ("UserActivityImpl.getCoordinator", result);
      return result;
      }


   public ActivityCoordinator getParentCoordinator () throws
      ServiceNotRegisteredException, NoImplementException, SystemException
      {
      Trace.entry ("UserActivityImpl.getParentCoordinator");
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      ActivityCoordinator result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context != null)
         {
         LocalThreadContext ltc = context.getParentContext ();
         if (ltc != null && (ltc instanceof LocalActivityContext))
            {
            result = ((LocalActivityContext) ltc).getCoordinator ();
            }

         }  // end if

      Trace.exit ("UserActivityImpl.getParentCoordinator", result);
      return result;
      }


   public PropertyGroup getPropertyGroup (String name) throws
      ServiceNotRegisteredException, PropertyGroupUnknownException,
      NoActivityException, SystemException
      {
      Trace.entry ("UserActivityImpl.getPropertyGroup", name);
      if (serviceManager == null)
         throw new ServiceNotRegisteredException ();

      PropertyGroup  result = null;

      LocalActivityContext  context = (LocalActivityContext)
         this.threadContextManager.getCurrentContext (
         this.serviceManager.getServiceInformation ().getServiceName ());

      if (context == null)
         throw new NoActivityException ();

      result = context.getPropertyGroup (name);
      if (result == null)
         throw new PropertyGroupUnknownException (name);

      Trace.exit ("UserActivityImpl.getPropertyGroup", result);
      return result;
      }


   public void registerService (ServiceManager service)
      throws ServiceAlreadyRegisteredException, PropertyGroupUnknownException,
      SystemException
      {
      Trace.entry ("UserActivityImpl.registerService");
      try
         {
         if (service == null)
            throw new SystemException ("Service Manager null");

         if (serviceManager != null)
            throw new ServiceAlreadyRegisteredException (
               /*"registerService already performed"*/);

         ServiceInformation si = service.getServiceInformation ();
         if (si == null ||
             si.getServiceName () == null ||
             si.getContextGroup () == null )
            throw new SystemException (
               "Service Manager implementation incomplete");

         String serviceName = si.getServiceName ();
         // ... no this is ok as long as SAME service ... check contextGp?
         // if (serviceManagers.containsKey (serviceName))
         //    // throw new ServiceRegisteredException (
         //    throw new SystemException (
         //       /*"Service already registered"*/);

         // get the ThreadContextManager ...
         threadContextManager = ThreadContextManager.
            getThreadContextManager (si.getContextGroup ());

         // now deal with the PropertyGroups ...
         propertyGroupManagers.clear ();          // just in case
         String [] propertyGroupNames = service.getPropertyGroupNames ();
         if (propertyGroupNames != null)
            {
            for (int i = 0; i < propertyGroupNames.length; i++)
               {
               if (propertyGroupNames [i] != null)
                  {
                  propertyGroupManagers.put (propertyGroupNames [i],
                     service.getPropertyGroupManager (propertyGroupNames [i]));
                  }  // end if
               }  // end for

            }  // end if

         // complete the registration ... after all danger of exceptions passed
         serviceManagers.put (serviceName, service);
         serviceManager = service;
         }
      // getPropertyGroupManager no longer throws NoImplementException
      // catch (NoImplementException exc)
      //    {
      //    Trace.exception ("UserActivityImpl.registerService", exc);
      //    throw new SystemException (
      //       "Service Manager implementation incomplete");
      //    }
      finally
         {
         Trace.exit ("UserActivityImpl.registerService");
         }
      }


   public ServiceManager getService() throws SystemException
      {
      Trace.entry ("UserActivityImpl.getService");
      Trace.exit ("UserActivityImpl.getService", serviceManager);
      return serviceManager;
      }


   public ActivityCoordinator recreate(GlobalId activity, GlobalId parent, boolean resume, RecreateData recData)
     throws ServiceNotRegisteredException, ActivityCompletedException,
     IllegalArgumentException, SystemException
     {
     Trace.entry("UserActivityImpl.recreate");
     if (serviceManager == null)
             throw new ServiceNotRegisteredException();

     // if (timeout < -1)
     //    throw new TimeoutRangeException ();

     if ((parent != null) && (resume))
        throw new IllegalArgumentException();
     if(activity == null)
         throw new IllegalArgumentException();

     ActivityCoordinator ac = null;
     LocalActivityContext context = new LocalActivityContext (
        this.propertyGroupManagers,
        this.serviceManager,
        this.threadContextManager,
        activity);

     context.beginOffThread (recData, parent, resume);
     ac = context.getCoordinator();

     // Set up the stuff recovered from the SignalSet/ServiceManager
     SignalSet[] signalsets = serviceManager.recoverSignalSets (activity);
     // need to add SYSTEM signalsets!

     if (signalsets != null)
        {
        for (int i = 0; i < signalsets.length; i++)
           {
           try
              {
              RecoverableAction[] actions = serviceManager.recoverActions (activity, signalsets[i].getSignalSetName());
              if (actions != null)
                 {
                 for (int j = 0; j < actions.length; j++)
                    ac.addAction (
                       actions[j].getAction (),
                       signalsets[i].getSignalSetName (),
                       actions[j].getPriority ());
                 }  // end if
              }
           catch (IllegalStateException e2)
              {
              Trace.exception("UserActivityImpl.recreate", e2);
              throw e2;
              }
           catch (SignalSetUnknownException e2)
              {
              // Really shouldn't get here
              Trace.exception("UserActivityImpl.recreate", e2);
              throw new SystemException(e2);
              }

           }  // end for

        }  // end if

     // global actions and System SignalSets
     try
        {
        RecoverableAction[] actions = serviceManager.recoverActions (activity, null);
        if (actions != null)
           {
           for (int j = 0; j < actions.length; j++)
              ac.addGlobalAction (
                 actions[j].getAction(), actions[j].getPriority());
           }  // end if

        actions = serviceManager.recoverActions (activity, "org.omg.CosActivity.Synchronization");
        if (actions != null)
           {
           for (int j = 0; j < actions.length; j++)
              ac.addAction (
                 actions[j].getAction(),
                 "org.omg.CosActivity.Synchronization",
                 actions[j].getPriority());
           }  // end if

        actions = serviceManager.recoverActions (activity, "org.omg.CosActivity.ChildLifetime");
        if (actions != null)
           {
           for (int j = 0; j < actions.length; j++)
              ac.addAction (
                 actions[j].getAction(),
                 "org.omg.CosActivity.ChildLifetime",
                 actions[j].getPriority());
           }  // end if
        }
     catch (IllegalStateException e2)
        {
        Trace.exception("UserActivityImpl.recreate", e2);
        throw e2;
        }
     catch (SignalSetUnknownException e2)
        {
        // Really shouldn't get here
        Trace.exception("UserActivityImpl.recreate", e2);
        throw new SystemException(e2);
        }

     Trace.exit("UserActivityImpl.recreate");
     return ac;
     }


   public GlobalId[] recover () throws ServiceNotRegisteredException,
      SystemException
      {
      Trace.entry ("UserActivityImpl.recover");

      if (serviceManager == null)
          throw new ServiceNotRegisteredException();

      Trace.exit ("UserActivityImpl.recover");
      return null;
      }


   public void forget (GlobalId globalId) throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.entry ("UserActivityImpl.forget");

      if (serviceManager == null)
         throw new ServiceNotRegisteredException();

      Trace.exit ("UserActivityImpl.forget");
      }

   }  // end class UserActivityImpl
