package javax.activity;
import javax.activity.coordination.*;

/**
  * The ActivityCoordinator is responsible for broadcasting signals to       
  * registered {@link javax.activity.coordination.Action Actions}. 
  * It has no logic to understand the signals or the meaning
  * of Action outcomes, it simply acts as the messenger for the 
  * signals.  There is a single ActivityCoordinator instance per
  * Activity context.
  * <br>This objects <code>equals</code> method returns <i>true</i> if the parameter object represents
  * the same Activity as the target object.
  */                                                                          
public interface ActivityCoordinator
{
   /**
    * Establishes an interest relationship between the specified <code>Action</code> and <code>SignalSet</code>
    * for the Activity represented by the target <code>ActivityCoordinator</code>.
    * When a <code>Signal</code> which is a member of the <code>SignalSet</code> is sent, the             
    * <code>processSignal</code> method of the <code>Action</code> will be invoked with that <code>Signal</code>.
    * If multiple <code>Actions</code> are registered, then <code>priority</code>    
    * may be used to place an order on how they will be invoked when <code>Signals</code> are sent: higher       
    * priority <code>Actions</code> will occur first in the <code>Action</code> list, and hence be invoked before other,      
    * lower priority, <code>Actions</code>. The priority value must be a positive value; a value of zero means   
    * that the Activity Service implementation is free to place the <code>Action</code> at any point in the      
    * <code>Action</code> list.
    * <br>If the specified <code>Action</code> is registered multiple          
    * times for the same <code>SignalSet</code> then it will be invoked multiple times with the <code>Signals</code> from     
    * that <code>SignalSet</code>.                                                                               
    *
    * @param action the {@link javax.activity.coordination.Action action} to add an interest for.
    * @param signalSetName the name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    from which <code>Signals</code> are required by the <code>Action</code>
    * @param priority the priority order, with respect to other registered <code>Actions</code>,
    *     with which the specified <code>Action</code> is called by the <code>ActivityCoordinator</code>.
    *     <br>A value of zero means that the Activity Service implementation is free to
    *     place the <code>Action</code> at any point in the <code>Action</code> list.
    *
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> has begun completion or has completed.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void addAction(Action action, String signalSetName, int priority) throws
                     SignalSetUnknownException, SystemException, IllegalStateException;

   /**
    * Establishes an interest relationship between the specified <code>Action</code> and all <code>SignalSets</code>
    * used by the Activity represented by the target <code>ActivityCoordinator</code>.
    * When a <code>Signal</code> which is a member of any of the <code>SignalSets</code> is sent, the             
    * <code>processSignal</code> method of the <code>Action</code> will be invoked with that <code>Signal</code>.
    * If multiple <code>Actions</code> are registered, then <code>priority</code>    
    * may be used to place an order on how they will be invoked when <code>Signals</code> are sent: higher       
    * priority <code>Actions</code> will occur first in the <code>Action</code> list, and hence be invoked before other,      
    * lower priority, <code>Actions</code>. The priority value must be a positive value; a value of zero means   
    * that the Activity Service implementation is free to place the <code>Action</code> at any point in the      
    * <code>Action</code> list.
    * <br>If the specified <code>Action</code> is registered multiple          
    * times with the target <code>ActivityCoordinator</code> then it will be invoked multiple times with each <code>Signal</code>.
    *
    * @param action the {@link javax.activity.coordination.Action action} to add an interest for.
    * @param priority the priority order, with respect to other registered <code>Actions</code>,
    *     with which the specified <code>Action</code> is called by the <code>ActivityCoordinator</code>.
    *     <br>A value of zero means that the Activity Service implementation is free to
    *     place the <code>Action</code> at any point in the <code>Action</code> list.
    *
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> has begun completion or has completed.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void addGlobalAction(Action action, int priority) throws SystemException, IllegalStateException;

   /**
    * Removes the interest relationship between the specified <code>Action</code> and <code>SignalSet</code>
    * for the Activity represented by the target <code>ActivityCoordinator</code>. If 
    * <code>signalSetName</code> is an empty String then the interest relationship is
    * removed between the specified <code>Action</code> and all <code>SignalSets</code> in the target Activity.
    * This operation has no effect on any global actions.
    * <br>If the specified <code>Action</code> was registered multiple          
    * times with the target <code>ActivityCoordinator</code> then only a single
    * instance of this Action is removed by each call to this method.
    *
    * @param action the {@link javax.activity.coordination.Action action} to
    *    remove.
    * @param signalSetName the name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    from which <code>Signals</code> are no longer required by the <code>Action</code>. If
    *    <code>signalSetName</code> is an empty String then no <code>Signals</code> from any
    *    any <code>SignalSets</code> will be sent to the <code>Action</code>.
    *
    * @exception ActionNotFoundException Thrown if the specified <code>Action</code>
    *    is not recognized.
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> has begun completion or has completed.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void removeAction(Action action, String signalSetName) throws
                     ActionNotFoundException, SystemException, IllegalStateException;

   /**
    * Removes the interest relationship between the specified <code>Action</code> and all <code>SignalSets</code>
    * for the Activity represented by the target <code>ActivityCoordinator</code>. 
    * This operation has no effect on any non-global actions.
    * <br>If the specified <code>Action</code> was registered multiple          
    * times with the target <code>ActivityCoordinator</code> then only a single
    * instance of this Action is removed by each call to this method.
    *
    * @param action the {@link javax.activity.coordination.Action action} to
    *    remove.
    *
    * @exception ActionNotFoundException Thrown if the specified <code>Action</code>
    *    is not recognized.
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> has begun completion or has completed.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void removeGlobalAction(Action action) throws
                     ActionNotFoundException, SystemException, IllegalStateException;

   /**
    * Returns the number of Actions, including global Actions, registered with the target <code>ActivityCoordinator</code>
    * with an interest in the specified <code>SignalSet</code>.
    * 
    * @return The number of Actions registered to receive signals from the specified
    *    <code>SignalSet</code>.
    *
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    */
   public int getNumberRegisteredActions(String signalSetName) throws
      SignalSetUnknownException, SystemException;

   /**
    * Sets the name of the {@link javax.activity.coordination.SignalSet SignalSet}
    * that should be used for the distribution of completion signals
    * when the current Activity completes.
    * <br>This method can be called multiple times during the lifetime of an Activity,
    * before the Activity starts completion processing. The last value specified before
    * completion processing starts is used during completion processing. 
    * 
    * @param signalSetName the name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    that will be used for the distribution of completion signals
    *    when the current Activity completes.
    *
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> has begun completion or has completed.
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void setCompletionSignalSetName(String signalSetName) throws
      SignalSetUnknownException, SystemException, IllegalStateException;

   /**
    * Returns the name of the {@link javax.activity.coordination.SignalSet SignalSet},
    * if any, that will be used for the distribution of completion signals
    * when the current Activity completes.
    * 
    * @return The name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    that will be used for the distribution of completion signals
    *    when the current Activity completes.
    *    If no CompletionSignalSet has been set then <code>null</code> is returned.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    */
   public String getCompletionSignalSetName() throws SystemException;

   /**
    * Returns the parent <code>ActivityCoordinator</code>
    * or <code>null</code> if the target <code>ActivityCoordinator</code>
    * represents a top-level Activity.
    * 
    * @return The parent <code>ActivityCoordinator</code> 
    *         or <code>null</code> if the target <code>ActivityCoordinator</code>
    *         represents a top-level Activity.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityCoordinator getParent() throws SystemException;

   /**
    * Returns the {@link javax.activity.GlobalId GlobalId} of the
    * Activity represented by the target <code>ActivityCoordinator</code>.
    * 
    * @return The {@link javax.activity.GlobalId GlobalId} of the
    *         Activity represented by the target <code>ActivityCoordinator</code>.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public GlobalId getGlobalId() throws SystemException;

   /**
    * Returns the {@link javax.activity.Status Status}
    * of the Activity represented by the target <code>ActivityCoordinator</code>.
    * 
    * @return The {@link javax.activity.Status Status}
    *    of the Activity represented by the target <code>ActivityCoordinator</code>.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public int getStatus() throws SystemException;

   /**
    * Returns the {@link javax.activity.Status Status}
    * of the Activity represented by the target <code>ActivityCoordinator's</code>
    * parent. If the target <code>ActivityCoordinator</code> represents a top-level
    * Activity, then the <code>Status</code> of this top-level Activity is returned.
    * 
    * @return The {@link javax.activity.Status Status}
    *    of the target <code>ActivityCoordinator's</code> parent Activity or the <code>Status</code>
    *    of the target <code>ActivityCoordinator</code> if the target object represents
    *    a top-level Activity.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public int getParentStatus() throws SystemException;

   /**
    * Returns a printable string describing the Activity represented by the target
    * <code>ActivityCoordinator</code>.
    * 
    * @return A printable string describing the Activity represented by the target
    * <code>ActivityCoordinator</code>.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public String getName() throws SystemException;

   /**
    * Returns <code>true</code> if the specified <code>coord</code> represents
    * the same Activity as the target <code>ActivityCoordinator</code>.
    * 
    * @return <code>true</code> if the specified <code>coord</code> represents
    *         the same Activity as the target <code>ActivityCoordinator</code>. 
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public boolean isSameActivity(ActivityCoordinator coord) throws SystemException;

   /**
    * Returns all the <code>Actions</code> that have been registered with an interest in the specified
    * <code>signalSetName</code>. This method may be used by a HLS that has determined that
    * an Activity needs to be persisted and needs to log the <code>Actions</code> registered in the Activity.
    * 
    * @param signalSetName the name of a {@link javax.activity.coordination.SignalSet SignalSet}
    *    supported by the HLS. Only Actions registered with an interest in this
    *    <code>SignalSet</code> are returned by this operation. If this parameter is <i>null</i> then all
    *    <i>global</i> Actions are returned.
    *
    * @return The array of <code>Actions</code>
    *    registered with an interest in the specified <code>SignalSet</code>.
    *    If there are none, then a zero element array is returned. 
    *
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Action[] getActions(String signalSetName) throws SignalSetUnknownException, SystemException;


   /**
    * Causes the Activity associated with the target <code>ActivityCoordinator</code> to complete
    * with the specified {@link javax.activity.CompletionStatus CompletionStatus}.
    * The Activity service sets the {@link javax.activity.Status Status} to <i>StatusCompleting</i>
    * before asking any <i>completion</i> {@link javax.activity.coordination.SignalSet SignalSet} to start producing
    * <code>Signals</code>.
    * <br>If there are any child active or suspended Activities or
    * transactions and the CompletionStatus is CompletionStatusFail or
    * CompletionStatusFailOnly then those child Activites will have
    * their CompletionStatuses set to CompletionStatusFailOnly and any
    * child transaction will be called to setRollbackOnly.
    * <br>The {@link javax.activity.UserActivity UserActivity} interface should be used in preference
    * to the <code>ActivityCoordinator</code> for completing Activities in most circumstances.
    * This method is provided primarily for use after recovery when the Activity to be completed
    * is not associated with the calling thread.
    *
    * @param completionStatus the {@link javax.activity.CompletionStatus CompletionStatus}
    *     with which the target Activity should end. 
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the SignalSet used for completion.
    *    In the absence of a CompletionSignalSet a null <code>Outcome</code> object
    *    reference is returned.
    *
    * @exception ActivityPendingException Thrown if the thread from which the
    *    completion is initiated is not the only thread associated with the
    *    target Activity. The application response should be to try again
    *    later when any asynchronous work on other threads has been suspended.
    *
    * @exception ContextPendingException Thrown if the CompletionStatus is
    *    CompletionStatusSuccess and the target Activity encompasses
    *    active or suspended Activities or transactions. The application
    *    response should be to complete the encompassed Activities and
    *    transactions and then complete the target Activity or to force the
    *    target Activity to end by setting the CompletionStatus to
    *    CompletionStatusFail and then ending it.
    *
    * @exception NotOriginatorException Thrown if an attempt is made by
    *    an application to end an Activity in a different execution
    *    environment from that in which the Activity was begun.
    *
    * @exception InvalidStateException Thrown if the target Activity cannot
    *    be completed with the requested <code>CompletionStatus</code>, for example
    *    because the <code>CompletionStatus</code> may not be changed from <code>CompletionStatusFailOnly</code>
    *    to any other value, or if a <code>null</code>
    *    or invalid value is specified by <code>completionStatus</code>.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced. The Activity's final
    *    <code>Status</code> is <code>StatusError</code>.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome completeActivity(int completionStatus) throws ActivityPendingException, ContextPendingException,
                            NotOriginatorException, InvalidStateException, ActivityNotProcessedException, SystemException;

   /**
    * Causes the Activity associated with the target <code>ActivityCoordinator</code> to complete
    * with the specified {@link javax.activity.CompletionStatus CompletionStatus}, but in a manner
    * that acknowledges that the completion request has been driven heuristically. 
    * The Activity service sets the {@link javax.activity.Status Status} to <i>StatusCompletingHeuristic</i>
    * before asking any <i>completion</i> {@link javax.activity.coordination.SignalSet SignalSet} to start producing
    * <code>Signals</code>.
    * <br>This method may be used in an environment that is different from that in which the Activity
    * to be completed was started. It is otherwise identical to 
    * {@link #completeActivity(int) completeActivity}.
    *
    * @param completionStatus the {@link javax.activity.CompletionStatus CompletionStatus}
    *     with which the target Activity should end. 
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the SignalSet used for completion.
    *    In the absence of a CompletionSignalSet a null <code>Outcome</code> object
    *    reference is returned.
    *
    * @exception ActivityPendingException Thrown if the thread from which the
    *    completion is initiated is not the only thread associated with the
    *    target Activity. The application response should be to try again
    *    later when any asynchronous work on other threads has been suspended.
    *
    * @exception ContextPendingException Thrown if the CompletionStatus is
    *    CompletionStatusSuccess and the target Activity encompasses
    *    active or suspended Activities or transactions. The application
    *    response should be to complete the encompassed Activities and
    *    transactions and then complete the target Activity or to force the
    *    target Activity to end by setting the CompletionStatus to
    *    CompletionStatusFail and then ending it.
    *
    * @exception InvalidStateException Thrown if the target Activity cannot
    *    be completed with the requested <code>CompletionStatus</code>, for example
    *    because the <code>CompletionStatus</code> may not be changed from <code>CompletionStatusFailOnly</code>
    *    to any other value, or if a <code>null</code>
    *    or invalid value is specified by <code>completionStatus</code>.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced. The Activity's final
    *    <code>Status</code> is <code>StatusError</code>.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome heuristicComplete(int completionStatus) throws ActivityPendingException, ContextPendingException,
                            InvalidStateException, ActivityNotProcessedException, SystemException;


   /**
    * Causes the {@link javax.activity.coordination.SignalSet SignalSet} specified by <code>signalSetName</code>
    * to start producing signals for all registered {@link javax.activity.coordination.Action Actions}
    * at times other than during completion. 
    * Pre-defined <code>SignalSets</code>, such as <code>Synchronization</code>, cannot be requested
    * to produce signals via this method.
    *
    * @param signalSetName the name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    that is to produce signals.
    *
    * @param completionStatus the {@link javax.activity.CompletionStatus CompletionStatus}
    *     of the Activity at the time of calling. The <code>completionStatus</code> may influence
    *     the signals that are produced.
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the specified <code>SignalSet</code>. It may be null.
    *
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced. The Activity's final
    *    <code>Status</code> is <code>StatusError</code>.
    *
    * @exception InvalidActivityException Thrown if the SignalSet is already producing signals
    *    or if the Activity has completed. 
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome processSignalSet(String signalSetName, int completionStatus) throws SignalSetUnknownException,
                                         ActivityNotProcessedException, InvalidActivityException, SystemException ;

   /**
    * Enables the Activity service to evict the target Activity from active memory.
    * This method may be driven by an HLS if it does not expect further work
    * to occur on behalf of the Activity in the immediate future.
    * On completion of this method the Activity is not associated with the thread and the
    * target ActivityCoordinator instance is no longer valid.
    * The HLS may call this method at any time prior to completion of an Activity.
    * When the HLS determines that the Activity needs to be brought back into memory it calls
    * {@link javax.activity.UserActivity#recreate(javax.activity.GlobalId, javax.activity.GlobalId, boolean, RecreateData) UserActivity.recreate}.
    * An Activity hierarchy that contains a transaction cannot be hibernated.
    * <br>Any active child Activities must be hibernated before the target Activity may be hibernated.
    * 
    * @return a {@link javax.activity.RecreateData RecreateData} object that
    * is opaque to the HLS. This may be null. This object should be passed on any future
    * call to <code>recreate</code> the Activity.
    *
    *
    * @exception ActivityPendingException Thrown if the target Activity 
    *    is associated with a thread other than the one on which the request is made.
    *    The HLS response should be to try again
    *    later when any asynchronous work on other threads has been suspended.
    *
    * @exception ContextPendingException Thrown if the target Activity encompasses
    *    a child Activity that is not already hibernated or if the target Activity
    *    encompasses a transaction. The HLS response
    *    should be to complete any encompassed transaction and either complete or
    *    hibernate any encompassed child Activities.
    *
    * @exception IllegalStateException Thrown if the Activity represented by the 
    *    target <code>ActivityCoordinator</code> is not in a state where it can be hibernated.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public RecreateData hibernate() throws ActivityPendingException, ContextPendingException,
                                                                          IllegalStateException, SystemException;

   /**
    * Indicates to the Activity service that the HLS is about to persist its
    * recoverable state and that the Activity must be recoverable
    * after this point. The <code>ActivityCoordinator</code> returns a serializable
    * {@link javax.activity.RecreateData RecreateData}
    * object that is opaque to the HLS and that the HLS must pass
    * on any future call to {@link javax.activity.UserActivity#recreate(GlobalId, GlobalId, boolean, RecreateData) recreate}
    * the Activity.
    * 
    * <br>The point, if any, at which an Activity needs to become recoverable is
    * determined exclusively by the HLS that is using the Activity service.
    *
    * @return a {@link javax.activity.RecreateData RecreateData} object that
    * is opaque to the HLS. This may be null. This object should be passed on any future
    * call to <code>recreate</code> the Activity.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public RecreateData setPersistent() throws SystemException;

}
