package javax.activity;

/**
 * The <code>ActivityManager</code> interface extends the <code>UserActivity</code>
 * interface by offering methods to manage the association of Activity context
 * with the thread of execution.
 * <br> An instance of the <code>ActivityManager</code> interface is a specialization
 * of the <code>UserActivity</code> instance and requires
 * a {@link javax.activity.coordination.ServiceManager ServiceManager} for a
 * specific HLS to be registered with it prior to use.
 * <br> Methods of the <code>ActivityManager</code> interface that operate
 * on the <i>active</i>, or <i>current</i>, Activity are operating on the HLS
 * Activity instance most recently associated with the calling thread.
 * <br> The <code>suspend</code> and <code>resume</code> operations of the
 * <code>ActivityManager</code> interface use an 
 * {@link javax.activity.ActivityToken ActivityToken} to represent the
 * hierarchy of Activities and transactions affected by the operation.
 * A hierarchy relationship may be established on a thread between Activities
 * or between an  Activity and a transaction when an Activity is started in the presence of
 * another Activity or transaction already associated with the thread.
 * <br> This interface is available to the EJB container in which the Activity
 * HLS is configured. It is not intended to be used directly by the HLS without
 * container mediation.
 * <br>The UserManager interface is available to the EJB and web containers via a JNDI
 * <b>lookup</b> of <code>services:activity/ActivityManager</code>. 
 */
public interface ActivityManager extends UserActivity
{

   /**
    * Suspends the association of the current Activity from the
    * calling thread of execution along with any child Activities of the same
    * ContextGroup nested within that Activity.
    * <br>If the suspended Activity is nested within a parent then the parent
    * Activity becomes associated with the thread on completion of this method.
    * 
    * @return An {@link javax.activity.ActivityToken ActivityToken} representing
    * the suspended Activity. This ActivityToken maintains
    * knowledge of the nested Activities and transactions that were suspended when the target
    * Activity was suspended as well as any parent Activity that was
    * established as a result of the suspend operation.
    * <br>The ActivityToken may be used on a subsequent call to 
    * {@link #resume(ActivityToken) resume} but not to
    * {@link #resumeAll(ActivityToken) resumeAll} or
    * {@link #resumeGroup(ActivityToken) resumeGroup}.
    * <br>If the calling thread is not associated with any Activity then <code>null</code>
    * is returned.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityToken suspend() throws ServiceNotRegisteredException, SystemException;

   /**
    * Resumes the association of the Activity, and any nested Activities and transactions,
    * represented by the <code>ActivityToken</code> with the calling
    * thread of execution. The <code>ActivityToken</code> represents a hierarchy
    * of one of more Activities and transactions and must have been obtained
    * by a prior call to {@link #suspend() suspend}.
    * If there is an Activity associated with the thread prior
    * to the method invocation then the re-associated Activity is resumed as a
    * child of the Activity that is already on the thread. Any such parent Activity must
    * be the same parent Activity 
    * within which the child Activity was originally begun. 
    * <br>If the specified ActivityToken is <code>null</code>, then no new
    * associated is made.
    * 
    * @param activity the <code>ActivityToken</code> to resume.
    *    The <code>ActivityToken</code> represents a hierarchy
    *    of one of more Activities and transactions and must have been obtained
    *    by a prior call to {@link #suspend() suspend}.
    *    
    * @exception InvalidActivityException May be thrown if the specified
    *    ActivityToken was not obtained by a prior call to {@link #suspend() suspend}
    *    or if the ActivityToken does not represent a valid Activity or Activity hierarchy.
    *    No new association is made.
    *
    * @exception InvalidParentContextException Thrown if the parent Activity 
    *    is different from that in which the resumed Activity was begun.
    *    No new association is made.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void resume(ActivityToken activity) throws InvalidActivityException,
                    InvalidParentContextException, ServiceNotRegisteredException, SystemException;

   /**
    * Suspends the association of the current Activity  from the
    * calling thread of execution along with any Activities
    * belonging to the same <code>ContextGroup</code>. Transaction context
    * is also suspended if the Activity belongs to the default <code>ContextGroup</code>.
    * The <code>ContextGroup</code>
    * is an attribute of the {@link javax.activity.coordination.ServiceManager ServiceManager}
    * for the target ActivityManager instance. 
    * <br>On completion of this method, no Activity that is part of the same
    * ContextGroup as the suspended Activity remains associated with the
    * thread.
    * 
    * @return An {@link javax.activity.ActivityToken ActivityToken} representing
    * the suspended ContextGroup hierarchy of Activities and transactions.
    * <br>The ActivityToken may be used on a subsequent call to 
    * {@link #resumeGroup(ActivityToken) resumeGroup} but not to
    * {@link #resume(ActivityToken) resume} or
    * {@link #resumeAll(ActivityToken) resumeAll}.
    * <br>If the calling thread is not associated with any Activity then <code>null</code>
    * is returned.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityToken suspendGroup() throws ServiceNotRegisteredException, SystemException;

   /**
    * Resumes the association of the hierarchy of Activities and transactions 
    * represented by the <code>ActivityToken</code> with the calling
    * thread of execution. The <code>ActivityToken</code> represents a hierarchy
    * of Activities in a single <code>ContextGroup</code> and must have been obtained
    * by a prior call to {@link #suspendGroup() suspendGroup}.
    * There must be no Activity
    * belonging to this <code>ContextGroup</code> active on the thread at the
    * initiation of this operation.
    * <br>If the specified ActivityToken is <code>null</code>, then no new
    * associated is made.
    * 
    * @param activity the <code>ActivityToken</code> to resume.
    *    The <code>ActivityToken</code> represents a hierarchy
    *    of one of more Activities and transactions and must have been obtained
    *    by a prior call to {@link #suspendGroup() suspendGroup}.
    *    
    * @exception InvalidActivityException May be thrown if the specified
    *    ActivityToken was not obtained by a prior call to {@link #suspendGroup() suspendGroup}
    *    or if the ActivityToken does not represent a valid Activity or Activity hierarchy.
    *    No new association is made.
    *
    * @exception InvalidParentContextException Thrown if an Activity belonging to the
    *    same ContextGroup as the resuming Activities is already associated with
    *    the thread of execution.
    *    No new association is made.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void resumeGroup(ActivityToken activity) throws InvalidActivityException,
                    InvalidParentContextException, ServiceNotRegisteredException, SystemException;

   /**
    * Suspends the association of all transactions and Activities from the
    * calling thread of execution.
    * On completion of this method, no transaction or Activity
    * remain associated with the thread.
    * 
    * @return An {@link javax.activity.ActivityToken ActivityToken} representing
    * the suspended hierarchy of Activities and transactions.
    * <br>The ActivityToken may be used on a subsequent call to 
    * {@link #resumeAll(ActivityToken) resumeAll} but not to
    * {@link #resume(ActivityToken) resume} or
    * {@link #resumeGroup(ActivityToken) resumeGroup}.
    * <br>If the calling thread is not associated with any Activity then <code>null</code>
    * is returned.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityToken suspendAll() throws ServiceNotRegisteredException, SystemException;

   /**
    * Resumes the association of all transactions and Activities
    * represented by the <code>ActivityToken</code> with the calling
    * thread of execution. The <code>ActivityToken</code> represents a hierarchy
    * of Activities and transactions and must have been obtained
    * by a prior call to {@link #suspendAll() suspendAll}.
    * There must be no transaction or Activity active on the
    * thread at the initiation of this operation.
    * <br>If the specified ActivityToken is <code>null</code>, then no new
    * associated is made.
    * 
    * @param activity the <code>ActivityToken</code> to resume.
    *    The <code>ActivityToken</code> represents a hierarchy
    *    of one of more Activites and transactions and must have been obtained
    *    by a prior call to {@link #suspendAll() suspendAll}.
    *    
    * @exception InvalidActivityException May be thrown if the specified
    *    ActivityToken was not obtained by a prior call to {@link #suspendAll() suspendAll}
    *    or if the ActivityToken does not represent a valid hierarchy of transactions and
    *    Activities.
    *    No new association is made.
    *
    * @exception InvalidParentContextException Thrown if a transaction or Activity
    *    is already associated with the thread of execution.
    *    No new association is made.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void resumeAll(ActivityToken activity) throws InvalidActivityException,
                    InvalidParentContextException, ServiceNotRegisteredException, SystemException;
}
