package javax.activity;

/**
 * A CoordinationInformation object is returned from the <code>SignalSet's</code> 
 * {@link javax.activity.coordination.SignalSet#setResponse setResponse}
 * method. It is used by the 
 * {@link javax.activity.ActivityCoordinator ActivityCoordinator}
 * to determine how next to proceed during signaling after a 
 * response from a particular <code>Action</code> has been processed by the <code>SignalSet</code>.
 */
public class CoordinationInformation
{
   /**
    * Constructs a <code>CoordinationInformation</code> object to indicate to the calling <code>ActivityCoordinator</code>
    * how to proceed with signaling. This object is created by a </code>SignalSet</code>
    * during the processing of a 
    * {@link javax.activity.coordination.SignalSet#setResponse setResponse}
    * operation. 
    *
    * @param isActionInterested indicates to the calling <code>ActivityCoordinator</code>
    *        whether the <code>Action</code>, whose <code>Outcome</code> is being processed by 
    *        {@link javax.activity.coordination.SignalSet#setResponse setResponse},
    *        should receive any further signals during the current broadcast.
    *        A value of <b>true</b> indicates that the <code>Action</code> should continue
    *        to receive signals. A value of <b>false</b> indicates that the Action is in a
    *        state where is expects no further signals.
    * @param useNextSignal indicates to the calling <code>ActivityCoordinator</code>
    *        that it should request the next <code>Signal</code> from the <code>SignalSet</code>
    *        even if the current <code>Signal</code> has not yet been distributed to all
    *        registered <code>Actions</code>.
    *        A value of <b>true</b> indicates that the next <code>Signal</code> should be
    *        requested and distributed to registered <code>Actions</code>. A value of
    *        <b>false</b> indicates that the current <code>Signal</code> should continue
    *        to be distributed.
    *
    */
   public CoordinationInformation(boolean isActionInterested, boolean useNextSignal)
   {
      _isActionInterested = isActionInterested;
      _useNextSignal = useNextSignal;
   }

   /**
    * Returns a boolean to indicate to the calling <code>ActivityCoordinator</code>
    * whether a particular <code>Action</code> should receive any further signals
    * during the lifetime of the current Activity.
    * The specific <code>Action</code> affected is the <code>Action</code> instance
    * whose <code>Outcome</code> was processed by the
    * the call to {@link javax.activity.coordination.SignalSet#setResponse setResponse}
    * in which the target <code>CoordinationInformation</code> object was returned.
    * A value of <b>true</b> indicates that the <code>Action</code> should continue
    * to receive signals. A value of <b>false</b> indicates that the Action is in a
    * state where is expects no further signals during the current Activity.
    *
    * @return <dl><dt>true<dd> if the Action whose <code>Outcome</code> has been processed
    *         may receive further signals during the current broadcast.
    *         <dt>false<dd> if the Action whose <code>Outcome</code> has been processed
    *         is in a state where it expects no further signals during the current Activity.
    */
   public boolean isActionInterested()
   {
      return _isActionInterested;
   }

   /**
    * Returns a boolean to indicates to the calling <code>ActivityCoordinator</code>                 
    * whether it should request the next <code>Signal</code> from the <code>SignalSet</code>     
    * even if the current <code>Signal</code> has not yet been distributed to all             
    * registered <code>Actions</code>.                                                        
    *
    * @return <dl><dt>true<dd> if the the next <code>Signal</code> should be                        
    *         requested and distributed to registered <code>Actions</code>. 
    *         <dt>false<dd> if the current <code>Signal</code> should continue       
    *         to be distributed.                                    
    */
   public boolean useNextSignal()
   {
      return _useNextSignal;
   }

   // Private attributes
   private boolean _isActionInterested = false;
   private boolean _useNextSignal = false;

}
