package javax.activity;

/**
  * An <code>Outcome</code> is produced by, and given meaning by, a
  * {@link javax.activity.coordination.SignalSet SignalSet} when it has finished
  * producing signals. A <code>CompletionSignalSet</code> produces such an
  * <code>Outcome</code> and this is returned on the <code>complete()</code> and
  * <code>completeWithStatus(CompletionStatus)</code> methods of the
  * {@link javax.activity.UserActivity UserActivity} interface.
  * <p>The Activity service itself provides implementations of the following
  * pre-defined Outcomes:
  * <dl>
  * <dt><code><b>ActionError</b></code>
  * <dd>The <code>ActionError</code> Outcome is produced by the <code>ActivityCoordinator</code>
  * as the response to a <code>Signal</code> when the <code>Action</code> that was sent the <code>Signal</code> throws an
  * <code>ActionErrorException</code>.
  * <dt><code><b>ActionSystemException</b></code>
  * <dd>The <code>ActionSystemException</code> Outcome is produced by the <code>ActivityCoordinator</code>
  * as the response to a <code>Signal</code> when the <code>Action</code> that was sent the <code>Signal</code>,
  * or the communications subsystem, throws a system exception. The system exception is encoded
  * in the Outcome's specific data and may be retrieved through {@link #getExtendedValue() getExtendedValue}.
  * </dl>
  * 
  */                                                                          
public class Outcome
{
   /**
    * Constructor used by a HLS to create an <code>Outcome</code> object.
    * The Activity service encapsulates the Serializable <code>extOutcome</code> in a 
    * <code>java.rmi.MarshalledObject</code> to avoid class-loading problems when propagating 
    * <code>CosActivity.Outcome</code> structures containing this object.
    *
    * @param outcomeName The name of the <code>Outcome</code>.
    *
    * @param extOutcome Data that is specific to a particular <code>Outcome</code>.
    *        This parameter may be <code>null</code>.
    *
    */
   public Outcome(String outcomeName, java.io.Serializable extOutcome )
   {
      _outcome = outcomeName;
      _extendedOutcome = extOutcome; 
   }

   /**
    * Constructor used by the Activity service to create an <code>Outcome</code> object.
    * It is used by the Activity service when processing a received <code>CosActivity.Outcome</code>
    * if the <code>CosActivity.Outcome.application_specific_data</code> field of that <code>Outcome</code>
    * contains a <code>MarshalledObject</code>.
    *
    * @param outcomeName The name of the <code>Outcome</code>.
    *
    * @param extOutcome Data that is specific to a particular <code>Outcome</code>.
    *        This parameter may be <code>null</code>.
    *
    */
   public Outcome(String outcomeName, java.rmi.MarshalledObject extOutcome )
   {
      _outcome = outcomeName;
      _propagatedExtendedOutcome = extOutcome; 
   }

   /**
    * Constructor used by the Activity service or HLS to create an <code>Outcome</code> object.
    * <br>It is used by the Activity service when processing a received <code>CosActivity.Outcome</code>
    * if the <code>CosActivity.Outcome.application_specific_data</code> field of that <code>Outcome</code>
    * does not contain a <code>MarshalledObject</code>.
    * <br>It may also used by a HLS to create an <code>Outcome</code>.
    *
    * @param outcomeName The name of the <code>Outcome</code>.
    *
    * @param extData Data that is specific to a particular
    *        <code>Outcome</code>.
    *
    */
   public Outcome(String outcomeName, org.omg.CORBA.Any extOutcome )
   {
      _outcome = outcomeName;
      _extendedOutcomeAny = extOutcome;
   }

   /**
    * Returns a <code>String</code> that identifies the <code>Outcome</code> name.
    * 
    * @return The <code>Outcome</code> name.
    *
    */
   public String getName()
   {
      return _outcome;
   }

   /**
    * Returns a <code>java.io.Serializable</code> that contains information specific to the <code>Outcome</code>.
    * If this extended data is not a <code>Serializable</code> object then
    * this method returns <code>null</code> and {@link #getExtendedAny() getExtendedAny}
    * should be called instead.
    * 
    * @return A java.io.Serializable that contains information specific to the <code>Outcome</code>.
    *   Returns null if the extended data is not a <code>Serializable</code> object
    *         or if there is no extended data.
    */
   public java.io.Serializable getExtendedValue()
   {
      if (_extendedOutcome == null)
      {
        // if a propagated java.io.Serializable is available, extract it
        // now from the MarshalledObject
        if (_propagatedExtendedOutcome != null)
        { 
          try
          {
            // can throw IOException and ClassNotFoundException
            _extendedOutcome = (java.io.Serializable)_propagatedExtendedOutcome.get();
          }
          catch (Exception e)
          {
            _extendedOutcome = null;
          }
        }

      }
      return _extendedOutcome;
   }

   /**
    * Returns a CORBA <code>Any</code> that contains information specific to the <code>Outcome</code>.
    * 
    * @return A CORBA <code>Any</code> that contains information specific to the <code>Outcome</code>.
    *         <br> Returns null if there is no extended data.
    */
   public org.omg.CORBA.Any getExtendedAny()
   {
      return _extendedOutcomeAny;
   }


   // Private attributes
   private String _outcome = null;
   private java.io.Serializable _extendedOutcome = null;
   private org.omg.CORBA.Any _extendedOutcomeAny = null;
   private java.rmi.MarshalledObject _propagatedExtendedOutcome = null;
}
