package javax.activity;

/**
  * The PropertyGroupContext utility object is provided by the Activity service 
  * to assist a {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}
  * read and write <code>org.omg.CosActivity.PropertyGroupIdentity</code> context data
  * during marshalling and unmarshalling of the
  * <code>org.omg.CosActivity.ActivityContext</code> that incorporates it.
  */                                                                          
public class PropertyGroupContext
{
   /**
    * Constructor used by a
    * {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}
    * to hold the contents of the
    * <code>org.omg.CosActivity.PropertyGroupIdentity</code> structure.
    * <br>It is used by a <code>PropertyGroupManager</code> when it is
    * marshaling an outbound request or response context in the case where the
    * <code>contextData</code> is a <code>java.io.Serializable</code> object.
    * The Activity service encapsulates the Serializable <code>contextData</code> in a 
    * <code>java.rmi.MarshalledObject</code> to avoid class-loading problems when propagating 
    * <code>PropertyGroupIdentity</code> structures containing this object.
    *
    * @param propertyGroupName The name of the PropertyGroup.
    *
    * @param contextData Data that is specific to a particular
    *        <code>PropertyGroupManager</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public PropertyGroupContext(String propertyGroupName, java.io.Serializable contextData)
   {
      _pgName = propertyGroupName;
      _pgContext = contextData;
   }

   /**
    * Constructor used by the Activity service to hold the contents of a
    * <code>org.omg.CosActivity.PropertyGroupIdentity</code> structure.
    * <br>It is used by the Activity service when importing an Activity service context if
    * the <code>PropertyGroupIdentity.context_data</code> of that
    * context contains a <code>MarshalledObject</code>.
    *
    * @param propertyGroupName The name of the PropertyGroup.
    *
    * @param contextData Data that is specific to a particular
    *        <code>PropertyGroupManager</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public PropertyGroupContext(String propertyGroupName, java.rmi.MarshalledObject contextData)
   {
      _pgName = propertyGroupName;
      _propagatedPGContext = contextData;
   }

   /**
    * Constructor used by the Activity service or a 
    * {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}
    * to hold the contents of the
    * <code>org.omg.CosActivity.PropertyGroupIdentity</code> structure.
    * <br>It is used by the Activity service when importing an Activity service context if
    * the <code>PropertyGroupIdentity.context_data</code> of that
    * context does not contain a <code>MarshalledObject</code>.
    * <br>It may also be used by a <code>PropertyGroupManager</code> when it is
    * marshaling an outbound request or response context in the case where the
    * <code>contextData</code> is not a <code>java.io.Serializable</code> object.
    *
    * @param propertyGroupName The name of the PropertyGroup.
    *
    * @param contextData Data that is specific to a particular
    *        <code>PropertyGroupManager</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public PropertyGroupContext(String propertyGroupName, org.omg.CORBA.Any contextData)
   {
      _pgName = propertyGroupName;
      _pgContextAny = contextData;
   }


   /**
    * Returns a <code>String</code> that identifies the <code>PropertyGroup</code> name.
    *
    * @return The name of the PropertyGroup that has been received from or is to be inserted
    *         into the <code>CosActivity.PropertyGroupIdentity.property_group_name</code>
    *         field of the Activity context.
    *
    */
   public String getPropertyGroupName()
   {
      return _pgName;
   }

   /**
    * Returns a CORBA <code>Any</code> that contains context data specific
    * to the implementation of the
    * {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}.
    *
    * @return A CORBA Any that contains the context data that has been
    *         received from or is to be inserted
    *         into the <code>org.omg.CosActivity.PropertyGroupIdentity.context_data</code>
    *         field of the Activity context. Only the PropertyGroupManager has the
    *         knowledge required to understand the structure of the context data.
    *         <br> Returns null if there is no <code>context data</code>.
    *
    */
   public org.omg.CORBA.Any getContextDataAny()
   {
      return _pgContextAny;
   }

   /**
    * Returns a <code>java.io.Serializable</code> that contains context data specific
    * to the implementation of the
    * {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}.
    * If the <code>context data</code> is not a <code>Serializable</code> object then
    * this method returns <code>null</code> and {@link #getContextDataAny() getContextDataAny}
    * should be called instead.
    *
    * @return A <code>java.io.Serializable</code> object that represents the context data that has been
    *          that has been received from or is to be inserted into
    *         the <code>org.omg.CosActivity.PropertyGroupIdentity.context_data</code>
    *         field of the Activity context. Only the PropertyGroupManager has the
    *         knowledge required to understand the structure of the contextData.
    *         <br> Returns null if the
    *         <code>context data</code> is not a <code>Serializable</code> object
    *         or if there is no <code>context data</code>.
    *
    */
   public java.io.Serializable getContextDataValue()
   {
      if (_pgContext == null)
      {
        // if a propagated java.io.Serializable is available, extract it
        // now from the MarshalledObject
        if (_propagatedPGContext != null)
        { 
          try
          {
            // can throw IOException and ClassNotFoundException
            _pgContext = (java.io.Serializable)_propagatedPGContext.get();
          }
          catch (Exception e)
          {
            _pgContext = null;
          }
        }
      }
      return _pgContext;
   }

   // Private attributes
   private String _pgName = null;
   private java.io.Serializable _pgContext = null;
   private org.omg.CORBA.Any _pgContextAny = null;
   private java.rmi.MarshalledObject _propagatedPGContext = null;

}
