package javax.activity;

/**
  * An instance of a ServiceInformation object is used by each 
  * {@link javax.activity.coordination.ServiceManager ServiceManager}
  * to identify the name of the high-level service (HLS) and the context group to which
  * a particular Activity belongs. This information is propagated as part of the
  * <code>org.omg.CosActivity.ActivityIdentity</code> structure of the Activity
  * service context, in the <code>activity_specific_data</code> field,
  * when the <code>type</code> field of the <code>ActivityIdentity</code>
  * is set to <code>0x4A324545</code> to indicate a J2EE Activity.
  * <br>The <code>activity_specific_data</code> field is of type <code>org.omg.CORBA.Any</code>
  * and, for a J2EE Activity type, contains a <code>TypeCode</code> with a <code>TCKind</code> 
  * of <code>_tk_struct</code> and a value of a <code>j2ee_activity_specific_information</code>
  * structure which is defined in IDL as follows:
  * <pre>
  *  module javax
  *  {
  *    module activity
  *    {
  *      struct j2ee_activity_specific_data
  *      {
  *        string service_name;
  *        string context_group;
  *        any service_specific_data;
  *        any extended_data;
  *      };
  *    };
  *  };
  * </pre>
  * 
  */                                                                          
public class ServiceInformation implements java.io.Serializable
{
   /**
    * Constructor used by a HLS to create a <code>ServiceInformation</code>
    * object to hold the contents of a
    * <code>org.omg.CosActivity.ActivityIdentity.activity_specific_data</code>.
    * <br>This constructor is used by a HLS in order to provide this data to the 
    * Activity service to be propagated as part of an outbound Activity service context.
    * The Activity service encapsulates the Serializable <code>serviceSpecificData</code> in a 
    * <code>java.rmi.MarshalledObject</code> to avoid class-loading problems when propagating 
    * <code>ActivityIdentity</code> structures containing this object.
    *
    * @param serviceName The name used to identify the high-level-service (HLS)
    *        to which an Activity belongs. This name is the URL to which the
    *        associated
    *        {@link javax.activity.coordination.ServiceManager ServiceManager}
    *        is bound in JNDI.
    *
    * @param contextGroup The name of the context-group to which all Activities
    *        of the HLS belong. 
    *
    * @param serviceSpecificData Data that is specific to a particular
    *        HLS implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public ServiceInformation(String serviceName, 
                             String contextGroup, 
                             java.io.Serializable serviceSpecificData)
   {
      _serviceName = serviceName;
      _contextGroup = contextGroup;
      _serviceSpecificData = serviceSpecificData;
   }

   /**
    * Constructor used by the Activity service to create a <code>ServiceInformation</code>
    * object to hold the contents of a
    * <code>org.omg.CosActivity.ActivityIdentity.activity_specific_data</code>.
    * <br>It is used by the Activity service when importing an Activity service context if
    * the <code>j2ee_activity_specific_data.service_specific_data</code> field of that
    * context contains a <code>MarshalledObject</code>.
    *
    * @param serviceName The name used to identify the high-level-service (HLS)
    *        to which an Activity belongs. This name is the URL to which the
    *        associated
    *        {@link javax.activity.coordination.ServiceManager ServiceManager}
    *        is bound in JNDI.
    *
    * @param contextGroup The name of the context-group to which all Activities
    *        of the HLS belong. 
    *
    * @param serviceSpecificData Data that is specific 
    *        to a particular HLS implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public ServiceInformation(String serviceName, 
                             String contextGroup, 
                             java.rmi.MarshalledObject serviceSpecificData)
   {
      _serviceName = serviceName;
      _contextGroup = contextGroup;
      _propagatedServiceSpecificData = serviceSpecificData;
   }

   /**
    * Constructor used by the Activity service or HLS to create a <code>ServiceInformation</code>
    * object to hold the contents of a
    * <code>org.omg.CosActivity.ActivityIdentity.activity_specific_data</code>.
    * <br>It is used by the Activity service when importing an Activity service context if
    * the <code>j2ee_activity_specific_data.service_specific_data</code> field of that
    * context does not contain a <code>MarshalledObject</code>.
    * <br>It is also used by a HLS in order to provide this data to the 
    * Activity service to be propagated as part of an outbound Activity service context.
    *
    * @param serviceName The name used to identify the high-level-service (HLS)
    *        to which an Activity belongs. This name is the URL to which the
    *        associated
    *        {@link javax.activity.coordination.ServiceManager ServiceManager}
    *        is bound in JNDI.
    *
    * @param contextGroup The name of the context-group to which all Activities
    *        of the HLS belong. 
    *
    * @param serviceSpecificData Data that is specific to a particular
    *        HLS implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public ServiceInformation(String serviceName, String contextGroup,
                             org.omg.CORBA.Any serviceSpecificData)
   {
      _serviceName = serviceName;
      _contextGroup = contextGroup;
      _serviceSpecificDataAny = serviceSpecificData;
   }


   /**
    * Returns a <code>String</code> that identifies the high-level-service (HLS) name.
    * This name is the URL to which the associated
    * {@link javax.activity.coordination.ServiceManager ServiceManager}
    * is bound in JNDI.
    * 
    * @return The JNDI name of the associated <code>ServiceManager</code>.
    *
    */
   public String getServiceName()
   {
      return _serviceName;
   }

   /**
    * Returns the name of the context group to which all the
    * <code>ServiceManager's</code> activities belong.
    * This name is a String that different <code>ServiceManagers</code> use to identify
    * a group to which their Activity contexts belong. By convention, a <code>ServiceManager</code> 
    * that does not intend its context to be managed cooperatively with
    * transactions or another <code>ServiceManager</code> should use a
    * <code>ContextGroup</code> name that is the same as the name of the
    * service itself. 
    * <br>The returned value must not change if this method is called multiple times.
    * 
    * @return The name of the context group that the
    *    <code>ServiceManager's</code> activities
    *    belong to. If no <code>ContextGroup</code> has been specified then this
    *    method returns <code>null</code> and the <code>ServiceManager's</code> activities belong to
    *    the default context group.
    *
    */
   public String getContextGroup()
   {
      return _contextGroup;
   }

   /**
    * Returns a <code>java.io.Serializable</code> that contains information specific to the particular HLS 
    * {@link javax.activity.coordination.ServiceManager ServiceManager}.
    * If the <code>service-specific data</code> is not a <code>Serializable</code> object then
    * this method returns <code>null</code> and {@link #getServiceSpecificDataAny() getServiceSpecificDataAny}
    * should be called instead.
    * <br>The contents of the returned <code>Serializable</code> should not change if this method is called multiple times.
    * 
    * @return A java.io.Serializable that contains information specific to the HLS. Returns null if
    *    the <code>service-specific data</code> is not a <code>Serializable</code> object
    *         or if there is no <code>service-specific data</code> data.
    */
   public java.io.Serializable getServiceSpecificDataValue()
   {
      if (_serviceSpecificData == null)
      {
        // if a propagated java.io.Serializable is available, extract it
        // now from the MarshalledObject
        if (_propagatedServiceSpecificData != null)
        { 
          try
          {
            // can throw IOException and ClassNotFoundException
            _serviceSpecificData = (java.io.Serializable)_propagatedServiceSpecificData.get();
          }
          catch (Exception e)
          {
            _serviceSpecificData = null;
          }
        }
      }
      return _serviceSpecificData;
   }
   /**
    * Returns a CORBA <code>Any</code> that contains information specific to the particular HLS 
    * {@link javax.activity.coordination.ServiceManager ServiceManager}.
    * <br>The contents of the returned <code>Any</code> should not change if this method is called multiple times.
    * 
    * @return A CORBA Any that contains information specific to the HLS. 
    *         <br> Returns null if there is no <code>service-specific data</code> data.
    */
   public org.omg.CORBA.Any getServiceSpecificDataAny()
   {
      return _serviceSpecificDataAny;
   }



   // Private attributes
   private String _serviceName = null;
   private String _contextGroup = null;
   private java.io.Serializable _serviceSpecificData = null;
   private org.omg.CORBA.Any _serviceSpecificDataAny = null;
   private java.rmi.MarshalledObject _propagatedServiceSpecificData = null;
}
