package javax.activity;

/**
  * <code>Signals</code> are events that are broadcast to interested parties
  * as part of a coordinated {@link javax.activity.coordination.SignalSet SignalSet}.
  * Each <code>Signal</code> is uniquely
  * identified by a combination of its <code>SignalName</code> and the name
  * of the containing <code>SignalSet</code>.
  */                                                                          
public class Signal
{
   /**
    * Constructor used by a HLS to create a <code>Signal</code> object.
    * The Activity service encapsulates the Serializable <code>extData</code> in a 
    * <code>java.rmi.MarshalledObject</code> to avoid class-loading problems when propagating 
    * <code>CosActivity.Signal</code> structures containing this object.
    *
    * @param signalName The name of the <code>Signal</code>.
    *
    * @param ssName The name of the <code>SignalSet</code> that produces the <code>Signal</code>.
    *
    * @param extData Data that is specific to a particular
    *        <code>SignalSet</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public Signal(String signalName, String ssName, java.io.Serializable extData)
   {
      _signalName = signalName;
      _ssName = ssName;
      _extendedData = extData; 
   }

   /**
    * Constructor used by the Activity service to create a <code>Signal</code> object.
    * It is used by the Activity service when processing a received <code>CosActivity.Signal</code>
    * if the <code>CosActivity.Signal.application_specific_data</code> field of that <code>Signal</code>
    * contains a <code>MarshalledObject</code>.
    *
    * @param signalName The name of the <code>Signal</code>.
    *
    * @param ssName The name of the <code>SignalSet</code> that produces the <code>Signal</code>.
    *
    * @param extData Data that is specific to a particular
    *        <code>SignalSet</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public Signal(String signalName, String ssName, java.rmi.MarshalledObject propagatedExtData)
   {
      _signalName = signalName;
      _ssName = ssName;
      _propagatedExtendedData = propagatedExtData; 
   }

   /**
    * Constructor used by the Activity service or HLS to create a <code>Signal</code> object.
    * <br>It is used by the Activity service when processing a received <code>CosActivity.Signal</code>
    * if the <code>CosActivity.Signal.application_specific_data</code> field of that <code>Signal</code>
    * does not contain a <code>MarshalledObject</code>.
    * <br>It may also used by a HLS to create a <code>Signal</code>.
    *
    * @param signalName The name of the <code>Signal</code>.
    *
    * @param ssName The name of the <code>SignalSet</code> that produces the <code>Signal</code>.
    *
    * @param extData Data that is specific to a particular
    *        <code>SignalSet</code> implementation.
    *        This parameter may be <code>null</code>.
    *
    */
   public Signal(String signalName, String ssName, org.omg.CORBA.Any extData)
   {
      _signalName = signalName;
      _ssName = ssName;
      _extendedDataAny = extData;
   }

   /**
    * Returns a <code>String</code> that identifies the <code>Signal</code> name.
    * 
    * @return The <code>Signal</code> name.
    *
    */
   public String getName()
   {
      return _signalName;
   }

   /**
    * Returns a <code>String</code> that identifies the name of the <code>SignalSet</code> that produced
    * the <code>Signal</code>.
    * 
    * @return The <code>SignalSet</code> name.
    *
    */
   public String getSignalSetName()
   {
      return _ssName;
   }

   /**
    * Returns a <code>java.io.Serializable</code> that contains information specific to the <code>Signal</code>.
    * If this extended data is not a <code>Serializable</code> object then
    * this method returns <code>null</code> and {@link #getExtendedAny() getExtendedAny}
    * should be called instead.
    * 
    * @return A java.io.Serializable that contains information specific to the <code>Signal</code>.
    *   Returns null if the extended data is not a <code>Serializable</code> object
    *         or if there is no extended data.
    */
   public java.io.Serializable getExtendedValue()
   {  
      if (_extendedData == null)
      {
        // if a propagated java.io.Serializable is available, extract it
        // now from the MarshalledObject
        if (_propagatedExtendedData != null)
        { 
          try
          {
            // can throw IOException and ClassNotFoundException
            _extendedData = (java.io.Serializable)_propagatedExtendedData.get();
          }
          catch (Exception e)
          {
             _extendedData = null;
          }
        }
      }
      return _extendedData;
   }

   /**
    * Returns a CORBA <code>Any</code> that contains information specific to the <code>Signal</code>.
    * 
    * @return A CORBA <code>Any</code> that contains information specific to the <code>Signal</code>.
    *         <br> Returns null if there is no extended data.
    */
   public org.omg.CORBA.Any getExtendedAny()
   {
      return _extendedDataAny;
   }


   // Private attributes
   private String _signalName = null;
   private String _ssName = null;
   private java.io.Serializable _extendedData = null;
   private org.omg.CORBA.Any _extendedDataAny = null;
   private java.rmi.MarshalledObject _propagatedExtendedData = null;
}
