package javax.activity;
import javax.activity.propertygroup.*;
import javax.activity.coordination.*;

/**
 * The <code>UserActivity</code> interface defines the methods offered to a high-level service
 * (HLS) constructed on top of the Activity service framework. 
 * These methods control the demarcation scope of an Activity associated
 * with the calling thread of execution.
 * <br> The specific behaviour of an individual HLS is encapsulated by
 * a {@link javax.activity.coordination.ServiceManager ServiceManager}. An instance
 * of the <code>UserActivity</code> interface has a <code>ServiceManager</code> registered
 * with it prior to any Activities being begun by that instance.
 * <br> Each Activity started by a <code>UserActivity</code> instance is an
 * Activity instance of the HLS represented by the registered
 * <code>ServiceManager</code>. Methods of the <code>UserActivity</code> interface that operate
 * on the <i>active</i> Activity context are operating on the HLS
 * Activity instance most recently associated with the calling thread.
 * <br>The UserActivity interface may be obtained by a HLS via a JNDI <b>lookup</b>
 * of <code>java:comp/UserActivity</code>.
 * 
 */
public interface UserActivity
{

   /**
    * Create a new Activity and associate it with the current thread.
    *
    * @param timeout the period (in seconds) within which the Activity must be completed.
    * If it is not completed within this period then it is subject to being completed
    * by the Activity service with the CompletionStatusFail status. <code>timeout</code>
    * can have the following possible values:
    * <dl>
    * <dt><b>any positive value</b>
    * <dd>the Activity must complete within this number of seconds.
    * <dt><b>-1</b>
    * <dd>the Activity will never be completed automatically by the Activity Service
    * implementation (i.e., it will never be considered to have timed out).
    * <dt><b>0</b>
    * <dd>the last value specified using {@link #setTimeout(int) setTimeout} method is used.
    * If no prior call to {@link #setTimeout(int) setTimeout}
    * has occurred for this thread, or the value returned is 0, then it is
    * implementation dependent as to the timeout value associated with this Activity.
    * </dl>
    * <p>Any other value results in the TimeoutRangeException being thrown.
    *
    * @exception InvalidStateException Thrown if the parent Activity has been marked
    *    as CompletionStatusFailOnly.
    *
    * @exception TimeoutRangeException Thrown if <b>timeout</b> is less than
    *    -1 or if it is outside an administratively configured range in
    *    the deployed system.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void begin(int timeout) throws InvalidStateException,
                ServiceNotRegisteredException, TimeoutRangeException, SystemException;

   /**
    * Causes the <i>active</i> Activity context to be completed
    * with its current
    * {@link javax.activity.CompletionStatus CompletionStatus}.
    * If a CompletionSignalSet is used by the registered ServiceManager,
    * then it will be driven by the Activity
    * service to obtain completion signals to distribute to any registered
    * Actions.
    * <br>If the Activity is nested within a parent, then the parent Activity
    * becomes associated with the thread on completion of this operation.
    * <br>If there are any child active or suspended Activities or
    * transactions and the CompletionStatus is CompletionStatusFail or
    * CompletionStatusFailOnly then those child Activites will have
    * their CompletionStatus set to CompletionStatusFailOnly and any
    * child transaction will be called to setRollbackOnly.
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the SignalSet used for completion.
    *    In the absence of a CompletionSignalSet a null <code>Outcome</code> object
    *    reference is returned.
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception ActivityPendingException Thrown if the thread from which the
    *    completion is initiated is not the only thread associated with the
    *    target Activity. The application response should be to try again
    *    later when any asynchronous work on other threads has been suspended.
    *
    * @exception ContextPendingException Thrown if the CompletionStatus is
    *    CompletionStatusSuccess and the target Activity encompasses
    *    active or suspended Activities or transactions. The application
    *    response should be to complete the encompassed Activities and
    *    transactions and then complete the target Activity or to force the
    *    target Activity to end by setting the CompletionStatus to
    *    CompletionStatusFail and then ending it.
    *
    * @exception NotOriginatorException Thrown if an attempt is made by
    *    an application to end an Activity in a different execution
    *    environment from that in which the Activity was begun.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced. The Activity's final
    *    <code>Status</code> is <code>StatusError</code>.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome complete() throws NoActivityException, ActivityPendingException, ContextPendingException,
                            ServiceNotRegisteredException, NotOriginatorException,
                            ActivityNotProcessedException, SystemException;

   /**
    * Causes the <i>active</i> Activity context to complete
    * with the specified
    * {@link javax.activity.CompletionStatus CompletionStatus}.
    * If a CompletionSignalSet is used by the registered ServiceManager,
    * then it will be driven by the Activity
    * service to obtain completion signals to distribute to any registered
    * Actions.
    * <br>If the Activity is nested within a parent, then the parent Activity
    * becomes associated with the thread on completion of this operation.
    * <br>If there are any child active or suspended Activities or
    * transactions and the CompletionStatus is CompletionStatusFail or
    * CompletionStatusFailOnly then those child Activites will have
    * their CompletionStatuses set to CompletionStatusFailOnly and any
    * child transaction will be called to setRollbackOnly.
    *
    * @param completionStatus the {@link javax.activity.CompletionStatus CompletionStatus}
    *     with which the <i>active</i> Activity should end. 
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the SignalSet used for completion.
    *    In the absence of a CompletionSignalSet a null <code>Outcome</code> object
    *    reference is returned.
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception ActivityPendingException Thrown if the thread from which the
    *    completion is initiated is not the only thread associated with the
    *    target Activity. The application response should be to try again
    *    later when any asynchronous work on other threads has been suspended.
    *
    * @exception ContextPendingException Thrown if the CompletionStatus is
    *    CompletionStatusSuccess and the target Activity encompasses
    *    active or suspended Activities or transactions. The application
    *    response should be to complete the encompassed Activities and
    *    transactions and then complete the target Activity or to force the
    *    target Activity to end by setting the CompletionStatus to
    *    CompletionStatusFail and then ending it.
    *
    * @exception NotOriginatorException Thrown if an attempt is made by
    *    an application to end an Activity in a different execution
    *    environment from that in which the Activity was begun.
    *
    * @exception InvalidStateException Thrown if the target Activity cannot
    *    be completed with the requested <code>CompletionStatus</code>, for example
    *    because the <code>CompletionStatus</code> may not be changed from <code>CompletionStatusFailOnly</code>
    *    to any other value, or if a <code>null</code>
    *    or invalid value is specified by <code>completionStatus</code>.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced. The Activity's final
    *    <code>Status</code> is <code>StatusError</code>.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome completeWithStatus(int completionStatus) throws NoActivityException, ActivityPendingException, ContextPendingException,
                            ServiceNotRegisteredException, NotOriginatorException, InvalidStateException,
                            ActivityNotProcessedException, SystemException;

   /**
    * Sets the {@link javax.activity.CompletionStatus CompletionStatus} of the
    * <i>active</i> Activity. 
    * This method may be called multiple times before the Activity is
    * completed, and the CompletionStatus may be changed between
    * CompletionStatusSuccess and CompletionStatusFail to indicate a
    * <i>point in time</i> value. Once the CompletionStatus has been set
    * to CompletionStatusFailOnly, it may not be changed again.
    * 
    * @param completionStatus the {@link javax.activity.CompletionStatus CompletionStatus}
    *     value to set. 
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception InvalidStateException Thrown if an attempt is made to update
    *    a CompletionStatus of CompletionStatusFailOnly or if a <code>null</code>
    *    or invalid value is specified by <code>completionStatus</code>.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void setCompletionStatus(int completionStatus) throws NoActivityException, InvalidStateException, 
                            ServiceNotRegisteredException, SystemException;

   /**
    * Returns the current value of the {@link javax.activity.CompletionStatus CompletionStatus}
    * of the <i>active</i> Activity. 
    * 
    * @return The {@link javax.activity.CompletionStatus CompletionStatus}
    *    of the <i>active</i> Activity. 
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public int getCompletionStatus() throws ServiceNotRegisteredException, NoActivityException, SystemException;

   /**
    * Returns the current value of the {@link javax.activity.Status Status}
    * of the <i>active</i> Activity. 
    * 
    * @return The {@link javax.activity.Status Status}
    *    of the <i>active</i> Activity. 
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public int getStatus() throws ServiceNotRegisteredException, SystemException;

   /**
    * Returns a printable string describing the <i>active</i> Activity.
    * 
    * @return A printable string describing the <i>active</i> Activity.
    *    If no Activity is associated with the calling thread then
    *    <code>null</code> is returned.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public String getName() throws ServiceNotRegisteredException, SystemException;

   /**
    * Sets the default timeout, in seconds, after which an Activity may
    * be automatically completed by the Activity service.
    * This default is used for an Activity that is begun with a timeout
    * parameter value of 0. A value of -1 indicates no timeout. A value
    * of 0 for the default timeout indicates that a system-managed value
    * or implementation-specific default should be used.
    * <br>The timeout is a useful mechanism for protecting against clients
    * that fail to end an Activity in a timely fashion.
    *
    * @param timeout the default timeout period (in seconds) used by
    * any future Activities begun with a <code>timeout</code> value of <i>zero</i>.
    *
    * @exception TimeoutRangeException Thrown if <b>timeout</b> is less than
    *    -1 or if it is outside an administratively configured range in
    *    the deployed system.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void setTimeout(int timeout) throws ServiceNotRegisteredException, TimeoutRangeException, SystemException;

   /**
    * Returns the default timeout value.
    * 
    * @return The default timeout value, in seconds.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public int getTimeout() throws ServiceNotRegisteredException, SystemException;

   /**
    * Returns the {@link javax.activity.GlobalId GlobalId} of the
    * <i>active</i> Activity.
    * 
    * @return The {@link javax.activity.GlobalId GlobalId} of the
    *    <i>active</i> Activity, or <code>null</code> if
    *    there is no Activity associated with the thread.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public GlobalId getGlobalId() throws ServiceNotRegisteredException, SystemException;

   /**
    * Causes the {@link javax.activity.coordination.SignalSet SignalSet} specified by <code>signalSetName</code>
    * to start producing signals for all registered {@link javax.activity.coordination.Action Actions}.  
    * This method is used to distribute Signals to Actions at times other than
    * during completion.
    * Pre-defined <code>SignalSets</code>, such as <code>Synchronization</code>, cannot be requested
    * to produce signals via this method.
    *
    * @param signalSetName the name of the <code>SignalSet</code> that is to produce
    *    the signals.
    *
    * @return The {@link javax.activity.Outcome Outcome} returned is both
    *    set by and given meaning by the SignalSet.
    *    A null <code>Outcome</code> object reference may be returned if the SignalSet
    *    does not produce an Outcome.
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception SignalSetUnknownException Thrown if the named SignalSet is not recognized.
    *
    * @exception InvalidActivityException Thrown if an attempt is made to use the
    *    Synchronization or ChildLifetime SignalSets, or if the Activity is in the
    *    process of completion.
    *
    * @exception ActivityNotProcessedException Thrown if the signals required to
    *    complete this operation could not be produced.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public Outcome broadcast(String signalSetName) throws NoActivityException, SignalSetUnknownException,
                                  ServiceNotRegisteredException, InvalidActivityException, 
                                  ActivityNotProcessedException, SystemException;

   /**
    * Returns the {@link javax.activity.ActivityCoordinator ActivityCoordinator}
    * of the <i>active</i> Activity, or <code>null</code> if there is no Activity
    * associated with the calling thread.  In some
    * execution environments, such as a J2EE client, this method need not be
    * implemented and calling it may result in a NoImplementationException.
    * 
    * @return The {@link javax.activity.ActivityCoordinator ActivityCoordinator}
    *    of the <i>active</i> Activity.
    *    If there is no Activity associated with the calling thread then a
    *    <code>null</code> object reference is returned.
    *
    * @exception NoImplementException Thrown if the Activity service does not
    *    provide an implementation of this method in the calling environment, for
    *    example if a J2EE client calls this method.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityCoordinator getCoordinator() throws ServiceNotRegisteredException, NoImplementException, SystemException;

   /**
    * Returns the {@link javax.activity.ActivityCoordinator ActivityCoordinator}
    * of the parent of the <i>active</i> Activity,
    * or <code>null</code> if the <i>active</i> Activity is a top-level Activity.
    * In some execution environments, such as a J2EE client, this method need not be
    * implemented and calling it may result in a NoImplementationException.
    * 
    * @return The {@link javax.activity.ActivityCoordinator ActivityCoordinator}
    *    of the parent of the <i>active</i> Activity.
    *    If the <i>active</i> Activity is a top-level Activity then a
    *    <code>null</code> object reference is returned.
    *
    * @exception NoImplementException Thrown if the Activity service does not
    *    provide an implementation of this method in the calling environment, for
    *    example if a J2EE client calls this method.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityCoordinator getParentCoordinator() throws ServiceNotRegisteredException, NoImplementException, SystemException;

   /**
    * Returns the instance of the type of {@link javax.activity.propertygroup.PropertyGroup PropertyGroup},
    * identified by </i>name</i>, scoped to the <i>active</i> Activity.
    * The <code>PropertyGroup</code> identified by <code>name</code> must be 
    * supported by the 
    * {@link javax.activity.coordination.ServiceManager ServiceManager}
    * for this <code>UserActivity</code> instance.
    * 
    * @param name the name of the type of <code>PropertyGroup</code> to return.
    *
    * @return A {@link javax.activity.propertygroup.PropertyGroup PropertyGroup} instance
    *    of the type indicated by <code>name</code>.
    *
    * @exception PropertyGroupUnknownException Thrown if the named PropertyGroup
    *    type is not recognized.
    *
    * @exception NoActivityException Thrown if there is no Activity associated
    *    with the calling thread of execution.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public PropertyGroup getPropertyGroup(String name) throws PropertyGroupUnknownException,
                            ServiceNotRegisteredException, NoActivityException, SystemException;

   /**
    * Registers a {@link javax.activity.coordination.ServiceManager ServiceManager}
    * for the type of high-level service (HLS) whose activities are to be demarcated
    * through the target <code>UserActivity</code> instance.
    * The Activity service uses the properties of the registered <code>ServiceManager</code>
    * to create and operate on Activities specific to that service.
    * The <code>UserActivity</code> (or specialised ActivityManager) instance requires
    * a <code>ServiceManager</code> to be registered with it before it may be 
    * used to begin new Activities.
    * 
    * @param service the <code>ServiceManager</code> for the HLS whose
    *    activities are to be demarcated through the target <code>UserActivity</code> instance.
    *
    * @exception PropertyGroupUnknownException Thrown if the ServiceManager
    *    cannot obtain a PropertyGroupManager for one or more of the PropertyGroups
    *    it uses.
    *
    * @exception ServiceAlreadyRegisteredException Thrown if a ServiceManager
    *    has already been registered with the <code>UserActivity</code> instance.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void registerService(ServiceManager service)
               throws PropertyGroupUnknownException, ServiceAlreadyRegisteredException, SystemException;

   /**
    * Returns the registered {@link javax.activity.coordination.ServiceManager ServiceManager}
    * for this <code>UserActivity</code> instance.
    * 
    * @return The {@link javax.activity.coordination.ServiceManager ServiceManager}
    *    for this <code>UserActivity</code> instance, or <code>null</code> if none has been
    *    registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ServiceManager getService() throws SystemException;

   /**
    * Recreates an Activity after a failure or after the activity has been hibernated.
    * The Activity to be recreated is identified by the <code>activity</code> parameter.
    * Activity context hierarchies may be recreated by an HLS in two ways:
    * <ol>
    * <li>By specifying the <code>parent</code>, if any, of each recreated Activity. Activities so created are
    * not associated with the calling thread on completion of this operation.
    * <li>By specifying <code>true</code> for the <code>resume</code> flag to associate each recreated Activity with the calling thread.
    * An Activity recreated in this way is recreated as a child of any Activity already associated with the
    * calling thread. The <code>parent</code> parameter must be <code>null</code> when using this approach.
    * </ol>
    * <br>The Activity service contacts the registered <code>ServiceManager</code> to recreate the <code>Actions</code>
    * and <code>SignalSets</code> for the Activity identified by the <code>GlobalId</code> parameter.
    *
    * @param activity the {@link javax.activity.GlobalId GlobalId} for the Activity to be recreated.
    *
    * @param parent the {@link javax.activity.GlobalId GlobalId} of the parent of the Activity to be recreated.
    *
    * @param resume specifies whether or not the recreated Activity context becomes associated with the
    *    calling thread. This parameter is used only when the <code>parent</code> is <code>null</code>.
    *    A value of <code>true</code> combined with a non-null <code>parent</code> is an illegal combination
    *    and results in an IllegalArgumentException.
    *
    * @param recData the {@link javax.activity.RecreateData RecreateData} object created earlier when the
    * HLS hibernated the Activity or indicated that it had become persistent.
    *
    * @return The {@link javax.activity.ActivityCoordinator ActivityCoordinator}
    *    for the recreated Activity.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered or if the ServiceManager for a recreated Activity is not available.
    *
    * @exception ActivityCompletedException Thrown if the Activity service and ServiceManager have no 
    *    knowledge of the specified <code>globalId</code> - for example if the Activity it identifies
    *    has already completed or if it never got as far as logging persistent state.
    *
    * @exception IllegalArgumentException Thrown if an illegal combination of <code>parent</code> and <code>resume</code> values is specified.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public ActivityCoordinator recreate(GlobalId activity, GlobalId parent, boolean resume, RecreateData recData) throws ServiceNotRegisteredException, 
                                           ActivityCompletedException, IllegalArgumentException, SystemException;

   /**
    * Driven by a HLS, after a failure, to retrieve a list of <code>GlobalIds</code> that the Activity service itself
    * has logged information about.
    * The Activity service itself is not responsible for persisting any information
    * that is owned by an HLS, such as <code>Actions</code> and <code>SignalSets</code>.
    * <p>The Activity service may
    * persist data of its own to facilitate recoverability. Any data persisted by the Activity service is discarded
    * when the relevant Activity completes. The <code>recover</code> method ensures that the HLS has a means by which it can
    * drive the Activity service to complete any Activity that the Activity service had persisted data for.
    * The recovery requirements placed on an HLS mean that any Activity whose <code>GlobalId</code> is returned on the 
    * <code>recover</code> method,
    * for which the HLS has no record of its own, may be assumed to have completed successfully already 
    * or may be assumed never to have performed any recoverable work. Such recovered Activities may be immediately
    * discarded by calling {@link #forget(GlobalId) forget}.
    * <p>An Activity service implementation that does not log information of its own returns null when this method
    * is called.
    *
    * @return The list of <code>GlobalIds</code> for which the Activity service has information persisted.
    *    If there are none, or if the Activity service does not log information of its own, then <i>null</i> is returned. 
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public GlobalId[] recover() throws ServiceNotRegisteredException, SystemException;

   /**
    * Forget the specified <code>GlobalId</code>.
    * Driven by a HLS for each <code>GlobalId</code> returned by the
    * {@link #recover() recover} method for which the HLS has no knowledge. Such Activities may be
    * assumed to have completed successfully already 
    * or may be assumed never to have performed any recoverable work.
    *
    * @param globalId the <code>GlobalId</code> to be forgotten.
    *
    * @exception ServiceNotRegisteredException Thrown if no <code>ServiceManager</code>
    *    has been registered.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public void forget( GlobalId globalId) throws ServiceNotRegisteredException, SystemException;
}
