package javax.activity.coordination;
import javax.activity.*;
import javax.activity.propertygroup.*;

/**
  * A ServiceManager is an entity that is provided by a high-level service (HLS)
  * that uses the Activity service; it is a factory for the HLS's objects and also
  * specifies how the HLS's Activities should be managed.
  * In particular, it is used to specify:
  * <ul>
  * <li> which {@link javax.activity.propertygroup.PropertyGroup PropertyGroups}
  * the HLS uses
  * <li>the <code>CompletionSignalSet</code> that is used to complete the HLS's
  * Activities.
  * <li>the {@link javax.activity.ServiceInformation ServiceInformation} for 
  * the HLS's Activities. This object describes which <code>ContextGroup</code>
  * the HLS participates in and is propagated as part of the Activity service
  * context.
  * </ul>
  * <br>The methods of the ServiceManager interface may execute
  * with or without an Activity service context being associated with the thread.
  * <br>The Activity service uses an HLS's ServiceManager when it
  * creates and operates on Activities specific to that HLS. 
  *
  */                                                                          
public interface ServiceManager
{
   /**
    * Returns a reference to the
    * {@link javax.activity.ServiceInformation ServiceInformation} object
    * that encapsulates all the information about the HLS that needs to
    * be propagated as part of Activity service context in the </code>type_specific_data</code>,
    * for example the ContextGroup name.
    * <br>The returned value must not change if this method is called multiple times.
    * 
    * @return The {@link javax.activity.ServiceInformation ServiceInformation} for
    * the HLS.
    *
    */
   public ServiceInformation getServiceInformation();

   /**
    * Returns a reference to an instance of the named
    * {@link javax.activity.coordination.SignalSet SignalSet}.
    * <br> For each named <code>SignalSet</code> that an HLS provides, it must provide a
    * <code>SubordinateSignalSet</code> of the same name. If the <code>SignalSets</code> provided
    *  by an HLS can be used either to produce <code>Signals</code> or to consume
    * <code>Signals</code> received from a superior server then the same <code>SubordinateSignalSet</code>
    * may be returned in response to both
    * {@link #getSignalSet(String) getSignalSet} and
    * {@link #getSubordinateSignalSet(String) getSubordinateSignalSet} 
    * <br> In some
    * execution environments, such as a J2EE client, this method need not be
    * implemented and calling it may result in a NoImplementException.
    * 
    * @param signalSetName the name of the type of {@link javax.activity.coordination.SignalSet SignalSet}
    *    whose object reference is to be returned. 
    *
    * @exception SignalSetUnknownException Thrown if the named <code>SignalSet</code>
    *    is not recognized by the <code>ServiceManager</code>.
    *
    * @exception NoImplementException Thrown if the Activity service does not
    *    provide an implementation of this method in the calling environment, for
    *    example if a J2EE client calls this method.
    *
    */
   public SignalSet getSignalSet(String signalSetName) throws SignalSetUnknownException,
                                                NoImplementException;

   /**
    * Returns a reference to an instance of the named
    * {@link javax.activity.coordination.SubordinateSignalSet SubordinateSignalSet}.
   * <br> For each named <code>SignalSet</code> that an HLS provides, it must provide a
   * <code>SubordinateSignalSet</code> of the same name. If the <code>SignalSets</code> provided
   *  by an HLS can be used either to produce <code>Signals</code> or to consume
   * <code>Signals</code> received from a superior server then the same <code>SubordinateSignalSet</code>
   * may be returned in response to both
   * {@link #getSignalSet(String) getSignalSet} and
   * {@link #getSubordinateSignalSet(String) getSubordinateSignalSet} 
    * <br> In some 
    * execution environments, such as a J2EE client, this method need not be
    * implemented and calling it may result in a NoImplementException.
    * 
    * @param signalSetName the name of the type of {@link javax.activity.coordination.SignalSet SignalSet}
    *    whose object reference is to be returned. 
    *
    * @exception SignalSetUnknownException Thrown if the named <code>SignalSet</code>
    *    is not recognized by the <code>ServiceManager</code>.
    *
    * @exception NoImplementException Thrown if the Activity service does not
    *    provide an implementation of this method in the calling environment, for
    *    example if a J2EE client calls this method.
    *
    */
   public SubordinateSignalSet getSubordinateSignalSet(String signalSetName) throws SignalSetUnknownException,
                                                NoImplementException;

   /**
    * Returns a reference to an instance of a
    * {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager},
    * responsible for creating and managing the PropertyGroup identified
    * in <code>PropertyGroupName</code>.
    * <br> In some
    * execution environments, such as a J2EE client, this method need not be
    * implemented and calling it may result in a NoImplementationException.
    * 
    * @param propertyGroupName the name of the type of
    *    {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}
    *    whose object reference is to be returned. 
    *
    * @return A {@link javax.activity.propertygroup.PropertyGroupManager PropertyGroupManager}
    *         instance.
    *
    * @exception PropertyGroupUnknownException Thrown if the named <code>PropertyGroup</code>
    *    is not recognized by the ServiceManager.
    *
    */
   public PropertyGroupManager getPropertyGroupManager(String propertyGroupName) throws PropertyGroupUnknownException;

   /**
    * Returns the array of {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    * names that are used by the HLS.
    * <br>Each time an Activity for this HLS is started, an instance
    * of each PropertyGroup in the list is started and associated with the
    * Activity.
    * <br>The returned value must not change if this method is called multiple times.
    * 
    * @return An array of {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    names that are used by the HLS. If no <code>PropertyGroups</code>
    *    have been specified then this method returns <code>null</code>.
    *
    */
   public String[] getPropertyGroupNames();

   /**
    * Returns the name of the <i>default</i> {@link javax.activity.coordination.SignalSet SignalSet},
    * if any, that will be used for the distribution of completion signals
    * when any Activity associated with this HLS completes. The default <code>SignalSet</code>
    * for the HLS can be overridden on a per-Activity basis; the specific <code>SignalSet</code>
    * for an individual Activity for this HLS can be determine through the <code>ActivityCoordinator</code>
    * {@link javax.activity.ActivityCoordinator#getCompletionSignalSetName() getCompletionSignalSetName} and
    * {@link javax.activity.ActivityCoordinator#setCompletionSignalSetName(String) setCompletionSignalSetName} methods.
    * 
    * <br>The returned value must not change if this method is called multiple times.
    * 
    * @return The name of the {@link javax.activity.coordination.SignalSet SignalSet}
    *    that will be used for the distribution of completion signals
    *    when the current Activity completes.
    *    If no CompletionSignalSet has been set then <code>null</code> is returned.
    *
    */
   public String getCompletionSignalSetName();

   /**
    * Returns the <code>type</code> of the Activities that are created for this
    * HLS. The <code>type</code> is propagated in the <code>ActivityIdentity</code>
    * part of the Activity context and confers information about the format
    * of the <code>activity_specific_data</code> encoded in the <code>ActivityIdentity</code>.
    * <br>This function should return either a value of <code>2</code> to indicate that
    * no J2EE-specific information is contained in the
    * <code>ActivityIdentity.activity_specific_data</code>
    * or <code>0x4A324545</code> to indicate that a
    * {@link javax.activity.ServiceInformation ServiceInformation} object is contained in the
    * <code>ActivityIdentity.activity_specific_data</code>
    * <br>The returned value must not change if this method is called multiple times.
    * 
    * @return The <code>type</code> value that will be set into the
    *    <code>omg.org.CosActivity.ActivityIdentity</code> for Activities created on
    *    behalf of this HLS.
    *
    */
   public int getActivityType();

   /**
    * Indicates that an Activity of the type specific to the target <code>ServiceManager</code>
    * has been received from another
    * process and is active on the current thread. This method is 
    * called by the Activity service after its context filter (for example an IIOP portable
    * interceptor), executed on receipt of an inbound request, has imported a new Activity
    * context into the execution domain.
    * <br>Optional <code>ServiceInformation</code> specific to the type of target
    * <code>ServiceManager</code> may be received with the request in the <code>ActivityContext</code>.
    * 
    * @param serviceInfo the {@link javax.activity.ServiceInformation ServiceInformation} object 
    *    encoded in the </code>type_specific_data</code> of a received context.  
    */
   public void activityImported(ServiceInformation serviceInfo);

   /**
    * Driven after a failure in order to retrieve any recovered <code>SignalSets</code> that
    * were involved in the Activity identified by <code>globalId</code> prior to the failure.
    * The HLS that provides the
    * <code>ServiceManager</code> determines the point, during the processing of an Activity, at
    * which it must persist the state of its <code>SignalSets</code> and <code>Actions</code>. Any failure,
    * between this point and the end of the Activity, requires the HLS to recreate
    * the Activity and, eventually, complete it. Each recreated Activity that
    * uses the target <code>ServiceManager</code> calls this method to retrieve the list of
    * <code>SignalSets</code> recovered by the HLS for the Activity identified by <code>globalId</code>.
    *
    * @param globalId the <code>GlobalId</code> for the recreated Activity.
    *
    * @return The list of <code>SignalSets</code> recovered by the HLS for the Activity
    *    identified by <code>globalId</code>.
    *    If there are none, then a zero element array is returned. 
    *
    * @exception ActivityCompletedException Thrown if the HLS has no 
    *    knowledge of the specified <code>globalId</code> - for example if the Activity it identifies
    *    has already completed or if it never got as far as logging persistent state.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public SignalSet[] recoverSignalSets(GlobalId globalId) throws ActivityCompletedException, SystemException;

   /**
    * Driven after a failure in order to retrieve a <code>RecoverableAction</code> object for each
    * <code>Action</code> that was involved in the Activity identified by <code>globalId</code> prior to the failure
    * and which had been registered with an interest in the <code>SignalSet</code>
    * identified by <i>signalSetName</i>.
    * The HLS that provides the
    * <code>ServiceManager</code> determines the point, during the processing of an Activity, at
    * which it must persist the state of its <code>SignalSets</code> and <code>Actions</code>. Any failure,
    * between this point and the end of the Activity, requires the HLS to recreate
    * the Activity and, eventually, complete it. Each recreated Activity that
    * uses the target <code>ServiceManager</code> recovers its <code>SignalSets</code> via
    * {@link #recoverSignalSets(GlobalId) recoverSignalSets} and then, for each
    * recovered <code>SignalSet</code>, calls this method
    * to obtain an array of <code>RecoverableAction</code> recovered by the HLS.
    *
    * @param globalId the <code>GlobalId</code> for the recreated Activity.
    *
    * @param signalSetName the name of a {@link javax.activity.coordination.SignalSet SignalSet}
    *    supported by the HLS. Only recovered Actions that registered an interest in this
    *    <code>SignalSet</code> are returned by this operation. If this parameter is <i>null</i> then <code>RecoverableActions</code>
    *    for all recovered <i>global</i> Actions are returned.
    *
    * @return The array of <code>RecoverableActions</code> recovered by the HLS for the specified <code>SignalSet</code>
    *    in the Activity identified by <code>globalId</code>.
    *    If there are none, then a zero element array is returned. 
    * 
    * @exception SignalSetUnknownException Thrown if the specified <code>signalSetName</code>
    *    is not recognized.
    *
    * @exception ActivityCompletedException Thrown if the HLS has no 
    *    knowledge of the specified <code>globalId</code> - for example if the Activity it identifies
    *    has already completed or if it never got as far as logging persistent state.
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public RecoverableAction[] recoverActions(GlobalId globalId, String signalSetName) throws SignalSetUnknownException,
                                                      ActivityCompletedException, SystemException;
   /**
    * Directs the HLS to initiate the recreation of the Activity, specified by <code>globalId</code>,
    * after the Activity has been hibernated. 
    * The Activity service may call this method if it receives an Activity context for an Activity
    * that it does not recognize. 
    * If the HLS has knowledge of the Activity it must drive
    * {@link javax.activity.UserActivity#recreate(javax.activity.GlobalId, javax.activity.GlobalId, boolean, RecreateData) UserActivity.recreate}
    * to initiate the recreation of the Activity and return a value of <code>true</code>.
    * If the HLS does not recognize this Activity then it must return a value of <code>false</code>.
    *
    * @param globalId the {@link javax.activity.GlobalId GlobalId} of the 
    *    Activity which is to be reactivated from hibernation.
    *    
    * @return <code>true</code> if the specified <code>globalId</code> represents
    *         an Activity that the HLS recognizes. Otherwise returns <code>false</code>.
    *
    * @exception ActivityCompletedException Thrown if the Activity
    *    has already completed or if there has been a failure following hibernation of a non-persistent
    *    Activity
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public boolean reactivate(GlobalId globalId) throws ActivityCompletedException, SystemException;


}
