package javax.activity.coordination;
import javax.activity.*;

/**
  * A SignalSet is an entity that is provided by a high-level service (HLS) built
  * on top of the Activity service that produces
  * {@link javax.activity.Signal Signals} and understands
  * the responses to those <code>Signals</code>. The <code>SignalSet</code> abstracts from the
  * {@link javax.activity.ActivityCoordinator ActivityCoordinator}
  * the knowledge of which <code>Signal</code> should be
  * distributed to the registered <code>Actions</code> based on the state of the Activity
  * and responses to previous <code>Signals</code>.
  * The Activity service itself then needs to provide only a very 
  * generic <code>ActivityCoordinator</code> to drive any specific <code>SignalSet</code>.
  * The <code>ActivityCoordinator</code> simply asks a <code>SignalSet</code> for the next
  * <code>Signal</code> and then broadcasts it to each interested <code>Action</code> in turn.
  * The response from each <code>Action</code> is fed back to the <code>SignalSet</code> which
  * has the knowledge of what that result means and which <code>Signal</code>
  * should be sent next.
  * <p>The Activity service itself provides implementations of the following
  * pre-defined SignalSets:
  * <dl>
  * <dt><code><b>org.omg.CosActivity.Synchronization</b></code>
  * <dd>The <code>org.omg.CosActivity.Synchronization</code> SignalSet contains the signals
  * <code>preCompletion</code> and <code>postCompletion</code>.
  * <dt><code><b>org.omg.CosActivity.ChildLifetime</b></code>
  * <dd>The <code>org.omg.CosActivity.ChildLifetime</code> SignalSet contains the signal
  * <code>childBegin</code>
  * <dt><code><b>org.omg.CosActivity.Failure</b></code>
  * <dd>The <code>org.omg.CosActivity.Failure</code> SignalSet contains the signals
  * <code>initialFailure</code> and <code>finalFailure</code>.
  * </dl>
  */                                                                          
public interface SignalSet
{
   /**
    * Returns the name of the target <code>SignalSet</code>.
    *
    * @return the name of the target <code>SignalSet</code>.
    *
    */
   public String getSignalSetName();


   /**
    * Returns the {@link javax.activity.Signal Signal} to be sent to the
    * {@link javax.activity.coordination.Action Actions} registered for this   
    * <code>SignalSet</code>.  The <code>Signal</code> returned may depend upon
    * the responses received from <code>Actions</code> that have been sent
    * previous signals.  If <code>null</code> is returned then this indicates that 
    * no other signals are to be sent and the <code>SignalSet</code> should
    * not be asked for further Signals.
    *
    * @return The next {@link javax.activity.Signal Signal} to distribute to the
    *    registered <code>Actions</code>.
    *
    */
   public Signal getSignal();

   /**
    * Called by an <code>ActivityCoordinator</code> to notify the target <code>SignalSet</code>
    * of the <code>Outcome</code>  returned by an <code>Action</code> in response to a <code>Signal</code>
    * produced by the <code>SignalSet</code>.
    * The {@link javax.activity.CoordinationInformation CoordinationInformation} object returned indicates whether 
    * the <code>Action</code> whose <code>Outcome</code> has just been processed should
    * continue to receive signals and whether the calling <code>ActivityCoordinator</code>
    * should continue to use the current <code>Signal</code> or request the next one.
    * <br>The logic of the <code>SignalSet</code> implementation determines how signaling
    * should proceed. For example, an OTS Resource could be registered
    * as an <code>Action</code> interested in a two-phase-commit <code>SignalSet</code>. If such an
    * <code>Action</code> produced a <code>VoteReadOnly</code> Outcome in response to a <code>prepare</code> signal
    * then it should receive no further signals. If it produced a <code>VoteRollback</code>
    * Outcome then the signaling could proceed immediately to the next <code>Signal</code>
    * even though some <code>Actions</code> may not have received the <code>prepare</code> Signal.
    *
    * @param response The {@link javax.activity.Outcome Outcome} of a registered <code>Action</code>
    *        in response to the current <code>Signal</code>. It is valid for <code>response</code>
    *        to be <code>null</code>.
    *
    * @return The {@link javax.activity.CoordinationInformation CoordinationInformation} object that describes
    *    whether the <code>Action</code> that produced the <code>response</code> Outcome should receive further
    *    signals and whether the current <code>Signal</code> should continue to be used.
    *
    * @exception SignalSetInactiveException Thrown if <code>response</code> is set
    *    without there first having been a call to the <code>SignalSet</code>
    *    {@link javax.activity.coordination.SignalSet#getSignal() getSignal} or
    *    {@link javax.activity.coordination.SubordinateSignalSet#setSignal(Signal) setSignal} methods.
    */
   public CoordinationInformation setResponse(Outcome response) throws SignalSetInactiveException;


   /**
    * Returns the consolidated {@link javax.activity.Outcome Outcome} of the <code>SignalSet</code>
    * following the completion of signaling. This is called by the Activity service
    * during {@link javax.activity.UserActivity#broadcast(java.lang.String) broadcast}
    * or {@link javax.activity.UserActivity#complete() complete}
    * processing.
    *
    * @return The consolidated {@link javax.activity.Outcome Outcome} of the <code>SignalSet</code>.
    *    It is valid for the <code>Outcome</code> to be <code>null</code>.
    *
    * @exception SignalSetActiveException Thrown if an attempt is made to obtain the
    *    <code>Outcome</code> after the <code>SignalSet</code>
    *    has started to produce signals and before it has finished producing them.
    */
   public Outcome getOutcome() throws SignalSetActiveException;

   /**
    * Called by the Activity service to set both the
    * {@link javax.activity.CompletionStatus CompletionStatus} and
    * {@link javax.activity.Status Status} before any signals are requested.
    * The signals produced may be influenced by these two statuses.
    * Once completion of an Activity has begun, control of the
    * <code>CompletionStatus</code> belongs to the <i>completion</i> <code>SignalSet</code> and cannot
    * be overridden by any other entity.
    *
    * @param completionStatus The {@link javax.activity.CompletionStatus CompletionStatus}
    *        of the Activity at the time of the invocation.
    *
    * @param status The {@link javax.activity.Status Status}
    *        of the Activity at the time of the invocation.
    *
    */
   public void setCompletionStatus(int completionStatus, int status);

   /**
    * Returns the {@link javax.activity.CompletionStatus CompletionStatus}
    * for the Activity as the <code>SignalSet</code>
    * has recorded it. This may be different from the <code>CompletionStatus</code>
    * of the Activity prior to the <code>SignalSet</code> producing signals as it
    * may have been modified during <code>Signal</code> processing.
    *
    * @exception SignalSetActiveException Thrown if no <code>Signals</code> have been
    *    produced by the <code>SignalSet</code>.
    */
   public int getCompletionStatus() throws SignalSetActiveException;

   /**
    * Called by the Activity service to pass a reference to the <code>ActivityCoordinator</code>
    * to the target <code>SignalSet</code>. This may be used by a <code>SignalSet</code>
    * to obtain references to all registered <code>Actions</code> in order to satisfy
    * persistence requirements if the HLS, through its <code>SignalSet</code> and <code>Actions</code>,
    * is recoverable.
    * For example, consider the case of a two-phase commit <code>SignalSet</code>: once 
    * <code>prepare</code> signals have been sent and acknowledged successfully by <code>Actions</code>, 
    * the HLS needs to make those Action references persistent.
    *
    * @param coord The {@link javax.activity.ActivityCoordinator ActivityCoordinator} reference
    *        for the Activity under which the target <code>SignalSet</code> is operating.
    *
    *
    * @exception SignalSetActiveException Thrown if this method is called
    *    after the <code>SignalSet</code>
    *    has started to produce signals.
    */
   public void setActivityCoordinator(ActivityCoordinator coord) throws SignalSetActiveException;

   /**
    * Called on the target object, by the Activity service, when the
    * target object is no longer required and indicates to the object
    * that it should perform any necessary clean-up.  
    * The ActivityCoordinator with which the target object was associated 
    * is not available at the time this method is called. 
    *
    */
   public void destroy();

}
