package javax.activity.propertygroup;
import javax.activity.*;
import java.util.Properties;

/**
  * A <code>PropertyGroupManager</code> is an entity that is be provided, as part
  * of a High Level Service (HLS) <code>ServiceManager</code> implementation, 
  * together with any 
  * {@link javax.activity.propertygroup.PropertyGroup PropertyGroups} used by the
  * HLS. The <code>PropertyGroupManager</code> understands how to create and
  * manipulate a specific type of <code>PropertyGroup</code>. It is registered with
  * the Activity service and is used by the Activity service to
  * create <code>PropertyGroup</code> instances and to manipulate the
  * {@link javax.activity.PropertyGroupContext PropertyGroupContext}
  * that is implicitly propagated as part of an Activity context.
  * <br> For a particular type of <code>PropertyGroup</code>, there must be a
  * <code>PropertyGroupManager</code> registered in each client and server execution
  * environment for which the PropertyGroup will be accessed. If
  * <code>PropertyGroupContext</code> is propagated, as part of an Activity context,
  * to an environment in which there is
  * no appropriate PropertyGroupManager registered, then the PropertyGroupContext
  * is ignored and is not available within that environment although it will be
  * passed on to any downstream environment to which the Activity context is further
  * distributed.
  * 
  */                                                                          
public interface PropertyGroupManager
{
   /**
    * Creates a new instance of a {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    * specialization. This method is called by the Activity service when
    * a new Activity, with the specified {@link javax.activity.GlobalId GlobalId}, is begun.
    * A <code>null</code> value for the <code>parent</code> indicates that there is no
    * parent for this <code>PropertyGroup</code> instance.
    * <br>It is implementation-dependent whether or not
    * the Activity is active on the thread when this method executes.
    * 
    * @param parent the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    object associated with the most recent ancestor Activity from the same HLS as
    *    the new Activity. For example, if the parent Activity is from the same HLS as
    *    the new Activity then this will be that parent Activity's <code>PropertyGroup</code> reference.
    *    A value of <code>null</code> indicates that the new Activity has no ancestor Activities from the same HLS.
    * @param globalId the {@link javax.activity.GlobalId GlobalId} of the new Activity.
    *
    * @return The reference of the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    specialization created.
    *
    */
   public PropertyGroup create(PropertyGroup parent, GlobalId globalId);

   /**
    * Driven after a failure in order to retrieve any recovered <code>PropertyGroup</code> that
    * was involved in the Activity identified by <code>globalId</code> prior to the failure.
    * A <code>null</code> value for the <code>parent</code> indicates that there is no
    * parent for this <code>PropertyGroup</code> instance.
    * On completion of this method, the recovered <code>PropertyGroup</code> has been recreated and
    * associated with Activity identified by <code>globalId</code>.
    * The HLS that provides the
    * <code>PropertyGroupManager</code> determines the point, during the processing of an Activity, at
    * which it must persist the state of any <code>PropertyGroup</code> and other persistent state. Any failure,
    * between this point and the end of the Activity, requires the HLS to recreate
    * the Activity and, eventually, complete it. Each recreated Activity that
    * uses the target <code>ServiceManager</code> calls this method to retrieve any
    * <code>PropertyGroup</code> recovered by the HLS for the Activity identified by <code>globalId</code>.
    *
    * @param parent the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    object associated with the recovered Activity's parent.
    *    A value of <code>null</code> indicates that the new Activity has no ancestor Activities from the same HLS.
    * @param globalId the <code>GlobalId</code> for the recreated Activity.
    *
    * @return The <code>PropertyGroup</code> recovered by the HLS for the Activity
    *    identified by <code>globalId</code>.
    *    If there is none, then null is returned. 
    *
    * @exception SystemException Thrown if the Activity service encounters
    *    an unexpected error condition.
    *
    */
   public PropertyGroup recoverPropertyGroup(PropertyGroup parent, GlobalId globalId) throws SystemException;

   /**
    * Called by the Activity service when it marshals an Activity context
    * for an outbound request. The <code>PropertyGroupManager</code> is responsible
    * for marshalling its own <code>PropertyGroupContext</code> to be included in that
    * Activity context.
    * <br>The Activity is active on the thread when this method executes.
    * 
    * @param pg the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    object whose contents are
    *    to be encapsulated by a <code>PropertyGroupContext</code>.
    *
    * @return A {@link javax.activity.PropertyGroupContext PropertyGroupContext}
    *    that represents the context
    *    of the <code>PropertyGroup</code> being managed. The Activity service converts
    *    this object to an <code>org.omg.CosActivity.PropertyGroupIdentity</code> and
    *    marshals it for propagation over the wire with the remote request.
    *
    */
   public PropertyGroupContext marshalRequest(PropertyGroup pg);

   /**
    * Called by the Activity service when it marshals an Activity context
    * for an outbound response. The <code>PropertyGroupManager</code> is responsible
    * for marshalling its own <code>PropertyGroupContext</code> to be included in that
    * Activity context.
    * <br>The Activity is active on the thread when this method executes.
    * <br>This method is only called if the <code>MarshalResponseUpdate</code>
    * Property is set in the Properties returned by {@link #getProperties() getProperties}. 
    * 
    * @param pg the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    object whose contents are
    *    to be encapsulated by a <code>PropertyGroupContext</code>.
    *
    * @return A {@link javax.activity.PropertyGroupContext PropertyGroupContext}
    *    that represents the context
    *    of the <code>PropertyGroup</code> being managed. The Activity service converts
    *    this object to an <code>org.omg.CosActivity.PropertyGroupIdentity</code> and
    *    marshals it for propagation over the wire with the remote response.
    *
    */
   public PropertyGroupContext marshalResponse(PropertyGroup pg);

   /**
    * Creates a new instance of a {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    * specialization from an inbound <code>PropertyGroupContext</code>.
    * Called by the Activity service when it unmarshals an Activity context,
    * identified by <code>globalId</code>,
    * on an inbound request. The <code>PropertyGroupManager</code> is responsible
    * for unmarshalling its own <code>PropertyGroupContext</code>. 
    * If this is a nested Activity, then the parent Activity's <code>PropertyGroup</code>
    * is passed as the <code>parent</code>, otherwise <code>parent</code> is <code>null</code>.
    * Any existing <code>PropertyGroup</code> already maintained by the Activity, or <code>null</code>,
    * is passed in the <code>pg</code> parameter, and this is replaced by the
    * new <code>PropertyGroup</code>. 
    * <br>It is implementation-dependent whether or not
    * the Activity is active on the thread when this method executes.
    * 
    * @param ctx the {@link javax.activity.PropertyGroupContext PropertyGroupContext}
    *    received on an inbound request.
    * @param pg the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    currently associated with the Activity,
    *    or <code>null</code> if there is none.
    * @param parent the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    associated with the parent Activity,
    *    or <code>null</code> if the current Activity is a top-level Activity.
    * @param globalId the {@link javax.activity.GlobalId GlobalId} of the new Activity.
    *
    * @return The reference of the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    specialization created.
    *
    */
   public PropertyGroup unmarshalRequest(PropertyGroupContext ctx, PropertyGroup pg,
                                  PropertyGroup parent, GlobalId globalId);

   /**
    * Called by the Activity service when it unmarshals an Activity context
    * on an inbound response. The <code>PropertyGroupManager</code> is responsible
    * for unmarshalling its own <code>PropertyGroupContext</code>.
    * <br>The Activity is active on the thread when this method executes.
    * <br>This method is only called if the <code>unmarshalResponseUpdate</code>
    * property is set in the Properties returned by {@link #getProperties() getProperties}.
    * 
    * @param ctx the {@link javax.activity.PropertyGroupContext PropertyGroupContext}
    *    received on an inbound response.
    * @param pg the {@link javax.activity.propertygroup.PropertyGroup PropertyGroup}
    *    currently associated with the Activity,
    *    or <code>null</code> if there is none.
    */
   public void unmarshalResponse(PropertyGroupContext ctx, PropertyGroup pg);

   /**
    * Returns a <code>Properties</code> object that describes meta-properties
    * for the behaviour of the <code>PropertyGroupManager</code>.
    * <br>The following properties may be specified by a <code>PropertyGroupManager</code>
    * implementation and may be used to influence the behaviour of an Activity
    * service implementation, although these are hints to the Activity service
    * rather than required behaviour.
    * <dl>
    * <dt>"marshalResponseUpdate"
    * <dd>
    * Indicates whether or not the <code>PropertyGroupManager</code>
    * should be called when an outbound response is marshalled.
    * <br>A value of <code>"true"</code> indicates that the context for the
    * managed <code>PropertyGroup</code> should be updated on a response.
    * <br>A value of <code>"false"</code> indicates that the context for the
    * managed <code>PropertyGroup</code> is not updated on a response so the
    * <code>PropertyGroupManager</code> is not called for <code>marshalResponse</code>.
    * <br>The default value is <code>"false"</code>
    * <br>It may be preferable for either a security or a performance
    * point of view not to transmit server context back to a client with
    * a response.
    * <dt>"unmarshalResponseUpdate"
    * <dd>Indicates whether or not the <code>PropertyGroupManager</code>
    * should be called when an inbound response is unmarshalled.
    * <br>A value of <code>"true"</code> indicates that the context for the
    * managed <code>PropertyGroup</code> should be updated by the response.
    * <br>A value of <code>"false"</code> indicates that the context for the
    * managed <code>PropertyGroup</code> is not updated by the response so the
    * <code>PropertyGroupManager</code> is not called for <code>unmarshalResponse</code>.
    * <br>The default value is <code>"false"</code>
    * <br>It may be preferable for either a security or a performance
    * point of view not to allow the local context to be updated by
    * changes made in a downstream node.
    * <dt>"cacheable"
    * <dd>Indicates whether or not the <code>PropertyGroup</code> implementation caches
    * context during the lifetime of the Activity.  Any Activity that
    * uses such a <code>PropertyGroup</code> is required to manage the timeout of such
    * <code>PropertyGroups</code>. Possible values are <code>"true"</code> and <code>"false"</code>.
    * <br>The default value is <code>"false"</code>
    * <dt>"maxSendSize"
    * <dd>Indicates the maximum size of the <code>PropertyGroup</code> context data
    * that should be allowed to be passed on outbound requests
    * and responses. The maximum size applies to the length of the
    * InputStream or OutputStream occupied by the marshalled form of
    * the {@link javax.activity.PropertyGroupContext PropertyGroupContext}.
    * An attempt to send more data than the maximum allowed may result in
    * an org.omg.CORBA.IMP_LIMIT exception being raised during marshalling.
    * Possible values are <code>String</code> representations of an <code>int</code> type.
    * <br>There is no default value for this property, indicating that there is no
    * limit on the size of the data.
    * <dt>"maxRecvSize"
    * <dd>Indicates the maximum size of the <code>PropertyGroup</code> context data
    * that the Activity service should accept and pass on to a
    * <code>PropertyGroupManager</code> on an inbound request or
    * response. The maximum size applies to the length of the
    * InputStream or OutputStream occupied by the marshalled form of
    * the {@link javax.activity.PropertyGroupContext PropertyGroupContext}.
    * Any inbound request or response that contains more data than the
    * maximum allowed may result in
    * an org.omg.CORBA.IMP_LIMIT exception being raised during unmarshalling.
    * Possible values are <code>String</code> representations of an <code>int</code> type.
    * <br>There is no default value for this property, indicating that there is no
    * limit on the size of the data.
    * </dl>
    * <br>Other properties may be defined in the future or may be introduced between
    * consenting <code>PropertyGroup</code> and Activity service implementors.
    * 
    * @return A <code>Properties</code> object that describes meta-properties
    * for the behaviour of the <code>PropertyGroupManager</code>.
    *
    */
   public Properties getProperties();

}
